extern ebmdoc;
/* DOCUMENT  ebm package

   Functions:
     ebm_track_grp_members
     ebm_get_child_by_name
*/

/* Function ebm_track_grp_members */

func ebm_track_grp_members( grp_dol, check=, rec=, flog=, chat= )
/* DOCUMENT ebm_track_grp_members, grp_dol, check=, rec=, flog=, chat=

   Keyword check: >= 1  file existence is checked
                  >= 2  extension existence is checked
                  >= 3  reading of data is checked
           rec  :  recursive mode i.e. go through all children
           flog :  name of log file or stream
                    The log file will be closed unless it is a stream
          NB: for recursive mode 'flog' should be a stream opened and
              closed outside this function

           chat:   Integer, 0 - 5, the higher the more extra output

   2008-12-02/NJW
*/
{
   local grp_filename, extno, dirname, basename;
   local mem_filename, mem_extno;
   output_separator = "::::*******************************:::::::::";

   if( is_void(check) ) check = 0;
   if( is_void(chat) ) chat = 0;
   if( is_void(flog) ) {
      lg = 0;
   } else {
      if( typeof(flog) == "string" ) {
         flog = open( flog, "w" );
         keep_open = 0; // close when done
      } else if( typeof(flog) == "text_stream" ) {
         keep_open = 1;
      } else {
         write,"flog keyword of unrecognized kind";
         return;
      }
      lg = 1;
   }

   get_exten_no, grp_dol, grp_filename, extno;
   if( !file_test(grp_filename) ) {
      write,format="File not found: %s\n", grp_filename;
      if(lg){
         write,flog,format="File not found: %s\n", grp_filename;
         if(!keep_open)close, flog;
      }
      return;
   }
   splitfname, grp_filename, dirname, basename;
   dirname = app_slash(dirname);

   write,format="Entering group: %s\n", grp_dol;
   if(lg)write,flog,format="Entering group: %s\n", grp_dol;

   grp_hdr = headfits( grp_dol, nocheck=1 );
   extname = fxpar( grp_hdr, "extname" );
   if( extname != "GROUPING" ) error, "ebm_track_grp_members called for non-group.";
   grp_extver = fxpar( grp_hdr, "extver" );
   if( numberof(grp_extver) == 0 ) {
      write,format="No EXTVER keyword, reset to %i\n", 1;
      if(lg)write,flog,format="No EXTVER keyword, reset to %i\n", 1;
      grp_extver = 1;
   } else {
      write,format="   EXTVER keyword for the group is %i\n", grp_extver;
      if(lg)write,flog,format="   EXTVER keyword for the group is %i\n", grp_extver;
   }

   nrows = fxpar( grp_hdr, "naxis2");
   if( numberof(nrows) == 0 ) {
      write,"NAXIS2 keyword is missing in GROUPING structure";
      if(lg)write,flog,"NAXIS2 keyword is missing in GROUPING structure";
      if( rec ) write,output_separator;
      if( rec && lg ) write,flog,output_separator;
      if( lg && !keep_open ) close, flog;
      return;
   }
   if( nrows == 0 ) {
      write,"No rows in the GROUPING structure";
      if(lg)write,flog,"No rows in the GROUPING structure";
      if( rec ) write,output_separator;
      if( rec && lg ) write,flog,output_separator;
      if( lg && !keep_open ) close, flog;
      return;
   } else {
      write,format=" with %i members\n", nrows;
      if(lg)write,flog,format=" with %i members\n", nrows;
   }

   mem_xtension = strtrim(rdfitscol( grp_dol, "member_xtension" ));
   mem_name     = strtrim(rdfitscol( grp_dol, "member_name" ));
   mem_version  = rdfitscol( grp_dol, "member_version" );
   mem_position = rdfitscol( grp_dol, "member_position" );
   mem_location = strtrim(rdfitscol( grp_dol, "member_location" ));
//+   uri_type = strtrim(rdfitscol( grp_dol, "member_uri_type" ));
   n_members = numberof(mem_xtension);

   if( rec ) write,output_separator;
   if( rec && lg ) write,flog,output_separator;
   for( i = 1; i <= n_members; i++ ) {
      if( strlen(mem_location(i)) == 0 ) {
         //+ mem_location(i) = grp_filename;
         mem_location(i) = basename; // of 'grp_filename'
         in_another_file = 0;
      } else in_another_file = 1;
      write,format="#%i %s (%s) %s[pos%i] extver %i   ", i, mem_name(i), \
         mem_xtension(i), mem_location(i), mem_position(i), mem_version(i);
      if(lg)write,flog,format="#%i %s (%s) %s[pos%i] extver %i   ", i, mem_name(i), \
         mem_xtension(i), mem_location(i), mem_position(i), mem_version(i);
      if( check > 0 ) {
         ftest = file_test(dirname+mem_location(i),gz=1);
         if( ftest ) {
            if( in_another_file ) {
               write,format="%s","f";
               if(lg)write,flog,format="%s","f";
            }
            if( check >= 2 ) { // check for extension existence
               if( ftest == 2 ) { // we have gzipped file
                  proxyfile = gz_proxy_file(dirname+mem_location(i),silent=1);
                  write,format="%s","p";
                  if(lg)write,flog,format="%s","p";
               } else { // its own proxy
                  proxyfile = dirname+mem_location(i);
               }
               mem_dol = swrite(format="%s[%s,%i]", proxyfile, mem_name(i), mem_version(i));
               if( chat > 1 ) {
                  write,format="\nDOL(1) %s mem_position: %i\n", mem_dol, mem_position(i);
                  if(lg)write,flog,format="\nDOL(1) %s mem_position: %i\n", mem_dol, mem_position(i);
               }
               get_exten_no, mem_dol, mem_filename, mem_extno;
               if( mem_extno >= 0 ) {
                  mem_dol = swrite(format="%s[%i]", proxyfile, mem_extno);
                  fh = headfits( mem_dol );
                  extver = fxpar(fh,"extver");
                  if( chat > 1 ) {
                     write,format="DOL(2) %s extver: %i\n", mem_dol, extver;
                     if(lg)write,flog,format="DOL(2) %s extver: %i\n", mem_dol, extver;
                  }
               }
               fh = fits_open( proxyfile, "r" );
               if( !fits_cards_test( fits_goto_hdu(fh,mem_extno+1) ) ) {
                  fits_close, fh;
                  write,format="%s", "!e";
                  if(lg)write,flog,format="%s", "!e";
               } else {
                  fits_close, fh;
                  write,format="%s", "e";
                  if(lg)write,flog,format="%s", "e";
                  if( check >= 3 ) { // Check the data reading
                     if( mem_xtension(i) == "BINTABLE" ) {
                        ptr = rdfitsbin( mem_dol );
                        write,format="%s","r";
                        if(lg)write,flog,format="%s","r";
                     } else if( mem_xtension(i) == "IMAGE" ) {
                        im = readfits( mem_dol );
                        write,format="%s","r";
                        if(lg)write,flog,format="%s","r";
                     } else {
                        write,format="%s", "?r";
                        if(lg)write,flog,format="%s", "?r";
                     }
                  }
               }
            }
         } else {
            write,format="%s","!f";
            if(lg)write,flog,format="%s","!f";
         }
         write,"";
         if(lg)write,flog,"";
      }
      if(!check){
         write,"";
         if(lg)write,flog,"";
      }
      if( rec && mem_name(i) == "GROUPING" ) \
          ebm_track_grp_members, dirname+mem_location(i)+ \
         swrite(format="[GROUPING,%i]",mem_version(i)), \
              check=check,rec=1,flog=flog,chat=chat;
   }
   if( rec ) write,output_separator;
   if( rec && lg ) write,flog,output_separator;
   if( lg && !keep_open ) close, flog;
}

/* Function ebm_get_child_by_name */

func ebm_get_child_by_name( file_extno, child_name, \
                   &report, bygrp=, chat=)
/* DOCUMENT list = ebm_get_child_by_name( file_extno, child_name, \
                   >report, bygrp=, chat=)

    Returns a list of DOLs where the EXTNAME matches the given child_name

    Call:   file_extno   File name with extension number appended,
                           e.g.  myfile.fits[6]
                           Counting starts with 0 (zero) so that
                           the primary HDU has extno 0
            child_name   Name of child (EXTNAME) in grouping table

   Return:  report    a string array with elements:
                           1) grpname
                           2) nchild
                           3) n_missing
                           4) errmsg

   Keywords: bygrp     GRPNAME is used instead of EXTNAME for the search

*/
{

   local file, extno;

   // Set returned value in case of malfunction:

   report = array(string,4);
   outfile_extno_res = [];
   grpname = report(1) = "";
   nchild = -1;
   report(2) = itoa(-1);
   n_missing = 0;
   report(3) = itoa(0);
   errmsg = report(4) = "";

   // debug
   lun = open("output","w");
   write,lun,file_extno;
   write,lun,"## child_name:  ", child_name;

   // Separate filename and extension number

   name_init = strpart(file_extno,1:1)  ==  "/" ? "/" : "";
   get_exten_no, file_extno, file, extno;
   tok = strsplit(file,"/");
   n_tok = numberof(tok);

   // Find out if the file exists

   if( !file_test(file) ) {
      write,file," does not exist";
      errmsg = report(4) = "Input file not found";
      return;
   }

   // Make sure that the input is a grouping table

   hdr = headfits(file_extno);

   extname = fxpar(hdr,"EXTNAME");
   grpname = fxpar(hdr,"GRPNAME");
   if( typeof(grpname) == "string" ) report(1) = grpname;

   if( extname  !=  "GROUPING" ) {
      errmsg = report(4) = "Not a grouping table";
      write,"EBM_GET_CHILD_BY_NAME error: Not a grouping table";
      return;
   }

   mempos = rdfitscol(file_extno, 4, silent=1);
   memloc = rdfitscol(file_extno, 5, silent=1);

   n_children = numberof(mempos);
   nchild = n_children;

   //debug
   write,lun,"n_children = ", n_children;

   if( n_children  ==  0 ) {
      errmsg = report(4) = "No children in grouping table";
      write,"EBM_GET_CHILD_BY_NAME error: No children in grouping table";
      return;
   }


   // Search for the requested child

   nhits = 0;
   for( index = 1; index <= n_children; index++ ) {

      outextno = mempos(index) - 1;

      //debug
      write,lun,"index=",index,"  ",outextno,"  ",memloc(index);

      if( strlen(memloc(index))  ==  0 ) { // in same file
         outfile = file;
      } else { // filename given
         tok(n_tok) = strtrim(memloc(index));
         outfile = name_init+strjoin(tok,"/");
      }
      outfile_extno = outfile+"["+itoa(outextno)+"]";

      //debug
      write,lun,"outfile_extno: ", outfile_extno;

      // Make sure that the requested file exists. If not skip
      // write warning and continue

      //+ list = file_search(outfile, );

      if( file_test(outfile) ) { //   The file does exist!

         // Get the header

         hdr = headfits( outfile+"+"+itoa(outextno) );

         if( bygrp ) {
            compname = fxpar(hdr,"GRPNAME");
            if( typeof(compname) != "string" ) compname = "!!!!!!!";
          } else {
            compname = fxpar(hdr,"EXTNAME");
          }
         //debug
         write,lun,"compname: ", compname;

         if( compname  ==  child_name ) {
            nhits++;
            //debug
            write,lun,"Agree! hit #", nhits;

            errmsg = report(4) = "";
            if( nhits  ==  1 ) {
               outfile_extno_res = outfile_extno;
            } else {
               grow, outfile_extno_res, outfile_extno;
            }
         }

      } else {

         n_missing++;
         if( chat ) {
            write,"Warning: file in group: ", outfile;
            write,"does not exist. Skip and continue ...";
         }

      }

   }

   if( nhits  ==  0 ) {
      errmsg = report(4) = "Child does not exist in group";
      write,"EBM_GET_CHILD_BY_NAME error: Requested child was";
      write,"         not found in grouping table";
   }

   // Report on number of missing elements

   if( n_missing  >  0 ) {
      write,"Warning: "+itoa(n_missing)+" files are missing";
   }

   //debug
   close,lun;

   return;

}
