/* Function imagedoc */

extern imagedoc;
/* DOCUMENT ****************************************************
 *
 *      Package by Niels J. Westergaard 2003-03-30 for
 *      image analysis.

    _colscale           image2events        peaksearch0         
    _pal_write          imcolorbar          peaksearch1         
    add_peak            imcrop              peaksearch2         
    convol2d            imcut               rebin               
    cur_extract_box     imhcut              remap               
    curmark             makeimagewcs        remap_wcs           
    curmark1            makeimageu          repair2gezero       
    disp                makeimagez          signif              
    disp9               manip_image_get     src_cts             
    dispc               manip_image_put     unwiden_image       
    dispcc              maxim               w0                  
    distances           mk_palette          w1                  
    edge_suppress       pal                 w2                  
    flatfield           pal_edit            w3                  
    getimval            pal_plot            w4                  
    getimvals           palastro            widen_image         
    gkernel             contrast            rotate_image
    distances_ellipse   squarify            flex_image
    distribute_bkg

*****************************************************/

/* Function mk_palette */

func mk_palette( name )
/* DOCUMENT mk_palette, name

   The RGB are defined individually by mouse clicking in coordinate system.
   The argument 'name' is without the '.gp' extension.

   2008-10-10/NJW (or about that time)

*/
{
   if( typeof(name) != "string" ) error,"A name must be given";
   window,0;
   ncolors = 240;
   // define red
   plot,[1,ncolors],[0,0],yr=[0,256];
   oplot,[1,ncolors],[255,255];
   oplot,[1,1],[0,255];
   oplot,[ncolors,ncolors],[0,255];
   write,"Define the RED color function :";
   r = _colscale( ncolors );
   oplot, r, color="red";
   write,"Define the GREEN color function :";
   g = _colscale( ncolors );
   oplot, g, color="green";
   write,"Define the BLUE color function :";
   b = _colscale( ncolors );
   oplot, b, color="blue";

   _pal_write, name, r, g, b;

   write,format="Palette now stored in %s.gp\n", name;

}

/* Function _pal_write */

func _pal_write( name, r, g, b )
{
   ncolors = numberof(r);
   stream = open(GISTDIR+name+".gp","w");
   write,stream,format="# Palette created by mk_palette %s/NJW\n", ndate(3);
   write,stream,format="ncolors= %i\n", ncolors;
   write,stream,format="#    r   g    %s\n","b";
   for( i = 1; i <= ncolors; i++ ) {
      write,stream,format="%8i%8i%8i\n", r(i), g(i), b(i);
   }
   close, stream;

}

/* Function _colscale */

func _colscale( ncolors )
{
   rc = curmark();
   n = numberof(rc)/2;
   rcx = rc(1:0:2);
   rcy = rc(2:0:2);
   is = sort(rcx);
   rcx = rcx(is);
   rcy = rcy(is);
   rcx(1) = 1.;
   if( rcx(2) < rcx(1) ) rcx(2) = 2.;
   rcx(0) = ncolors;
   if( rcx(-1) > rcx(0) ) rcx(-1) = ncolors-1;
   r = long(interp( rcy, rcx, indgen(ncolors) ) );
   w = where( r < 0 );
   if( numberof(w) ) r(w) = 0;
   w = where( r > 255 );
   if( numberof(w) ) r(w) = 255;

   return r;

}

/* Function pal_edit */

func pal_edit( name, step= )
{
   if( is_void(step) ) step = 10;

   p = GISTDIR+name+".gp";
   pal_plot, name;
   r = rscol(p,1,lng=1,silent=1,nomem=1);
   g = rscol(p,2,lng=1,silent=1,nomem=1);
   b = rscol(p,3,lng=1,silent=1,nomem=1);
   ncolors = numberof(r);
   rgb = array(long,ncolors,3);
   rgb(,1) = r;
   rgb(,2) = g;
   rgb(,3) = b;

   ans = "";
   answ = ["r","g","b"];
   cstr = ["red","green","blue"];
   read,prompt="Enter color (r,g, or b) : ... ", ans;
   w = where( ans == answ );
   if( !numberof(w) ) error,"Illegal input";
   nc = w(1);
   again = 1;
   h = 1;
   grow,h,indgen(step:ncolors:step);

   //while( again ) {
      oplot,h,rgb(h,nc),ps=2,symsize=2,color=cstr(nc);
      for( i = 1; i <= numberof(h); i++ ) {
         oplot,[h(i),h(i)],[0,255],color=cstr(nc);
      }
      pts = curmark();
      npts = numberof(pts);
      xpts = pts(1:npts:2);
      ypts = pts(2:npts:2);
      npts /= 2;
      y0 = rgb(h,nc);
      for( i = 1; i <= npts; i++ ) {
         w = where( min(abs(xpts(i)-h)) == abs(xpts(i)-h) )(1);
         y0(w) = long(ypts(i));
      }
      rgb(,nc) = long(interp( y0, h, indgen(ncolors) ));
      w = where( rgb(,nc) < 0 ); if(numberof(w)) rgb(w,nc) = 0;
      w = where( rgb(,nc) > 255 ); if(numberof(w)) rgb(w,nc) = 255;
      oplot, rgb(,nc), color=cstr(nc);
      _pal_write, name, rgb(,1), rgb(,2), rgb(,3);
      pal_plot, name;
      again = 0;
   //}
}


/* Function pal_plot */

func pal_plot( name )
{
   pal, name;
   p = GISTDIR+name+".gp";
   r = rscol(p,1,lng=1,silent=1,nomem=1);
   g = rscol(p,2,lng=1,silent=1,nomem=1);
   b = rscol(p,3,lng=1,silent=1,nomem=1);
   ncolors = numberof(r);
   a = array(double,ncolors,256);
   for(i=1;i<=ncolors;i++) a(i,) = i-1;
   disp,a,reset=1,title=name+".gp";

   oplot,r,color="red";
   oplot,g,color="green";
   oplot,b,color="blue";

}

/* Function flatfield */

func flatfield( image, sigma, sel= )
/* DOCUMENT new_image = flatfield( image, sigma, sel= )

   The image 'image' is folded with a gaussian kernel
   with sigma = 'sigma' and then subtracted from the
   original image. The process is repeated with
   excluded source areas.
   Keyword 'sel' must be an array of same dimension
   as 'image' with 1 where pixels are useful and zero
   elsewhere.

   2007-06-19/NJW

*/
{
   require, "fconvol.i";

   di = dimsof(image);
   if( di(1) != 2 ) {
      write,"Not a 2D image";
      return [];
   }
   dosel = 0;
   if( !is_void(sel) ) {
      d = dimsof(sel);
      if( d(1) != 2 ) {
         write,"'sel' is not a 2D image";
         return [];
      }
      if( d(2) != di(2) || d(3) != di(3) ) {
         write,"'sel' is not commensurate with 'image'";
         return [];
      }
      dosel = 1;
   }
   w = dosel ? where( sel ) : indgen(di(2)*di(3));
   //   Avoid changing the input image
   im1 = image;
   fimage = gfconvol( image, sigma );
   //    First subtraction
   diff  = image - fimage;
   //    Substitute image parts that exceed 3sigma
   //    to avoid subtracting too much around sources
   rms_1 = wrms( diff(w) );
   q = where( abs(diff) > 3*rms_1 );
   if( numberof(q) > 0 ) {
      im1(q) = fimage(q);
      fimage = gfconvol( im1, sigma );
      diff = image - fimage;
   }

   return diff;
}


/* Function src_cts */

func src_cts( image )
/* DOCUMENT number_of_counts = src_cts( image )

   Interactive function to get the number of source
   counts under the assumption that 'image' is a 2D
   array displayed in an active plot window where the
   unit of the axes is in pixels.
   'curmark1' is called twice; first to define the
   source area and then to define the background area.

   The source area must either be completely included
   in the background area or the two areas should be
   disjunct.

   2007-02-06/NJW
*/
{

   p_src = curmark1( style=1, prompt="Mark source ..." );
   oplot,[p_src(1),p_src(3),p_src(3),p_src(1),p_src(1)], \
         [p_src(2),p_src(2),p_src(4),p_src(4),p_src(2)], color="blue";
   p_bkg = curmark1( style=1, prompt="Mark background ..." );
   oplot,[p_bkg(1),p_bkg(3),p_bkg(3),p_bkg(1),p_bkg(1)], \
         [p_bkg(2),p_bkg(2),p_bkg(4),p_bkg(4),p_bkg(2)], color="green";
   if( p_src(1) > p_src(3) ) { // swap
      x = p_src(3); p_src(3) = p_src(1); p_src(1) = x;
   }
   if( p_src(2) > p_src(4) ) { // swap
      x = p_src(4); p_src(4) = p_src(2); p_src(2) = x;
   }
   if( p_bkg(1) > p_bkg(3) ) { // swap
      x = p_bkg(3); p_bkg(3) = p_bkg(1); p_bkg(1) = x;
   }
   if( p_bkg(2) > p_bkg(4) ) { // swap
      x = p_bkg(4); p_bkg(4) = p_bkg(2); p_bkg(2) = x;
   }
   dms = dimsof( image );
   x = span(1,dms(2),dms(2))(,-:1:dms(3));
   y = span(1,dms(3),dms(3))(-:1:dms(2),);
   i_src = x > p_src(1) & x < p_src(3) & y > p_src(2) & y < p_src(4);
   w_src = where( i_src );
   nw_src = numberof( w_src );
   i_bkg = x > p_bkg(1) & x < p_bkg(3) & y > p_bkg(2) & y < p_bkg(4);
   w_bkg = where ( i_bkg & (!i_src) );
   nw_bkg = numberof( w_bkg );

   if( nw_src == 0 ) {
      write,"No pixels in source area!";
      return [];
   }
   if( nw_bkg == 0 ) {
      write,"No pixels in background area!";
      return [];
   }
   bkg_level = avg( image(w_bkg) );
   write,format="Number of src pixels: %i\n", nw_src;
   write,format="Number of bkg pixels: %i\n", nw_bkg;
   write,format="Bkg level: %f\n", bkg_level;
   write,format="Cts in source area: %f\n", double(sum(image(w_src)));
   write,format="Cts in backgr area: %f\n", double(sum(image(w_bkg)));
   write,format="Backgr cts in source area: %f\n", bkg_level * nw_src;
   src_counts = sum(image(w_src)) - bkg_level * nw_src;
   write,format="Net src counts: %f\n", src_counts;
   write,format="S/N: %f\n", src_counts / sqrt( src_counts + bkg_level * nw_src );
   write,format="N-sigma detection: %f\n", src_counts / sqrt(1. + bkg_level * nw_src );
   write,"       N-sigma uses formula: src_counts / sqrt(1. + bkg_counts)";

   return src_counts;

}

/* Function palastro */

func palastro(num)
/* DOCUMENT palastro[, num]

   defines the astro-num palette for plot window

    2007-02-06/NJW
*/
{
   require, "idlx.i";

   if( is_void(num) ) num = 1;
   snum = itoa(num);
   palette,GISTDIR+"astro"+snum+".gp";
}

/* Function pal */

func pal(name)
/* DOCUMENT pal, name

   defines the named palette for the current plot window.

   Argument
       name    stem of palette file (i.e. without the suffix: '.gs')
               If not given a list of possibilities is shown.

    2007-03-05/NJW, 2010-04-20/NJW, 2015-06-16/NJW update
*/
{
   require, "idlx.i";
   local dirname, basname;

   if( is_void(name) ) {
      write,format="GISTDIR: %s\n", GISTDIR;
      list = file_search("*.gp",GISTDIR);
      for( i = 1; i <= numberof(list); i++ ) {
         splitfname, list(i), dirname, basname;
         write,format="%20s\n",basname;
      }
      return;
   }
   palette,GISTDIR+"/"+name+".gp";
}

func w0(a) { window,0; }
func w1(a) { window,1; }
func w2(a) { window,2; }
func w3(a) { window,3; }
func w4(a) { window,4; }

/* Function 'distances' */

func distances( arg1, arg2, arg3, arg4, arg5, arg6 )
/* DOCUMENT array = distances( dim1, cen1 )
                     1 D version
            array = distances( dim1, dim2, cen1, cen2 )
                     2 D version
            array = distances( dim1, dim2, dim3, cen1, cen2, cen3 )
                     3 D version
      returns an array with dimensions dim1, dim2, ... with
      distances in units of pixel
      from the position (cen1, cen2, ...)

   To get a symmetrical image: cen = double(dim + 1)/2.

*/

{
   if( is_void(arg3) ) { // 1D version
      dim1 = arg1; cen1 = arg2;
      return abs(indgen(dim1) - cen1);
   }
   if( is_void(arg5) ) { // 2D version
      dim1 = arg1; dim2 = arg2;
      cen1 = arg3; cen2 = arg4;
      x = indgen(dim1)(,-:1:dim2);
      y = indgen(dim2)(-:1:dim1,);
      return sqrt((x-cen1)^2 + (y-cen2)^2);
   }
   // 3D version
   dim1 = arg1; dim2 = arg2; dim3 = arg3;
   cen1 = arg4; cen2 = arg5; cen3 = arg6;
   x = indgen(dim1)(,-:1:dim2,-:1:dim3);
   y = indgen(dim2)(-:1:dim1,,-:1:dim3);
   z = indgen(dim3)(-:1:dim1,-:1:dim2,);
   return sqrt((x-cen1)^2 + (y-cen2)^2 + (z-cen3)^2);
}

/* Function distances_ellipse */

func distances_ellipse( dim1, dim2, cen1, cen2, angle, excen )
/* DOCUMENT im = distances_ellipse( dim1, dim2, cen1, cen2, angle, excen )

      Returns image with 'elliptical distances' (isocurves outline an ellipse)

     Arguments:
        dim1, dim2      (integers) image dimensions
        cen1, cen2      center of ellipse
            angle       angle in degrees
            excen       excentricity  [0, 1[
*/
{
   a = angle*pi/180;
   can = cos(a); san = sin(a);
   r = 1 - excen^2;

   x = indgen(dim1)(,-:1:dim2) - cen1;
   y = indgen(dim2)(-:1:dim1,) - cen2;
   xp = x * can - y * san;
   yp = x * san + y * can;
   return sqrt(r*xp^2 + yp^2);
}

/* Function disp */

func disp( im, pane=, reset=, xax=, yax=, title=, xtitle=, \
           ytitle=, lcut=, ucut=, Lcut=, Ucut=, pal=, uplf=, cb=, lcb= )
/* DOCUMENT disp, im, pane=, reset=, xax=, yax=, title=, xtitle=, \
                      ytitle=, lcut=, ucut=, Lcut=, Ucut=, pal=, uplf=, cb=, lcb=

      Display an image on the screen

              pane : Plot window number (0 thru 7), default is current
     Keywords lcut : Lower cut
              ucut : Upper cut, values in accumulated distribution
                Both defined as fractions compared to the interval
                min(im) - max(im)
              Ucut : Upper cut in image values
              Lcut : Lower cut in image values
              pal  : Name of palette to use (.gp name extension is added if missing)
              cb   : Add a colorbar with this number of ticbars
              lcb  : Add a log colorbar with this number of ticbars

     2007-02-06/NJW
     2009-11-02/NJW Using 'pli' iso. 'plf'
*/

{
   local x, h;

   if( !is_void(pane) ) window, pane;
   if( !is_void(pal) ) {
      if( strpart(pal,-2:0) != ".gp" ) pal += ".gp";
      palette, pal;
   }

   dms = dimsof(im);
   if( dms(1) < 2 || dms(1) > 3 ) {
      write,"Bad image dimensions.";
      return;
   }
   flag_colorimage = 0;
   if( dms(1) == 3 ) { // this may be a color-image
      if( structof(im) == char ) {
         flag_colorimage = 1;
      } else {
         write,"3 image planes but not of type 'char'";
         return;
      } 
   } 

  /*
   *   Apply cuts if defined
   */

   im_work = im;
   if( !flag_colorimage ) {
      if( !is_void(lcut) || !is_void(ucut) ) {
         w = where( im_work != 0 );
         if( numberof(w) > 0 ) {
            binsize = (max(im_work) - min(im_work)) / 1000.0;
            histos, im_work(w), h, x, binsize=binsize;
            h = arr_accum(h, norm=1);
   
            if( !is_void(lcut) ) {
               index1 = long(interp(indgen(numberof(h)),h,lcut));
               val1 = x(index1);
               w1 = where( im_work < val1 );
               if( numberof(w1) > 0 ) im_work(w1) = val1;
            }
   
            if( !is_void(ucut) ) {
               index2 = long(interp(indgen(numberof(h)),h,ucut));
               val2 = x(index2);
               w2 = where( im_work > val2 );
               if( numberof(w2) > 0 ) im_work(w2) = val2;
            }
         }
      }
      if( !is_void(Ucut) ) {
         w = where( im_work > Ucut );
         if( numberof(w) ) im_work(w) = Ucut;
         else im_work(0) = Ucut;// statement inserted 2011-02-08/NJW
      }
      if( !is_void(Lcut) ) {
         w = where( im_work < Lcut );
         if( numberof(w) ) im_work(w) = Lcut;
         else im_work(1) = Lcut;// statement inserted 2011-02-08/NJW
      }
   }
   sz = flag_colorimage ? dimsof(im(1,,)) : dimsof(im_work);
   if( sz(1) == 2 ) {
      nxax = numberof(xax);
      nyax = numberof(yax);
      if( nxax > 0 ) {
         if( nxax != sz(2) ) {
            write,"Bad dimension of xax keyword";
            return;
         }
         f = double(sz(2))/(sz(2)-1);
         c = 0.5*(xax(0)+xax(1));
         d = 0.5*(xax(0)-xax(1));
         xv = span(c-f*d, c+f*d,sz(2)+1);
      } else {
         xv = span(0.5,sz(2)+0.5,sz(2)+1);
      }
      if( nyax > 0 ) {
         if( nyax != sz(3) ) {
            write,"Bad dimension of yax keyword";
            return;
         }
         f = double(sz(3))/(sz(3)-1);
         c = 0.5*(yax(0)+yax(1));
         d = 0.5*(yax(0)-yax(1));
         yv = span(c-f*d, c+f*d,sz(3)+1);
      } else {
         yv = span(0.5,sz(3)+0.5,sz(3)+1);
      }
      x = xv(,-:1:sz(3)+1);
      y = yv(-:1:sz(2)+1,);
      fma;
      limits; // this statement added 2009-03-13/NJW
      if( uplf ) {
         plf,im_work,y,x,legend="";
      } else {
         pli,im_work,xv(1),yv(1),xv(0),yv(0),legend="";
      }
      if( reset ) limits,"e","e","e","e";
      if( !is_void(title) ) pltitle, title;
      if( is_void(xtitle) ) xtitle="";
      if( is_void(ytitle) ) ytitle="";
      xytitles, xtitle, ytitle;
      plotcomments,init=1; // this statement added 2009-03-13/NJW
      if( cb ) colorbar,sigdig(min(im_work),3),sigdig(max(im_work),3),levs=cb;
      if( lcb ) colorbar,sigdig(min(im_work),3),sigdig(max(im_work),3),levs=lcb,lg=1;
      // Added statements to avoid limit change when later overplotting is done:
      lim_old = limits();
      lim_old(5) = 0;
      limits, lim_old;
   } else {
      write,"Not a 2D array";
   }
}


/* Function dispc */

func dispc( im, pane=, xax=, yax=, levels=, itype=, color=, thick=, \
            over=, colorversion=, title=, xtitle=, ytitle= )
/* DOCUMENT dispc, im, pane=, xax=, yax=, levels=, itype=, color=, thick=, over=,
                     colorversion=, title=, xtitle=, ytitle=
      Display a contour plot on the plot window
      Keywords:
          pane: plot window number
          itype: axis transforms 0: both linear, 1: y log, x lin,  2: y lin, xlog, 3: both log
          xax: x-axis coordinate values
          yax: y-axis coordinate values
          levels: array of level values or, if a single value is given, the number of
                  linearly spaced levels
          color: color of contour curves, standard meaning
          over: causes overplot
          colorversion: colors between contours
*/

{
   if( !is_void(pane) ) window, pane;

   sz = dimsof(im);
   if( sz(1) != 2 ) {
      write,"Not a 2D array";
      return;
   }
   if( is_void(levels) ) {
      levels=span(min(im),max(im),6)(zcen);
   } else if( numberof(levels) == 1 ) {
         levels=span(min(im),max(im),long(levels)+1)(zcen);
   }
   nxax = numberof(xax);
   nyax = numberof(yax);
   if( nxax > 0 ) {
      if( nxax != sz(2) ) {
         write,"Bad dimension of xax keyword";
         return;
      }
      xv = xax;
   } else {
      xv = indgen(sz(2));
   }
   if( nyax > 0 ) {
      if( nyax != sz(3) ) {
         write,"Bad dimension of yax keyword";
         return;
      }
      yv = yax;
   } else {
      yv = indgen(sz(3));
   }
   x = xv(,-:1:sz(3));
   y = yv(-:1:sz(2),);
   if( !over ) {
      fma;
      if( !is_void(itype) ) mlogxy, itype > 1, itype%2;
      limits, xv(1), xv(0), yv(1), yv(0);
   }
   if( colorversion) {
      plfc,im,y,x,levs=levels;
      if(over) plc,im,y,x,levs=levels,marks=0,color=color,width=thick,legend="";
      if( !is_void(title) ) pltitle, title;
      if( is_void(xtitle) ) xtitle="";
      if( is_void(ytitle) ) ytitle="";
      xytitles, xtitle, ytitle;
   } else {
      plc,im,y,x,levs=levels,marks=0,color=color,width=thick,legend="";
      if( !is_void(title) ) pltitle, title;
      if( is_void(xtitle) ) xtitle="";
      if( is_void(ytitle) ) ytitle="";
      xytitles, xtitle, ytitle;
   }
   plotcomments,init=1; // this statement added 2009-11-04/NJW
}

func dispcc( im, xax, yax, levels=, itype= )
/* FUNCTION dispcc, im, xax, yax, levels=, itype=

     Fills contours of 'im' based on 'levels'.
     If 'levels' is omitted then 9 contours will be shown
     with equidistant values.
     If 'xax' or 'yax' are omitted then the image dimensions
     will be used.
*/
{
   dms = dimsof(im);
   if( !is_void(xax) ) {
      if( numberof(xax) != dms(2) ) error,"Bad size of xax";
   } else { // define xax
      xax = span(1,dms(2),dms(2));
   }
   if( !is_void(yax) ) {
      if( numberof(yax) != dms(3) ) error,"Bad size of yax";
   } else { // define yax
      yax = span(1,dms(3),dms(3));
   }
   // setup the plotting range and axis transforms:
   plot,xax,yax,itype=itype;
   if( is_void(levels) ) levels = span(min(im),max(im),11)(2:-1);
   x = xax(,-:1:dms(3));
   y = yax(-:1:dms(2),);
   //+ limits, 1, dms(2), 1, dms(3);
   plfc,im,y,x,levs=levels;
}

/* Function add_peak */

func add_peak( im, xp, yp, sigmax, sigmay, peak=, volume= )
/* DOCUMENT add_peak( im, xp, yp, sigmax[, sigmay], peak=, volume= )

   Returns the image with a gaussian peak added.
   Coordinate (i,j) refers to center of pixel (i,j).
   If 'sigmay' is not given it is assumed equal to 'sigmax'.
   Keyword   peak   :  Defining peak value
             volume :  Volume of peak (defaults to 1)
                       If given it will override the 'peak' value

   2006-02-23/NJW (Updated from earlier version ca. 2004)
   2010-04-06/NJW Updated to use proper integral over pixels
                  and keywords
*/

{

   if( !is_void(volume) ) {
      peak = [];
   } else {
      volume = 1.;
   }
   if( is_void(sigmay) ) sigmay = sigmax;
   sx = abs(sigmax) * sqrt(2);
   sy = abs(sigmay) * sqrt(2);
   dms = dimsof(im);
   if( dms(1) == 2 ) {
      ii = indgen(dms(2));
      arg1 = (ii + 0.5 - xp)/sx;
      arg2 = (ii - 0.5 - xp)/sx;
      tx = erf(arg1) - erf(arg2);
      jj = indgen(dms(3));
      arg1 = (jj + 0.5 - yp)/sy;
      arg2 = (jj - 0.5 - yp)/sy;
      ty = erf(arg1) - erf(arg2);
      a = volume * 0.25 * tx * ty(-,);
      if( !is_void(peak) ) a *= (peak/max(a));
      return(im + a);
   } else {
      write,"Not a 2D array";
      return(-1.0);
   }
}

/* Function convol2d  */

func convol2d( a, b )
/* DOCUMENT res = convol2d(a,kernel)

   returns the convolution of 'a' and 'kernel', both 2D arrays.
*/

{

   sza = dimsof(a);
   szb = dimsof(b);
   if( (sza(1) != 2) || (szb(1) != 2) ) {
      print,"convol2d must be called with 2d arguments";
      return;
   }
   if( szb(2) % 2 != 1 || szb(3) % 2 != 1 ) {
      print,"Kernel must have odd dimensions";
      return;
   }
   ad = double(a);
   bd = double(b);
   dx = szb(2) / 2;
   dy = szb(3) / 2;

   res = ad;

   for( i = 1; i <= sza(2); i++ ) {
   for( j = 1; j <= sza(3); j++ ) {
      weight = 0.0;
      ssum = 0.0;
      for( k = -dx; k <= dx; k++ ) {
         if( k+i >= 1 && k+i <= sza(2) ) { // in large array in x
	 for( l = -dy; l <= dy; l++ ) {
	    if( l+j >= 1 && l+j <= sza(3) ) { // in large array in y
	       ssum += ad(i+k,j+l) * bd(k+dx+1,l+dy+1);
	       weight += bd(k+dx+1,l+dy+1);
	    }
	 }
	 }
      }
      res(i,j) = ssum / weight;
   }
   }
   return( res );
}

/* Function gkernel */

func gkernel( dim1, dim2, sigma)
/* DOCUMENT kernel = gkernel(dim1, dim2, sigma)

   Returns 2D kernel with 2d Gauss function
*/

{

   dim1 = (dim1/2)*2 + 1;
   dim2 = (dim2/2)*2 + 1;
   x = distances(dim1,dim2,dim1/2+1,dim2/2+1);
   kernel = exp(-0.5*(x/sigma)^2);
   return( kernel/sum(kernel) );
}

/* Function edge_suppress */

func edge_suppress( dim1, dim2, radius, lambda )
/* DOCUMENT suppress = edge_suppress( dim1, dim2, radius, lambda )

   Returns a double array dim1 x dim2 with values close to 1 when
   r < radius and approaching zero when r > radius. lambda determines
   the scale, the smaller the sharper the shift.

   2007-02-01/NJW
*/

{
   c1 = (dim1+1.0) / 2;
   c2 = (dim2+1.0) / 2;
   d = distances( dim1, dim2, c1, c2 );
   return 1.0 - sost(d, radius, lambda );
}

/* Function curmark1 */

func curmark1(void, wcs=, style=, prompt=, nomark=, im=, ps=, symsize=, color=, thick=, tag= )
/* DOCUMENT pts = curmark1( <keywords> )
    or      curmark1, <keywords>

   Returns a two element array [x,y] with the position marked with click of left mouse
   button in the current plot frame (world coordinates).

   Keywords:
   wcs      : Must be an instance of struct 's_Coords' and then a four element
              array [x,y,coor1, coor2] is returned (works only with 'style=0').
   style   0: simple click     returns (x,y)
           1: rubberband box   returns (xpres,ypres,xrelease,yrelease)
           2: rubber band line returns (xpres,ypres,xrelease,yrelease)
   prompt   : Prompt string (defaults to 'Expecting left mouse click: ...'
   nomark   : Set if point marking is to be avoided
   im       : Displayed image if contrast color is requested
   ps       : Plot symbol for point mark
   symsize  : Symbol size for point mark
   color    : Color for plot symbol
   thick    : Thickness for plot symbol
   tag      : A number or a string to be written next to the marked point

   When called as a subroutine the values will be printed on the screen.

   Right mouse button will return void ([]).

   SEE ALSO: curmark, mouse

*/

{
   require, "plot.i";

   local lx, ly;

   if( is_void(prompt) ) prompt = "Expecting left mouse click: ...";
   if( is_void(style) ) style = 0;
   if( is_void(ps) ) ps = 2;
   if( is_void(symsize) ) symsize = 1.;
   if( !is_void(wcs) ) {
      if( style != 0 ) error,"Incompatible 'style' and 'wcs'";
      do_wcs = 1;
   } else do_wcs = 0;

   // Check for logarithmic scales
   mlogxy,lx,ly,get=1;

   res = mouse( -1, style, prompt );
   if( long(res(10)) != 1 ) return []; // Right mouse click
   if( lx ) res(1:3:2) = 10.^res(1:3:2);
   if( ly ) res(2:4:2) = 10.^res(2:4:2);
   if( style == 0 ) {
      if(!nomark) {
         if( !is_void(im) ) color = contrast( im, int(res(1)+0.5), int(res(2)+0.5) );
         oplot,[res(1)],[res(2)],ps=ps,symsize=symsize,color=color,thick=thick;
         if( tag ) {
            stag = typeof(tag) == "string" ? tag : itoa(tag);
            xyouts,res(1),res(2),stag,color=color;
         }
      }
      if( do_wcs ) {
         local co1, co2;
         skypos_fits, wcs, res(1), res(2), co1, co2, to_sky=1,silent=1;
      }
      if( am_subroutine() ) {
         if(do_wcs) {
           write,format="Result of curmark1: (x,y) = (%9.3f,%9.3f)\n", res(1), res(2);
           write,format="       WCS: (coor1,coor2) = (%9.3f,%9.3f)\n", co1, co2;
         } else write,format="Result of curmark1: (x,y) = (%.4f,%.4f)\n", res(1), res(2);
      } else {
         if(do_wcs) return grow(res(1:2),co1,co2);
         return res(1:2);
      }
   } else {
      oplot,[res(1),res(3),res(3),res(1),res(1)], \
            [res(2),res(2),res(4),res(4),res(2)],color=color,thick=thick;
      if( am_subroutine() ) {
         write,format="Result of curmark1: (x1,y1) = (%.3f,%.3f)\n", res(1), res(2);
         write,format="                    (x2,y2) = (%.3f,%.3f)\n", res(3), res(4);
      } else return res(1:4);
   }
}

/* Function curmark */

func curmark( void, nomark=, im=, ps=, symsize=, color= )
/* DOCUMENT pts = curmark( <keywords> )
     or     curmark, <keywords>

   Returns array with points marked with click of left mouse
   button.

   Keywords:
    nomark     Set if point marking is to be avoided
        ps     Plot symbol for point mark
        im     Displayed image if contrast colors are requested
   symsize     Symbol size for point mark
     color     For color of point mark

   When called as a subroutine the values will be printed on the screen.

   SEE ALSO: curmark1, mouse, contrast

*/

{
   require, "plot.i";

   local lx, ly;

   if( is_void(ps) ) ps = 2;
   if( is_void(symsize) ) symsize = 1.;

   // Check for logarithmic scales
   mlogxy,lx,ly,get=1;

   pts = [];
   i = 0;
   while( 1 ) {
      res = mouse( -1, 0, "Expecting left mouse click (right to stop ) ..." );
      if( long(res(10)) != 1 ) return pts;
      if( lx ) res(1:3:2) = 10.^res(1:3:2);
      if( ly ) res(2:4:2) = 10.^res(2:4:2);
      if(!nomark) {
         if( !is_void(im) ) color = contrast( im, int(res(1)+0.5), int(res(2)+0.5) );
         oplot,[res(1)],[res(2)],ps=ps,symsize=symsize,color=color;
      }
      if( am_subroutine() ) {
         write,format="Position %3i: (x,y) = (%.3f,%.3f)\n", ++i, res(1), res(2);
      }
      grow, pts, res(1:2);
   }
}

/* Function peaksearch0 */

func peaksearch0( im, reg=, src_rad=, bkg_rad1=, bkg_rad2=, show= )
/* DOCUMENT  res = peaksearch0( im, reg=, src_rad=, bkg_rad1=, bkg_rad2=, show= )

    Returns highest pixel value, position, and total of the strongest peak
    in the image 'im' :
     [maximal value, xpos, ypos, summed peak - bkg, significance,
                                         src_rad,bkg_rad1,bkg_rad2].

    If the keyword 'reg' (for region) has been given then the search
    for a maximum is confined to im(reg).

    The keywords define the source radius (src_rad) and the
    background annulus between bkg_rad1 and bkg_rad2.
    Default values are  src_rad  =  3.0 pixels
                        bkg_rad1 =  4.0 pixels
                        bkg_rad2 = 10.0 pixels
    [The values are saved to externals _Src_rad, _Bkg_rad1, _Bkg_rad2
     so that they need not be supplied at the next call]

    Keyword 'show' will display the regions and results.

    It is assumed that the image contains no NAN values.
    A cleaning can be done with 'nan2zero' found in basic.i

    This version applies only a search for the maximum
    in the image.

    SEE ALSO: peaksearch1, peaksearch2

   2006-02-23/NJW

*/
{

   extern _Src_rad, _Bkg_rad1, _Bkg_rad2;

   local mxval, xc, yc, idx;

   sz = dimsof(im);

   // The search for a maximum

   if( is_void(reg) ) {
      // The global maximum
      maxim, im, mxval, xc, yc;
   } else {
      // the local maximum
      maxarr,im(reg),mxval,idx;
      idxx = indices(im,reg(idx(1)));
      xc = idxx(1);
      yc = idxx(2);
   }

   if( is_void(src_rad) ) {
      if( is_void(_Src_rad) ) {
         src_rad = 3.0;
         _Src_rad = 3.0;
      } else src_rad = _Src_rad;
   } else {
      _Src_rad = src_rad;
   }
   if( is_void(bkg_rad1) ) {
      if( is_void(_Bkg_rad1) ) {
         bkg_rad1 = 4.0;
         _Bkg_rad1 = 4.0;
      } else bkg_rad1 = _Bkg_rad1;
   } else {
      _Bkg_rad1 = bkg_rad1;
   }
   if( is_void(bkg_rad2) ) {
      if( is_void(_Bkg_rad2) ) {
         bkg_rad2 = 10.0;
         _Bkg_rad2 = 10.0;
      } else bkg_rad2 = _Bkg_rad2;
   } else {
      _Bkg_rad2 = bkg_rad2;
   }

   if( src_rad > bkg_rad1 ) {
      write,format="bkg_rad1 redefined to src_rad: %10.3f\n", src_rad;
      bkg_rad1 = src_rad;
   }
   if( bkg_rad1 > bkg_rad2 ) {
      write,format="bkg_rad2 redefined to 1.5*bkg_rad1: %10.3f\n", 1.5*bkg_rad1;
      bkg_rad2 = 1.5*bkg_rad1;
   }

   xarr = span(1,sz(2),sz(2))(,-:1:sz(3));
   yarr = span(1,sz(3),sz(3))(-:1:sz(2),);

   d = distances( sz(2), sz(3), xc, yc );
   ann = where( d > bkg_rad1 & d < bkg_rad2 );
   w = where( d < src_rad );
   x = xarr(w);
   y = yarr(w);
   z = im(w) - avg(im(ann));
   peakcts = sum(z);
   if( peakcts <= 0.0 ) {
      xpos = double(xc);
      ypos = double(yc);
   } else {
      xpos = sum(x*z)/peakcts;
      ypos = sum(y*z)/peakcts;
      if(abs(xpos-xc) > src_rad ) xpos = double(xc);
      if(abs(ypos-yc) > src_rad ) ypos = double(yc);
   }

   // iterate for better source position

   xc = xpos;
   yc = ypos;

   d = distances( sz(2), sz(3), xc, yc );
   ann = where( d > bkg_rad1 & d < bkg_rad2 );
   w = where( d < src_rad );
   x = xarr(w);
   y = yarr(w);
   z = im(w) - avg(im(ann));
   peakcts = sum(z);
   if( peakcts <= 0.0 ) {
      xpos = double(xc);
      ypos = double(yc);
   } else {
      xpos = sum(x*z)/peakcts;
      ypos = sum(y*z)/peakcts;
      if(abs(xpos-xc) > src_rad ) xpos = double(xc);
      if(abs(ypos-yc) > src_rad ) ypos = double(yc);
   }

   if( show ) {
      disp, im;
      circa = span(0,2*pi,100);
      circx = cos(circa);
      circy = sin(circa);
      oplot,xpos+src_rad*circx,ypos+src_rad*circy;
      oplot,xpos+bkg_rad1*circx,ypos+bkg_rad1*circy;
      oplot,xpos+bkg_rad2*circx,ypos+bkg_rad2*circy;
   }

   significance = signif(im,pos=long([xpos+0.5,ypos+0.5]));

   return [double(mxval),xpos,ypos,peakcts,significance, \
                       src_rad,bkg_rad1,bkg_rad2];
}

/* Function peaksearch1 */

func peaksearch1( im, src_rad=, bkg_rad1=, bkg_rad2= )
/* DOCUMENT  res = peaksearch1( im, src_rad=, bkg_rad1=, bkg_rad2= )

    Returns position and amplitude of the strongest peak
    in the image 'im'.

    The keywords define the source radius (src_rad) and the
    background annulus between bkg_rad1 and bkg_rad2.
    Default values are  src_rad  =  7.0 pixels
                        bkg_rad1 =  7.0 pixels
                        bkg_rad2 = 10.0 pixels

    It is assumed that the image contains no NAN values
    A cleaning can be done with 'nan2zero' found in basic.i

    This version applies only a search for the maximum
    in the image followed by an 'amoeba' gauss peak fitting.

   2006-02-23/NJW

*/
{
   require, "datafit.i";

   sz = dimsof(im);
   maxim, im, mxval, xc, yc;

   if( is_void(src_rad) ) src_rad = 7.0;
   if( is_void(bkg_rad1) ) bkg_rad1 = 7.0;
   if( is_void(bkg_rad2) ) bkg_rad2 = 10.0;

   if( src_rad > bkg_rad1 ) {
      write,format="bkg_rad1 redefined to src_rad: %10.3f\n", src_rad;
      bkg_rad1 = src_rad;
   }
   if( bkg_rad1 > bkg_rad2 ) {
      write,format="bkg_rad2 redefined to 1.5*bkg_rad1: %10.3f\n", 1.5*bkg_rad1;
      bkg_rad2 = 1.5*bkg_rad1;
   }

   d = distances( sz(2), sz(3), xc, yc );
   ann = where( d > bkg_rad1 & d < bkg_rad2 );
   w = where( d < src_rad );
   xarr = span(1,sz(2),sz(2))(,-:1:sz(3));
   yarr = span(1,sz(3),sz(3))(-:1:sz(2),);
   x = xarr(w);
   y = yarr(w);
   z = im(w);

   estimate = [mxval, xc, yc, 1.1, avg(im(ann))];
   parm = gaussfit2ds_uam( x , y, z, esti=estimate, nterm=5, chat=3);
   return parm;
}

/* Function peaksearch2 */

func peaksearch2( im )
/* DOCUMENT  res = peaksearch2( im )

    Returns position and amplitude of the strongest peak
    in the image 'im'.

    The keywords define the source radius (src_rad) and the
    background annulus between bkg_rad1 and bkg_rad2.
    Default values are  src_rad  =  7.0 pixels
                        bkg_rad1 =  7.0 pixels
                        bkg_rad2 = 10.0 pixels

    It is assumed that the image contains no NAN values
    A cleaning can be done with 'nan2zero' found in basic.i

    This version applies a Mexican hat wavelet
    in the image followed by a gauss peak fitting.

   2006-02-23/NJW

*/
{

   extern Mex_hat, Mex_fk511;

   if( is_void(Mex_hat) ) mexican_hat_ps2;
   // initializing wavelet

   sz = dimsof(im);

   print,"Convolving ...";
   fim = fconvol( im, Mex_fk511 );

   maxim, fim, mxval, xc, yc;

   d = distances( sz(2), sz(3), xc, yc );
   k = d < 7.0;
   l = d > 3.0;
   ann = where( k+l == 2 );
   w = where( d < 7.0 );
   xarr = span(1,sz(2),sz(2))(,-:1:sz(3));
   yarr = span(1,sz(3),sz(3))(-:1:sz(2),);
   x = xarr(w);
   y = yarr(w);
   z = fim(w);

   estimate = [mxval, xc, yc, 1.1, avg(im(ann))];
   parm = gaussfit2ds_uam( x , y, z, esti=estimate, nterm=5, chat=1, delta=0.03);
   return parm;
}

/* Function disp9 */

func disp9( image_or_file, opts=, smooth=, off= )
/* DOCUMENT disp9, image_or_file, opts=, smooth=, off=

   Display image with ds9 through the dsinit, dsplay,
   and dsexit scripts.

   Keywords
     'opts' is a string that may contain
               z    for zoom to fit
               o    for log color scale
               i    for linear color scale
               s    for smoothing with standard setting
     'smooth' smooth parameter (0 will unset smoothing)
     'off'     will terminate the ds9 window

   2007-10-03/NJW, Updated 2014-02-13/NJW
*/
{
   extern flag_for_ds9_window;

   if( off ) {
      if( flag_for_ds9_window ) {
         write,"DISP9 : call dsexit";
         system,"dsexit";
         flag_for_ds9_window = 0;
      } else {
         write,"DISP9 : no ds9 window, so noop ...";
      }
      return;
   }

   if( !flag_for_ds9_window ) {
      write,"DISP9 : call dsinit";
      system,"dsinit";
      pause,3000;
      flag_for_ds9_window = 1;
   }
   if( is_void(opts) ) opts = "a"; // neutral - a NOOP setting

   if( structof( image_or_file) == string ) { // we have a filename
      system,"dsplay "+image_or_file+" "+opts;
   } else { // must be an image then
      writefits,"tmp.fits", image_or_file, clobber=1;
      write,"File tmp.fits ready for display";
      system,"dsplay tmp.fits "+opts;
   }
   if( !is_void(smooth) ) {
      if( smooth == 0 ) {
         system,"xpaset -p ds9 smooth no";
      } else {
         system,"xpaset -p ds9 smooth yes";
         system,"xpaset -p ds9 smooth radius "+itoa(smooth);
      }
   }

}

/* Function image2events */

struct s_Point_i2e {
   double x;
   double y;
}

func image2events( image, crpix1, crval1, cdelt1, crpix2, crval2, cdelt2, crd= )
/* DOCUMENT xy = image2events( image, crpix1, crval1, cdelt1,
                               crpix2, crval2, cdelt2, crd= )

   returns array of struct 's_Point_i2e' with elements x and y (double).
   where x = xy.x and y = xy.y are randomized events from a count image.
   Using FITS and ds9 convention that integer image indices refer to pixel
   midpoints.
   Keyword:    crd    is the array returned by 'get_crd' and will
                      override arguments

   2008-04-07/NJW
*/
{
   dms = dimsof(image);
   if( dms(1) != 2 ) {
      write,"image2events only operates on 2D images";
      return [];
   }

   im = long(image + 0.5);
   nim = sum(im);
   if( nim < 1 ) {
      write,"image2events found no counts in the image";
      return [];
   }

   if( is_void(crpix1) ) { // assume default values
      crpix1 = crval1 = crpix2 = crval2 = 0.0;
      cdelt1 = cdelt2 = 1.0;
   }
   if( numberof(crd) == 6 ) {
      crpix1 = crd(1,1); crval1 = crd(2,1); cdelt1 = crd(3,1);
      crpix2 = crd(1,2); crval2 = crd(2,2); cdelt2 = crd(3,2);
   }

   naxis1 = dms(2);
   naxis2 = dms(3);
   xi = yi = [];
   for(j=1;j<=naxis2;j++) {
      for(i=1;i<=naxis1;i++) {
         n = im(i,j);
         if( n ) {
            grow, xi, array(i,n);
            grow, yi, array(j,n);
         }
      }
   }

   r = random(nim) - 0.5;
   p = random(nim) - 0.5;

   xy = array(s_Point_i2e,nim);

   xy.x = (xi + r - crpix1)*cdelt1 + crval1;
   xy.y = (yi + p - crpix2)*cdelt2 + crval2;

   return xy;

}

/* Function rebin */

func rebin( arr, dim1, .. )
/* DOCUMENT res = rebin( arr, dim1, .. )

   Rebin a 1 or 2-D array into new dimensions that are either
   multiples of or into the original dimensions.

   No attention to keeping level or sum (counts).

   SEE ALSO: remap

   2008-01-10/NJW
*/
{
   arr = double(arr);
   dms = dimsof(arr);
   newdms = [1,dim1];
   while( more_args() ) {
      grow, newdms, next_arg();
      newdms(1)++;
   }
   if( dms(1) != newdms(1) ) error,"REBIN dimensionality mismatch";
   if( dms(1) > 2 ) error,"REBIN number of dimensions may not exceed 2";
   // check modularity
   ndims = numberof(dms)-1;
   mod = array(long, ndims);
   for( i = 1; i <= ndims; i++ ) {
      if( newdms(i+1) == dms(i+1) ) {
         mod(i) = 1;
      } else if( newdms(i+1) < dms(i+1) ) {
         r = dms(i+1) / newdms(i+1);
         if( r*newdms(i+1) != dms(i+1) ) error,"REBIN requires integer fractions";
         mod(i) = -r;
      } else {
         r = newdms(i+1) / dms(i+1);
         if( r*dms(i+1) != newdms(i+1) ) error,"REBIN requires integer fractions";
         mod(i) =  r;
      }
   }

   // Do rebinning dimension by dimension

   xarr = array(double, newdms );
   for( i = 1; i <= newdms(2); i++ ) {
      if( mod(1) == 1 ) {
         io = i;
      } else if( mod(1) < 0 ) {
         io = indgen(-mod(1)) - (i-1)*mod(1);
      } else {
         io = (i-1)/mod(1) + 1;
      }
      if( ndims == 2 ) {
         for( j = 1; j <= newdms(3); j++ ) {
            if( mod(2) == 1 ) {
               jo = j;
            } else if( mod(2) < 0 ) {
               jo = indgen(-mod(2)) - (j-1)*mod(2);
            } else {
               jo = (j-1)/mod(2) + 1;
            }
            xarr(i,j) = sum(arr(io,jo));
         }
      } else {
         xarr(i) = sum(arr(io));
      }
   }

   return xarr;

}

/* Function remap */

func remap( arr, dim1, .. )
/* DOCUMENT res = remap( arr, dim1, .. )

   Remap a 1 or 2-D array into new dimensions of unrestricted
   values. Lowering the dimensionality may lead to an inaccurate
   result since the original array is sampled rather than
   average. (There is room for improvements).

   SEE ALSO: rebin, flex_image

   2008-01-10/NJW
*/
{
   arr = double(arr);
   dms = dimsof(arr);
   newdms = [1,dim1];
   while( more_args() ) {
      grow, newdms, next_arg();
      newdms(1)++;
   }
   if( dms(1) != newdms(1) ) error,"REMAP dimensionality mismatch";
   if( dms(1) > 2 ) error,"REMAP number of dimensions may not exceed 2";
   delta = 1.0 / dms(2);
   //+ xiarr = delta/2 + (indgen(dms(2))-1)*delta;
   ndelta = 1.0 / newdms(2);
   nxiarr = ndelta/2 + (indgen(newdms(2))-1)*ndelta;
   oldi = long(nxiarr/delta) + 1;
   if( dms(1) > 1 ) {
      felta = 1.0 / dms(3);
      //+ yiarr = felta/2 + (indgen(dms(3))-1)*felta;
      nfelta = 1.0 / newdms(3);
      nyiarr = nfelta/2 + (indgen(newdms(3))-1)*nfelta;
      oldj = long(nyiarr/felta) + 1;
   }

   xarr = array( double, newdms );
   for( i = 1; i <= newdms(2); i++ ) {
      if( dms(1) > 1 ) {
         for( j = 1; j <= newdms(3); j++ ) {
            xarr(i,j) = arr(oldi(i),oldj(j));
         }
      } else {
         xarr(i) = arr(oldi(i));
      }
   }

   return xarr;

}

/* Function unwiden_image */

func unwiden_image( im, w )
/* DOCUMENT new_image = unwiden_image( im, w )

   Returns an image where the edge of width 'w' has been removed.

   SEE ALSO: widen_image

*/
{
   return im(1+w:-w,1+w:-w);
}

/* Function widen_image */

func widen_image( im, w, val= )
/* DOCUMENT new_image = widen_image( im, w, val= )

   Returns an image that has been enlarged by 'w' (int)
   in all directions and values have been filled out
   with the edge values of the original image ('im').
   The datatype is unchanged.

   If keyword 'val' is set then the filling will be with this.

   SEE ALSO: unwiden_image

   2009-11-04/NJW

*/
{
   dms = dimsof(im);
   v = !is_void(val);
   // create new image
   wim = array(structof(im),dms(2)+2*w,dms(3)+2*w);
   // --- fill out central part
   wim(w+1:w+dms(2),w+1:w+dms(3)) = im;
   // --- fill out left side
   if( v ) wim(1:w,w+1:w+dms(3)) = val;
   else wim(1:w,w+1:w+dms(3)) = im(1,)(-:1:w,);
   // --- fill out right side
   if(v)wim(w+dms(2)+1:0,w+1:w+dms(3)) = val;
   else wim(w+dms(2)+1:0,w+1:w+dms(3)) = im(0,)(-:1:w,);
   // --- fill out lower side
   if(v)wim(w+1:w+dms(2),1:w) = val;
   else wim(w+1:w+dms(2),1:w) = im(,1)(,-:1:w);
   // --- fill out upper side
   if(v)wim(w+1:w+dms(2),w+1+dms(3):0) = val;
   else wim(w+1:w+dms(2),w+1+dms(3):0) = im(,0)(,-:1:w);
   // --- fill out lower left corner
   if(v)wim(1:w,1:w) = val;
   else wim(1:w,1:w) = im(1,1);
   // --- fill out lower right corner
   if(v)wim(dms(2)+w+1:0,1:w) = val;
   else wim(dms(2)+w+1:0,1:w) = im(0,1);
   // --- fill out upper left corner
   if(v)wim(1:w,dms(3)+w+1:0) = val;
   else wim(1:w,dms(3)+w+1:0) = im(1,0);
   // --- fill out upper right corner
   if(v)wim(dms(2)+w+1:0,dms(3)+w+1:0) = val;
   else wim(dms(2)+w+1:0,dms(3)+w+1:0) = im(0,0);

   return wim;
}

/* Function getimval */

func getimval( im )
/* DOCUMENT image_value = getimval( im )
     or     getimval, im

   Retrieve a pixel value from an image previously displayed
   by function 'disp'.

   When called as a subroutine the indices and value will be
   printed on the screen.

   SEE ALSO: disp, curmark1, getimvals

   2010-08-06/NJW
*/
{
   pos = curmark1(prompt="Mark the pixel ... ",nomark=1);
   i = long(floor(pos(1)+0.5));
   j = long(floor(pos(2)+0.5));
   value = im(i,j);

   if( am_subroutine() ) {
      stim = structof(im);
      if( stim == long || stim == int || stim == char ) {
         write,format="(%i,%i) has value %i\n", i, j, value;
      } else if( stim == double || stim == float ) {
         if( abs(value) < 1.e-3 || abs(value) > 1.e3 ) {
            write,format="(%i,%i) has value %.5e\n", i, j, value;
         } else {
            write,format="(%i,%i) has value %.5f\n", i, j, value;
         }
      } else error,"Data type not supported";
   }

   return value;

}

/* Function getimvals */

func getimvals( im )
/* DOCUMENT image_values = getimvals( im )
     or     getimvals, im

   Retrieve several pixel values from an image previously displayed
   by function 'disp'.

   When called as a subroutine the indices and values will be
   printed on the screen.

   SEE ALSO: disp, curmark, getimval

   2010-08-06/NJW
*/
{
   vals = [];
   do {
      pos = curmark1(prompt="Mark the pixel (or right click to stop) ... ",nomark=1);
      if( is_void(pos) ) break;

      i = long(floor(pos(1)+0.5));
      j = long(floor(pos(2)+0.5));
      value = im(i,j);
      grow, vals, value;

      if( am_subroutine() ) {
         stim = structof(im);
         if( stim == long || stim == int || stim == char ) {
            write,format="(%i,%i) has value %i\n", i, j, value;
         } else if( stim == double || stim == float ) {
            if( abs(value) < 1.e-3 || abs(value) > 1.e3 ) {
               write,format="(%i,%i) has value %.5e\n", i, j, value;
            } else {
               write,format="(%i,%i) has value %.5f\n", i, j, value;
            }
         } else error,"Data type not supported";
      }

   } while( 1 );

   return vals;

}

/* Function imcut */

func imcut( im, lofac, hifac, pane= )
/* DOCUMENT rescaled_im = imcut( im, lofac, hifac, pane= )

   The highest and lowest image values are changed according
   to the following algorithm:

   A histogram of image values is produced and the peak is found
   with value vpeak.

   Moving upwards the value is found where the
   histogram has fallen to 40% of the peak, called v40p.
   All image pixels where the value exceeds vpeak + hifac*(v40p - vpeak)
   is set to this value.

   Moving downwards the value is found where the
   histogram has fallen to 40% of the peak, called v40m.
   All image pixels where the value undershoots vpeak - lofac*(vpeak - v40m)
   is set to this value.

   In many cases lofac = 2 and hifac = 4 gives good results.

   2010-09-08/NJW

*/
{
   local h, x;

  /*
   *   Make histogram to decide where a sensible cut should be made
   */

   imw = double(im);
   imx = max(imw);
   imn = min(imw);
   bsize = (imx-imn)/200;
   histos, imw, h, x, binsize=bsize;
   if( !is_void(pane) ) {
      window,pane;
      plot,x,h,ps=10;
   }
   hmax = max(h);
   imax = where(hmax == h)(1);
   nh = numberof(h);
   if( imax == 1 || imax == nh ) {
      write,"IMCUT: image is not suited for this function";
      return imw;
   }
   if( hmax > 1.1 * max(h(imax-1), h(imax+1)) ) {
      hmax = h(imax) = 1.1 * max(h(imax-1), h(imax+1));
   }

   // find lower half

   for( i = imax; i > 0; i-- ) {
      if( h(i) < 0.4*hmax ) break;
   }

   // find upper half

   for( j = imax; j <= nh; j++ ) {
      if( h(j) < 0.4*hmax ) break;
   }

   if( j - i == 2 ) { // There must be one value that dominates
                      // Discard and set new value
      hmax = 1.1*max(h(i),h(j));
      if( !is_void(pane) ) {
         pause,1000;
         h(imax) = hmax;
         plot,x,h,ps=10;
      }

      // find lower half

      for( i = imax; i > 0; i-- ) {
         if( h(i) < 0.4*hmax ) break;
      }

      // find upper half

      for( j = imax; j <= nh; j++ ) {
         if( h(j) < 0.4*hmax ) break;
      }
   }

   dxhi = x(j) - x(imax);
   dxlo = x(imax) - x(i);

   // lower limit

   llim = x(imax) - lofac*dxlo;
   loreg = where( imw < llim );
   if( numberof(loreg) ) imw(loreg) = llim;

   // upper limit

   ulim = x(imax) + hifac*dxhi;
   hireg = where( imw > ulim );
   if( numberof(hireg) ) imw(hireg) = ulim;

   if( !is_void(pane) ) {
      oplot,x(i)*[1,1],[0,hmax],li=2;
      oplot,x(j)*[1,1],[0,hmax],li=2;
      oplot,llim*[1,1],[0,hmax];
      oplot,ulim*[1,1],[0,hmax];
   }

   return imw;

}

/* Function erase_source */

func erase_source( im, res )
/* DOCUMENT new_im = erase_source(im, res)

   Returns an image where the source region defined
   by 'peaksearch0' has been replaced by random
   values from the background annulus.

   res:

  [highest value, xpos, ypos, summed peak - bkg,significance,
                              src_rad,bkg_rad1,bkg_rad2].

   2010-09-20/NJW
*/
{
   dms = dimsof( im );
   d = distances(dms(2),dms(3),res(2),res(3));
   wsrc = where(d < res(6));
   wbkg = where(d > res(7) & d < res(8) );
   nsrc = numberof(wsrc);
   nbkg = numberof(wbkg);
   w = sort(random(nbkg))(1:nsrc);
   imr = im;
   imr(wsrc) = im(wbkg(w));

   return imr;
}

/* Function gflatfield */

func gflatfield( im, sigma, &gim, sel= )
/* DOCUMENT new_im = gflatfield( im, sigma[, >gim], sel= )

   The image 'im' is folded with a gaussian kernel
   with sigma = 'sigma' and then subtracted from the
   original image. The process is repeated with
   excluded source areas.
   Keyword 'sel' must be an array of same dimension
   as 'im' with 1 where pixels are useful and zero
   elsewhere.

   2007-06-19/NJW

*/
{
   require, "fconvol.i";

   di = dimsof(im);
   if( di(1) != 2 ) {
      write,"Not a 2D image";
      return [];
   }
   dosel = 0;
   if( !is_void(sel) ) {
      d = dimsof(sel);
      if( d(1) != 2 ) {
         write,"'sel' is not a 2D image";
         return [];
      }
      if( d(2) != di(2) || d(3) != di(3) ) {
         write,"'sel' is not commensurate with 'im'";
         return [];
      }
      dosel = 1;
   }
   wsel = dosel ? where( sel ) : indgen(di(2)*di(3));
   //   Avoid changing the input image
   im1 = im;
   gim = gfconvol( im, sigma );
   //    First subtraction
   diff  = im - gim;
   //    Substitute image parts that exceed 3sigma
   //    to avoid subtracting too much around sources
   rms_1 = wrms( diff(wsel) );
   q = where( abs(diff) > 3*rms_1 );
   if( numberof(q) > 0 ) {
      im1(q) = gim(q);
      gim = gfconvol( im1, sigma );
      diff = im - gim;
   }

   return diff;
}

/* Function fflatfield */

func fflatfield( im, freq, &ffim, sel= )
/* DOCUMENT new_im = fflatfield( im, freq[, >ffim], sel= )

   The image 'im' is Fourier transformed and
   frequencies above 'freq' are removed.
   'freq' is an integer or a two-element vector of
   integers where each is smaller than half of the
   corresponding dimension.
   The flattened image is then subtracted from the
   original image.
   Keyword 'sel' must be an array of same dimension
   as 'im' with 1 where pixels are useful and zero
   elsewhere.

   2010-09-20/NJW

*/
{
   dms = dimsof(im);
   o1 = dms(2)%2;
   o2 = dms(3)%2;
   if( dms(1) != 2 ) {
      write,"Not a 2D image";
      return [];
   }
   dosel = 0;
   if( !is_void(sel) ) {
      d = dimsof(sel);
      if( d(1) != 2 ) {
         write,"'sel' is not a 2D image";
         return [];
      }
      if( d(2) != dms(2) || d(3) != dms(3) ) {
         write,"'sel' is not commensurate with 'im'";
         return [];
      }
      dosel = 1;
   }
   wsel = dosel ? where( sel ) : indgen(dms(2)*dms(3));
   //   Avoid changing the input image
   freq1 = numberof(freq) > 1 ? freq(1) : freq;
   freq2 = numberof(freq) > 1 ? freq(2) : freq;
   im1 = im;
   fim = fft(im,-1);
   /*
    *   The higher frequencies are removed symmetrically
    *   so that the backwards transformation has zero
    *   imaginary part except for rounding-off errors.
    */
   i1 = 1 + freq1;
   i2 = dms(2) + 1 - freq1;
   j1 = 1 + freq2;
   j2 = dms(3) + 1 - freq2;
   fim(i1:i2,j1:j2) = 0.0;
   ffim = fft(fim,1).re/numberof(im);

   //    Subtraction
   diff  = im - ffim;
   //    Substitute image parts that exceed 3sigma
   //    to avoid subtracting too much around sources
   //+ rms_1 = wrms( diff(wsel) );
   //+ q = where( abs(diff) > 3*rms_1 );
   //+ if( numberof(q) > 0 ) {
   //+    im1(q) = gim(q);
   //+    gim = gfconvol( im1, sigma );
   //+    diff = im - gim;
   //+ }

   return diff;
}

/* Function signif */

func signif( im, pos=, src_rad=, bkg_rad1=, bkg_rad2= )
/* DOCUMENT significance = signif( im, pos=, src_rad=, bkg_rad1=, bkg_rad2= )

   Returns the significance defined as

     (im - avg(bkg))/rms(bkg)

   Keywords:    pos          a set of (i,j)
                src_rad      defaults to  3.0 pixels
                bkg_rad1     defaults to  4.0 pixels
                bkg_rad2     defaults to 10.0 pixels
    [The values are saved to externals _Src_rad, _Bkg_rad1, _Bkg_rad2
     so that they need not be supplied in the following call]

   If 'pos' is set as a two-element vector then a single value is
   returned, else the entire significance map is returned.
*/
{
   extern _Src_rad, _Bkg_rad1, _Bkg_rad2;

   dms = dimsof(im);
   im = double(im);

   if( is_void(src_rad) ) {
      if( is_void(_Src_rad) ) {
         src_rad = 3.0;
         _Src_rad = 3.0;
      } else src_rad = _Src_rad;
   } else {
      _Src_rad = src_rad;
   }
   if( is_void(bkg_rad1) ) {
      if( is_void(_Bkg_rad1) ) {
         bkg_rad1 = 4.0;
         _Bkg_rad1 = 4.0;
      } else bkg_rad1 = _Bkg_rad1;
   } else {
      _Bkg_rad1 = bkg_rad1;
   }
   if( is_void(bkg_rad2) ) {
      if( is_void(_Bkg_rad2) ) {
         bkg_rad2 = 10.0;
         _Bkg_rad2 = 10.0;
      } else bkg_rad2 = _Bkg_rad2;
   } else {
      _Bkg_rad2 = bkg_rad2;
   }

   if( numberof(pos) == 2 ) {
      d = distances(dms(2),dms(3),pos(1),pos(2));
      bkg_area = where( d >= bkg_rad1 & d < bkg_rad2 & im != 0.0 );
      if( !numberof(bkg_area) ) {
         sigim = 0.0;
      } else {
         g_bkg_avg = wavg(im(bkg_area));
         g_bkg_rms = wrms(im(bkg_area));
         sigim = (im(pos(1),pos(2)) - g_bkg_avg)/g_bkg_rms;
      }
      return sigim;
   } else {
      sigim = im;
      for( i = 1; i <= dms(2); i++ ) {
      for( j = 1; j <= dms(3); j++ ) {
         d = distances(dms(2),dms(3),i,j);
         bkg_area = where( d >= bkg_rad1 & d < bkg_rad2 & im != 0.0 );
         if( !numberof(bkg_area) ) {
            sigim(i,j) = 0.0;
         } else {
            g_bkg_avg = wavg(im(bkg_area));
            g_bkg_rms = wrms(im(bkg_area));
            sigim(i,j) = (im(i,j) - g_bkg_avg)/g_bkg_rms;
         }
      }}
      return sigim;
   }
}

/* Function cur_extract_box */

func cur_extract_box( im, &corners, ioff=, joff=, over=, color=, thick= )
/* DOCUMENT box = cur_extract_box( im, >corners, ioff=, joff=, over=, color=, thick= )

   Saves the part of image 'im' (previously displayed, or at least an image
   with identical or smaller dimensions) outlined by the use of the cursor.

   Argument
     corners    Returns [x1,y1,x2,y2]

   Keywords
     over       Causes overplotting the image with the extracted box outline
     color      Specifies overplotting color
     thick      Specifies overplotting line thickness
      ioff      Offset of axis in x-direction (defaults to zero)
      joff      Offset of axis in y-direction (defaults to zero)

   2012-05-02/NJW

   SEE ALSO:  imcrop (second version)
*/
{
   outline = curmark1(prompt="Click and draw to other corner ... ",style=2);
   if( is_void(ioff) ) ioff = 0;
   if( is_void(joff) ) joff = 0;
   x1 = long(outline(1)+0.5) - ioff;
   y1 = long(outline(2)+0.5) - joff;
   x2 = long(outline(3)+0.5) - ioff;
   y2 = long(outline(4)+0.5) - joff;
   if( x1 > x2 ) { x = x1; x1 = x2; x2 = x; }
   if( y1 > y2 ) { y = y1; y1 = y2; y2 = y; }

   // Test index limits

   dms = dimsof(im);
   if( x1 < 1 ) x1 = 1; if( y1 < 1 ) y1 = 1;
   if( x2 > dms(2) ) x2 = dms(2); if( y2 > dms(3) ) y2 = dms(3);

   corners = [x1,y1,x2,y2];

   if( over ) oplot,[x1-0.5,x2+0.5,x2+0.5,x1-0.5,x1-0.5], \
       [y1-0.5,y1-0.5,y2+0.5,y2+0.5,y1-0.5],color=color,thick=thick;

   return im(x1:x2,y1:y2);
}

/* Function imhcut */

func imhcut( im, locut, hicut )
/* DOCUMENT cut_im = imhcut( im, locut, hicut )

    Return an image with the values cut.

    Giving an argument as void means no action.
*/
{
   res = im;

   if( !is_void(locut) ) {
      w = where( im < locut );
      if( numberof(w) ) {
         res(w) = locut;
      } else res(1,1) = locut;       
   }

   if( !is_void(hicut) ) {
      w = where( im > hicut );
      if( numberof(w) ) {
         res(w) = hicut;
      } else res(2,1) = hicut;       
   }

   return res;
}

/* Function imcolorbar */

func imcolorbar( im, scale=, ndec=, levs= )
/* DOCUMENT imcolorbar, im, scale=, ndec=, levs=

    Uses 'colorbar' from 'util_fr_2106.i' to draw a colorbar
    to the right of the image with max and min from the image 'im'.

    Keywords:
     scale    used to scale the level values for a nice appearance
              Automatic scaling to largest value between 1 and 10
              when not given.
      ndec    Number of decimals
      levs    Number of tick marks

    2013-01-09/NJW
*/
{
   vx = max(im);
   vn = min(im);
   avn = abs(vn);
   avx = abs(vx);
   if( is_void(levs) ) levs = 7;
   if( is_void(ndec) ) ndec = 2;
   if( is_void(scale) ) {
      scale = 10.^long(floor(log10(max(avn,avx))));
   }
   if( scale == 1 ) scale = [];
   colorbar, vn, vx, levs=levs, ndec=ndec, scale=scale;
}

/* Function manip_image_get */

func manip_image_get( dol_input, silent= )
/* DOCUMENT im = manip_image_get( dol_input, silent= )

     Returns the image from a FITS file extension.
     The header is copied to externals to be used by
     the 'kwds_set' function, ready to save e.g. after
     manipulation by using 'manip_image_put'.

     The WCS struct is saved as an external into: Mig_wcs.

     SEE ALSO: manip_image_put

   2013-02-21/NJW
*/
{
   extern Mig_wcs;

   v = !silent;
   im = readfits(dol_input);
   hdr = headfits(dol_input,nocheck=1);
   fits_copy_keys, hdr, tokwds=1;
   Mig_wcs = get_wcs( hdr );
   if( v ) {
      write,"crpix1 = ", Mig_wcs.crpix1;
      write,"crpix2 = ", Mig_wcs.crpix2;
      write,"crval1 = ", Mig_wcs.crval1;
      write,"crval2 = ", Mig_wcs.crval2;
      write,"cd1_1 = ", Mig_wcs.cd1_1;
      write,"cd2_2 = ", Mig_wcs.cd2_2;
      write,"cd1_2 = ", Mig_wcs.cd1_2;
      write,"cd2_1 = ", Mig_wcs.cd2_1;
   }
   return im;
}

/* Function manip_image_put */

func manip_image_put( outfile, im, clobber= )
/* DOCUMENT manip_image_put, outfile, im, clobber=

    Writes the image 'im' to a FITS file. If 'manip_image_get'
    has been used in the same Yorick session then the header
    information from the input image is copied to the output image
    extension.

    SEE ALSO: manip_image_get

   2013-02-21/NJW
*/
{
   extern kwds_cnames, kwds_ssequence;
   if( !is_void(kwds_cnames) && kwds_cnames(1) == "SIMPLE" ) {  // must come from extension 0
      // Remove the SIMPLE keyword and let it be replaced by
      // XTENSION while taking care that it becomes number one.
      kwds_del, "SIMPLE";
      kwds_set, "XTENSION", "IMAGE", "Type of this extension";
      kwds_ssequence(0) = 1; // since SIMPLE had the number one place
      // it is now free
   }
      
   writefits, outfile, im, clobber=clobber;
}

/* Function remap_wcs */

func remap_wcs( image_or_dol, wcs_in, outfile, wcs_out, dims_out, restrict=, per_pixel=, clobber=, db= )
/* DOCUMENT remap_wcs, image_or_dol, wcs_in, outfile, wcs_out, dims_out, restrict=, per_pixel=, clobber=, db=
     newim = remap_wcs( image_or_dol, wcs_in, outfile, wcs_out, dims_out, restrict=, per_pixel=, clobber=, db=)

   Remaps the input image. The output image is scanned over all pixels and for each
   pixel the value of the corresponding WCS position in the input image is copied to
   the output pixel. This corresponds to values given per unit sky area which is
   useful for e.g. temperature maps and surface brightness maps.

   If values per pixel are given and wanted in the output image (indicated by the keyword
   'per_pixel') then the factor

        (CDELT1(output) * CDELT2(output)) / (CDELT1(input) * CDELT2(input))

   is applied to the output image.

   Arguments
     image_or_dol   : A 2D image or a DOL that points to a FITS extension
                      with WCS information.
     wcs_in         : An instance of the struct 's_Coords' to be supplied when
                      'image_or_dol' is an image array.
     outfile        : Filename for storing the result. If given as void or "none"
                      then no output file will be written.
     wcs_out        : An instance of the struct 's_Coords' for the output
                      array
     dims_out       : A two-element integer array with output image dimensions

    Keywords:
      restrict      If given, no extrapolation of values will be done and the uncovered output
                    image parts will be given this value.
     per_pixel      (bool) Apply the correction factor (not implemented yet /2014-12-02).
      clobber       (bool) Overwrite the output file if it exists beforehand.
           db       (bool) Get debugging output

   2014-02-20/NJW
*/
{

   /* --------------- comment for the programmer ------------------  
    *
    *   A future development could imply a subdivision of pixels
    *
    * ------------------------------------------------------------
    */

   id = gen_log_init( "remap_wcs.log", logflag=2, erase=1, reset=1 );

   // Decide whether we have an image or a DOL as first argument

   local filename, extno;

   if( structof(image_or_dol) == string ) { // This is a DOL
      get_exten_no, image_or_dol, filename, extno;
      if( !file_test(filename) ) {
         gen_log_entry,1,"Did not find: "+filename,id=id;
         write,"Did not find: "+filename+" quit!";
         return;
      }
      image_in = readfits( image_or_dol );
      hdr_in = headfits( image_or_dol );
      wcs_in = get_wcs( hdr_in );
      dms = dimsof(image_in);
      gen_log_entry,1,"Read image in "+image_or_dol,id=id;
   } else { // Supposedly an image that must be accompanied by a 'wcs_in'
      dms = dimsof( image_or_dol );
      if( dms(1) == 2 && !is_void(wcs_in ) ) {
         image_in = image_or_dol;
      } else error,"REMAP_WCS got bad 1. or 2. argument";
      gen_log_entry,1,"1. argument is an image",id=id;
   }
   gen_log_entry,1,"Image dimensions: "+swrite(dms(2),dms(3),format="%ix%i"),id=id;

   // Define the output image

   image_out = array(0.,dims_out(1),dims_out(2));
   local coord1, coord2, fi_in, fj_in;
   // Loop over all output pixels
   for(i = 1; i <= dims_out(1); i++ ) {
      for(j = 1; j <= dims_out(2); j++ ) {
         // where is the pixel on the sky?
         skypos_fits, wcs_out, i, j, coord1, coord2, to_sky=1, silent=1;
         // what pixel in the input image should we use?
         skypos_fits, wcs_in, fi_in, fj_in, coord1, coord2, to_pix=1, silent=1;
         i_in = long(fi_in+0.5);
         j_in = long(fj_in+0.5);
         if( is_void(restrict) ) {
            if( i_in < 1 ) i_in = 1;
            if( i_in > dms(2) ) i_in = dms(2);
            if( j_in < 1 ) j_in = 1;
            if( j_in > dms(3) ) j_in = dms(3);
            image_out(i,j) = image_in(i_in,j_in);
         } else {
            if( i_in < 1 || i_in > dms(2) || j_in < 1 || j_in > dms(3) ) image_out(i,j) = restrict;
            else image_out(i,j) = image_in(i_in,j_in);
         }
      }
   }

   if( per_pixel ) { // execute the correction
      factor = (wcs_out.scale/wcs_in.scale)^2;
      image_out *= factor;
   }

   if( !is_void(outfile) && outfile != "none" ) {
      put_wcs, wcs_out; // Save the WCS keywords to memory
      writefits, outfile, image_out, clobber=clobber;
   }
   if( !am_subroutine() ) return image_out;
}

/* Function makeimageu */

func makeimageu( imdim, xval, yval, xr=, yr=, nt=, weight= )
/* DOCUMENT image = makeimageu( imdim, xval, yval, xr=, yr=, nt=, weight= )

  Make 2D image (a 2D histogram) from array of x values
   and one of y values with optional weights

   Arguments
     imdim   :  2-element array with image dimensions
      xval   :  1D array with all x-values
      yval   :  1D array with all y-values - must have same number of
                elements as 'xval'
   Keywords
     xr, yr  :  2-element arrays with minimum and maximum.
     nt      : When set it implies integer values in xval and yval that
               are interpreted as indices directly. Overrides xr and yr.
     weight  : If given it must be an array conformable with xval.

   2006-08-25/NJW Converted makeimageu.pro to Yorick
   2012-10-21/NJW Updated with keyword 'nt'

*/
{

   index = dimsof(imdim);
   if( index(1) != 1 && index(1) != 2 ) {
      write,"Incorrect number of dimensions for image";
      return -1.;
   }
   nx = numberof(xval);
   if( nx != numberof(yval) ) {
      print,"Not matching number of dimensions in call";
      return -1.;
   }

   if( is_void(weight) ) weight = xval*0. + 1.;
   if( numberof(weight) != nx ) {
        print,"Incorrect number of elements in weight factor array";
        return -1.;
   }

   im = array( float, imdim(1), imdim(2));

   if( nt ) {
      for( i = 1; i <= nx; i++ ) {
         ix = xval(i);
         if( ix >= 1 && ix <= imdim(1) ) {
            iy = yval(i);
            if( iy >= 1 && iy <= imdim(2) ) im(ix,iy) = im(ix,iy) + weight(i);
         }
      }
      return im;
   }

   // Definition of limits;

   if( numberof(xr) == 2 ) {
        xmin = xr(1)*1.0;
        xmax = xr(2)*1.0;
   } else {
        xmin = min(xval)*1.0;
        xmax = max(xval)*1.0;
   }
   if( numberof(yr) == 2 ) {
        ymin = yr(1)*1.0;
        ymax = yr(2)*1.0;
   } else {
        ymin = min(yval)*1.0;
        ymax = max(yval)*1.0;
   }
   dx = (xmax - xmin) / imdim(1);
   dy = (ymax - ymin) / imdim(2);

   for( i = 1; i <= nx; i++ ) {
      ix = long((xval(i) - xmin) / dx + 1);
      if( ix >= 1 && ix <= imdim(1) ) {
         iy = long((yval(i) - ymin) / dy + 1);
         if( iy >= 1 && iy <= imdim(2) ) im(ix,iy) = im(ix,iy) + weight(i);
      }
   }
   return im;
}

/* Function makeimagez */

func makeimagez( imdim, x, y, z, xr=, yr=, fill= )
/* DOCUMENT arr = makeimagez( imdim, x, y, z, xr=, yr=, fill= )

    Make an image with dimensions 'imdim' (two element array) of 'z' values.
    A function z = z(x,y) has been sampled numberof(x) times. An image is
    produced from this sampling with the possibility to fill out pixels not
    covered by the sampling (by keyword 'fill'). More than one value in a pixel
    are averaged out.

    If more than one z-value falls in a pixel then the values are averaged.

    Arguments:
       imdim   (two element integer array) Resulting image dimensions
       x       array of x-values
       y       array of y-values
       z       array of z-values, numberof(x) == numberof(y) == numberof(z)
       
    Keywords:
       xr      (two element array) xmin and xmax. Defaults to min(x) and max(x).
       yr      (two element array) ymin and ymax. Defaults to min(y) and max(y).
       fill    set, if empty pixels are to be filled with value from closest
               neighbor.

*/
{
   n = numberof(x);

   if( numberof(y) != n ) error,"MAKEIMAGEZ ##1##";
   if( numberof(z) != n ) error,"MAKEIMAGEZ ##2##";
   nx = imdim(1);
   ny = imdim(2);

   x = double(x);
   y = double(y);
   z = double(z);
   if( is_void(xr) ) {
      xmin = min(x);
      xmax = max(x);
      // adjust values to keep extreme points within range
      xmin -= 1.e-6*(xmax - xmin);
      xmax += 1.e-6*(xmax - xmin);
   } else {
      xmin = double(xr(1));
      xmax = double(xr(2));
   }
   if( is_void(yr) ) {
      ymin = min(y);
      ymax = max(y);
      // adjust values to keep extreme points within range
      ymin -= 1.e-6*(ymax - ymin);
      ymax += 1.e-6*(ymax - ymin);
   } else {
      ymin = double(yr(1));
      ymax = double(yr(2));
   }

   v_arr = array(double, nx, ny);
   i_arr = array(long, nx, ny);
   deltax = (xmax - xmin) / nx;
   deltay = (ymax - ymin) / ny;

   nrejected = 0;
   for( i = 1; i <= n; i++ ) {
      if( x(i) >= xmax ) { nrejected++; continue; }
      if( x(i) < xmin ) { nrejected++; continue; }
      if( y(i) >= ymax ) { nrejected++; continue; }
      if( y(i) < ymin ) { nrejected++; continue; }
      ix = long((x(i) - xmin)/deltax + 1);
      iy = long((y(i) - ymin)/deltay + 1);

      ia = i_arr(ix,iy);
      va = v_arr(ix,iy); // previous value, average of 'ia' values
      v = va * ia + z(i); // new sum of values
      v /= (++ia); // new average
      i_arr(ix,iy) = ia;
      v_arr(ix,iy) = v;
   }
   write,"Rejected "+itoa(nrejected)+" points";

   if( fill ) {

      // Now 'i_arr' is searched for holes; they must be covered by looking for
      // filled-out neighbors

      w = where(i_arr == 0);
      nw = numberof(w);

      for( i = 1; i <= nw; i++ ) {
         iy = (w(i) - 1) / nx;
         ix = w(i) - (iy++)*nx;
         iasum = 0;
         vasum = 0.0;
         sep = 1;
         while( iasum == 0 ) {
            for( jj = -sep; jj <= sep; jj+= 2*sep ) {
               for( ii = -sep; ii <= sep; ii++ ) { // stepping +x
                  if( ix + ii < 1 ) continue;
                  if( ix + ii > nx ) continue;
                  if( iy + jj < 1 ) continue;
                  if( iy + jj > ny ) continue;
                  if( i_arr(ix+ii,iy+jj) ) {
                     iasum++;
                     vasum += v_arr(ix+ii,iy+jj);
                  }
               }
            }
            for( ii = -sep; ii <= sep; ii+= 2*sep ) {
               for( jj = -sep; jj <= sep; jj++ ) { // stepping +y
                  if( ix + ii < 1 ) continue;
                  if( ix + ii > nx ) continue;
                  if( iy + jj < 1 ) continue;
                  if( iy + jj > ny ) continue;
                  if( i_arr(ix+ii,iy+jj) ) {
                     iasum++;
                     vasum += v_arr(ix+ii,iy+jj);
                  }
               }
            }
            sep++; // prepare for next neighbors
         }
         v_arr(ix,iy) = vasum / iasum;
      }
   }
   return v_arr;
}

/* Function maxim */

func maxim( image, &maxval, &xc, &yc, chat= )
/* DOCUMENT maxim, image, >maxval, >xc, >yc, chat=
       or    res = maxim( image, >maxval, >xc, >yc, chat= )

   returns the maximum on an image as well as the
   coordinates (pixel numbers) in the arguments.
   Keyword 'chat' will cause a display on the screen.

   In the second form (function) [xc,yv] is returned as a result.
*/

{

   dms = dimsof(image);
   maxval = max(image);
   idx = where(image == maxval)(1) - 1;
   yc = idx / dms(2);
   xc = idx - yc*dms(2) + 1;
   yc++;
   if( chat ) \
   write,format="Max: %11.3g, pos: (%i,%i)\n", double(maxval), xc, yc;
   if( !am_subroutine() ) return [xc, yc];

}

/* Function makeimagewcs */

func makeimagewcs( imdim, xval, yval, outfile=, weight=, \
       coords=, ra=, dec=, rot=, scale=, crpix1=, crpix2=, clobber= )
/* DOCUMENT image = makeimagewcs( imdim, xval, yval, outfile=, weight= \
       coords=, ra=, dec=, rot=, scale=, crpix1=, crpix2=, clobber= )

  Make 2D image (a 2D histogram) from array of RA and DEC values
    with optional weights. The image is saved if keyword
   'outfile' is given together with the WCS information in FITS format.

   Arguments
      imdim  : A two element integer array with image dimensions
       xval  : An array with coordinates (Right Ascensions)
       yval  : An array with coordinates (Declinations) - must be conformable
               with 'xval'.

   Keywords
    outfile  : If given the image will be dumped in FITS format to this file.
     weight  : A weight applied to all entries in 'xval' and 'yval' (must
               conform to these).
     coords  : An instance of s_Coords to indicate the coordinate system.
               Overrides the other keywords 'ra', 'dec', 'rot', 'scale',
               'crpix1', and 'crpix2' if given.
         ra  : The Right Ascension that corresponds to 'crpix1'
        dec  : The Declination corresponds to 'crpix2'
        rot  : Rotation in degrees.
      scale  : Pixel size in degrees.
     crpix1  : Reference pixel number for coordinate 1 (Right Ascension)
     crpix2  : Reference pixel number for coordinate 2 (Declination)

   2014-02-25/NJW Cloned from 'makeimageu'

*/
{

   index = dimsof(imdim);
   if( index(1) != 1 && index(1) != 2 ) {
      write,"MAKEIMAGEWCS: Incorrect number of dimensions for image";
      return -1.;
   }
   nx = numberof(xval);
   if( nx != numberof(yval) ) {
      print,"MAKEIMAGEWCS: Not matching number of dimensions in call";
      return -1.;
   }

   if( is_void(weight) ) weight = xval*0. + 1.;
   if( numberof(weight) != nx ) {
        print,"MAKEIMAGEWCS: Incorrect number of elements in weight factor array";
        return -1.;
   }

   if( is_void(crpix1) ) crpix1 = (imdim(1)+1)/2.0;
   if( is_void(crpix2) ) crpix2 = (imdim(2)+1)/2.0;

   im = array( float, imdim(1), imdim(2));

   if(is_void(coords)) coords = im2skyfits( imdim, ra=ra, dec=dec, rot=rot, \
           scale=scale, crpix1=crpix1, crpix2=crpix2 );
   local fpix1, fpix2;

   num_outside = 0;
   for( i = 1; i <= nx; i++ ) {
      skypos_fits, coords, fpix1, fpix2, xval(i), yval(i), to_pix=1, silent=1;
      pix1 = long(fpix1+0.5);
      pix2 = long(fpix2+0.5);
      if( pix1 >= 1 && pix1 <= imdim(1) && pix2 >= 1 && pix2 <= imdim(2) ) {
         im(pix1,pix2) += weight(i);
      } else num_outside++;
   }

   if( num_outside ) {
      write,itoa(num_outside)+" fell outside the image.";
   } else write,"No spill over.";

   if( structof(outfile) == string ) {
      kwds_init;
      put_wcs, coords;
      writefits, outfile, im, clobber=clobber;
      write,"File "+outfile+" has been produced.";
   }
   return im;
}

/* Function repair2gezero */

func repair2gezero( im, &imwbox, chat= )
/* DOCUMENT newim = repair2gezero( im, >imwbox, chat= )

     Returns an image where all negative pixel values have
     been removed by reducing the neighboring positive pixel
     values in order to keep the total sum constant.

     The process tries to keep the pixel value changes very local.

     2014-04-23/NJW

*/
{
   if( is_void(chat) ) chat = 0;
   dms = dimsof(im);
   imwork = float(im);
   imwbox = array(int, dms );
   nz = where( im != 0 );
   epsi = min(abs(imwork(nz))) / 100.;

   if( sum(im) <= 0 ) {
      write,"Sum < 0,  beyond repair";
      return im;
   }

   wbox = 3;
   // locate all pixels with values < 0
   w = where( imwork < 0 );
   n = numberof(w);
   while( n ) { // 'n' keeps track of current number of pixels with negative values
      nbox = wbox^2; // statement is put inside loop, since 'wbox' may grow
      k = w(random_i(n)); // choose an index at random
      j1 = j = (k - 1) / dms(2) + 1; // second index in image
      i1 = i = (k - 1) % dms(2) + 1; // first index in image
      if( chat ) write,"n,i,j,wb=", n, i, j, wbox;
      // extract a box around the chosen position. 'i,j' are updated if box is too close
      // to the edge of the image
      box = extract_box( imwork, i, j, wbox, wbox, cen=1 );
      ibox = (wbox+1)/2 - (i - i1); // first index in box of the same pixel
      jbox = (wbox+1)/2 - (j - j1); // second index in box of the same pixel
      value = abs(box(ibox,jbox)); // this is the value that needs to be redistributed

      // can the box absorb this value?
      p = where( box > epsi ); // remember threshold
      np = numberof(p);
      boxsum = np? sum(box(p)) - np*epsi : -1.0; // available for absorption

      if( boxsum < value + epsi ) { // no, cannot be absorbed, need larger box
         wbox += 2;
         if( wbox > min(dms(2:3)) ) error,"repair2gezero failed, Mark 1";
         continue; // skip this one for now
      }

      // redistribute the value plus the minimum
      q = (value + epsi) * (box(p) - epsi) / boxsum;
      box(p) -= q; // "let the broadest shoulders carry the heaviest burden"
      /*
      // begin with testing full box
      nposi = nbox; // start with full box but it is reduced by one to begin with
      do {
         nposi--;
         v = where( box > value/nposi + epsi );
         nv = numberof( v );
      } while( nv < nposi && nposi > 1 );
      if( nv == 0 ) {
         wbox += 2;
         if( wbox > min(dms(2:3)) ) error,"repair2gezero failed, Mark 1";
         continue; // skip this one for now
      }
      */
      box(ibox,jbox) = epsi;
      //+ box(v) -= (value + epsi)/nv;
      imwork = put_box( imwork, box, i, j, cen=1 );
      imwbox(i1,j1) = wbox;
      if( chat > 2 ) {
         disp,imwork;
         pause, 1000;
      }
      w = where( imwork < 0 );
      n = numberof(w);
      //+ wbox -= 4;
      //+ wbox = max(3,wbox);
      wbox = 3;
   }
   return imwork;
}

/* Function imcrop */

func imcrop( dol, outfile=, mul=, lg=, limit=, ilimits=, query=, clobber= )
/* DOCUMENT imcrop, dol, outfile=, mul=, lg=, limit=, ilimits=, query=, clobber=
            newim = imcrop( im, mul=, lg=, limit=, query=, ilimits= )

    First version as subroutine:
    Crop a FITS image to > 'limit' values. Work 'in place' unless 'outfile' is given.
    Uses the ftool 'ftcopy' to reduce the actual size.

    Second version as function:
    Crop an image to  > 'limit' values. Returns cropped image. The indices of the
    retained image is saved into the extern variable 'imcrop_limits' as
      [x1,y1,x2,y2]  : newimage = im( x1:x2, y1:y2 )

    Keywords:
        outfile       if given the new image will be put here,
                      otherwise the input file will be overwritten
            mul       the output dimensions will be multiples of this value, if given
             lg       the image is displayed as "disp,log(im+lg)", if not given then as "disp,im"
          limit       limit for projected value: "where(im(sum,) > limit)"
                                                 "where(im(,sum) > limit)"
                      or, for interaction give "ia"
        ilimits       a 4 element array [x1,y1,x2,y2] when cropping indices are known e.g.
                      from a previous run (overrides 'limit').
        clobber       (bool) if set then the 'outfile' will be overwritten.
          query       (bool) set to T ('1') when user confirmation is wanted

    2014-05-07/NJW

    SEE ALSO: cur_extract_box (for the second version)
*/
{
   extern imcrop_limits;

   if( structof(dol) == string ) { // this must be a DOL
      if( !am_subroutine() ) error,"Called as a function - inconsistency.";

      local filename, extno;

      get_exten_no, dol, filename, extno;
      if( is_void(outfile) ) outfile = filename;
      if( file_test(outfile) && !clobber ) {
         write,outfile+" exists already";
         return;
      }
      im = readfits( dol );
   } else im = dol;

   get_limit = 0;
   if( is_void(ilimits) ) {
      if( is_void(limit) ) limit = 0;
      if( structof(limit) == string ) if( limit == "ia" ) get_limit = 1;
   } else {
      if( numberof(ilimits) != 4 ) error,"'ilimits' has a bad number of elements";
      if( structof(ilimits) != long && structof(ilimits) != int ) error,"'ilimits' is not integer";
   }

   dms = dimsof(im);
   if( is_void(lg) ) disp, im;
   else {
      if(min(im) + lg <= 0 ) {
         write,"Warning! lg is too small to avoid negative values.";
         lg = 1.e-6 - min(im);
         write,format="lg has been adjusted to %13.4e\n", lg;
      }
      disp, log(im+lg);
   }
   pal,"hsv";
   improj = im(,sum);
   if( get_limit ) {
      wd = window();
      window,wd+1;
      if( is_void(lg) ) plot,improj,ps=10,xtitle="X axis";
      else {
         if( lg > 1.e-3 * max(improj) ) lg = 1.e-3 * max(improj);
         plot,improj,itype=1,yr=[lg,1.2*max(improj)],ps=10,xtitle="X-axis";
      }
      r = curmark1(prompt="Mark the limit in window "+itoa(wd+1)+" ...");
      limit = r(2);
      write,format="Limit for X-axis: %13.4e\n", limit;
   }
   if( is_void(ilimits) ) {
      w = where(improj > limit);
      x1 = w(1);
      x2 = w(0);
      if( mul ) x2 = ((x2-x1+1)/mul)*mul-1+x1;
      improj = im(sum,);
      if( get_limit ) {
         window,wd+2;
         if( is_void(lg) ) plot,improj,ps=10,xtitle="Y axis";
         else {
            if( lg > 1.e-3 * max(improj) ) lg = 1.e-3 * max(improj);
            plot,improj,itype=1,yr=[lg,1.2*max(improj)],ps=10,xtitle="Y-axis";
         }
         r = curmark1(prompt="Mark the limit in window "+itoa(wd+2)+" ...");
         limit = r(2);
         write,format="Limit for Y-axis: %13.4e\n", limit;
         window,wd; // reset to original value
      }
      w = where(improj > limit);
      y1 = w(1);
      y2 = w(0);
      if( mul ) y2 = ((y2-y1+1)/mul)*mul-1+y1;
   } else {
      x1 = ilimits(1);
      x2 = ilimits(3);
      y1 = ilimits(2);
      y2 = ilimits(4);
   }
   oplot,[x1,x1],[1,dms(3)],color="white";
   oplot,[x2,x2],[1,dms(3)],color="white";
   oplot,[1,dms(2)],[y1,y1],color="white";
   oplot,[1,dms(2)],[y2,y2],color="white";

   if( query ) ans = strlowcase(rdline(prompt="Is this acceptable ? ... "));
   else ans = "y";

   if( ans == "y" ) {
      imcrop_limits = [x1,y1,x2,y2];
      if( am_subroutine() ) { // do the file writing ...
         remove,"x.fits";
         write,"ftcopy '"+filename+"["+itoa(x1)+":"+itoa(x2)+","+itoa(y1)+":"+itoa(y2)+"]' x.fits";
         system,"ftcopy '"+filename+"["+itoa(x1)+":"+itoa(x2)+","+itoa(y1)+":"+itoa(y2)+"]' x.fits";
         stream = open("x.hdr","w");
         write,stream,format="ORIGFILE='%s' / Original file\n", filename;
         write,stream,format="X1CROP  = %i / Lower limit of inclusion in X\n", x1;
         write,stream,format="X2CROP  = %i / Upper limit of inclusion in X\n", x2;
         write,stream,format="Y1CROP  = %i / Lower limit of inclusion in Y\n", y1;
         write,stream,format="Y2CROP  = %i / Upper limit of inclusion in Y\n", y2;
         close, stream;
         system,"fmodhead x.fits+"+itoa(extno)+" x.hdr";
         system,"mv x.fits "+outfile;
      } else { // return the cropped image
         return im(x1:x2,y1:y2);
      }
   } else write,"Action was cancelled.";
}

/* Function contrast */

func contrast( im, i, j )
/* DOCUMENT col = contrast( im, i, j )

    Returns the contrast color at position (i,j) when image
    'im' has been plotted in the current window.

    'col' is a 3-element vector of type 'int'.

    Arguments:
         im    a 2D array with displayed image
          i    index of first axis
          j    index of second axis

   2015-02-06/NJW
*/
{
   local r,g,b;
   palette,r,g,b,query=1;
   v = im(i,j);
   vmax = max(im);
   vmin = min(im);
   n = numberof(r);
   k = long((double(v - vmin) / (vmax - vmin)) * (n-1) + 1.5)
   //+ write,"n = ", n,"  k = ",k;
   //+ write,"red: "+itoa(int(r(k)))+", green: "+itoa(int(g(k)))+", blue: "+itoa(int(b(k)));
   return int(255*[1,1,1]) - int([r(k),g(k),b(k)]);
}

/* Function rotate_image */

/**************************************
  Function to return a rotated image

    Lower left pixel has center coordinates (1.,1.)

    2012-07-15/NJW

**************************************/

func rotate_image( im, angle, xc, yc, prec= )
/* DOCUMENT newim = rotate_image( im, angle, xc, yc, prec= )

      Angle in deg.

      Keyword 'prec' is precision i.e. number of subdivisions.
*/
{
   im = double(im);
   dms = dimsof( im );
   out = im*0.;
   if( is_void(prec) ) prec = 2;

   ca = cos(angle*pi/180.);
   sa = sin(angle*pi/180.);
   omega = [[ca,sa],[-sa,ca]];

   dp = 1./(2*prec);

   for( i = 1; i <= dms(2); i++ ) {
      for( j = 1; j <= dms(3); j++ ) {
         frac = im(i,j) / prec^2;
         for( k = 1; k <= prec; k++ ) {
            for( l = 1; l <= prec; l++ ) {
               subpixpos = [-0.5+dp+2*dp*(k-1+random()-0.5),-0.5+dp+2*dp*(l-1+random()-0.5)];
               pos = omega(,+)*([i,j]+subpixpos-[xc,yc])(+) + [xc,yc];
               ip = long(floor(pos(1)+0.5));
               jp = long(floor(pos(2)+0.5));
               if(ip > 0 && ip <= dms(2) && jp > 0 && jp <= dms(3) ) out(ip,jp) += frac;
            }
         }
      }
   }
   return out;
}

/* Function squarify */

func squarify( im )
/* DOCUMENT new_im = squarify( im )

      Return a square image with original image filled in. The side of the
      square equals the largest dimension of 'im'.

      The filling is done with zeros.

    2015-11-18/NJW

*/
{
   dms = dimsof( im );
   if( dms(2) == dms(3) ) return im; // no action if already square

   if( dms(2) > dms(3) ) {
      im_out = array(structof(im),dms(2),dms(2));
      sh = (dms(2) - dms(3) + 3)/2;
      im_out(,sh:sh+dms(3)-1) = im;
   } else {
      im_out = array(structof(im),dms(3),dms(3));
      sh = (dms(3) - dms(2) + 3)/2;
      im_out(sh:sh+dms(2)-1,) = im;
   }
   return im_out;
}

/* Function flex_image */

func flex_image( im_in, xdim=, ydim=  )
/* DOCUMENT new_image = flex_image( im_in, xdim=, ydim=  )

     Returns an expanded or shrunk image keeping image levels constant.

     Argument:      im_in    A 2D image

     Keywords:
         xdim       new dimension of the first index
         ydim       new dimension of the second index

     SEE ALSO :  rebin, remap, members of 'image.i'

      2015-11-17/NJW
*/
{
  /*
   *   Expanding so that the level is the same in the new image as in the old image
   *   implying that the sum is not constant.
   */

   dms = dimsof(im_in);

   if( !is_void(xdim) ) { // flex in x direction

      j_in = indgen(dms(2));
      im_out = array( structof(im_in), xdim, dms(3) );

      f = double( dms(2) ) / xdim;

      for( i = 1; i <= dms(3); i++ ) { // loop over y direction
         for( jh = 1; jh <= xdim; jh++ ) {
            k = where( j_in < f*jh + 1 & j_in > f*(jh-1) );
            nk = numberof(k);
            rsum = 0.0;
            for( l = 1; l <= nk; l++ ) {
               // compute fractional overlap
               fyh1 = f*(jh-1);
               fyh2 = f*jh;
               y1 = k(l)-1;
               y2 = k(l);
               // we must have fyh2 > y1
               if( fyh2 < y2 ) { // case A
                  if( fyh1 < y1 ) { // case Aa, fractional filling
                     rsum += im_in(k(l),i)*(fyh2 - y1)/f;
                  } else {          // case Ab, new pixel completely inside old one
                     rsum += im_in(k(l),i);
                  }
               } else {          // case B
                  if( fyh1 < y1 ) { // case Ba, old pixel completely inside new one
                     rsum += im_in(k(l),i)/f;
                  } else {          // case Bb, fractional filling
                     rsum += im_in(k(l),i)*(y2 - fyh1)/f;
                  }
               }
            }
            im_out( jh, i ) = rsum;
         }
      }
      im_in = im_out;
      dms(2) = xdim;
   }



   if( !is_void(ydim) ) { // flex in y direction

      j_in = indgen(dms(3));
      im_out = array( structof(im_in), dms(2), ydim );

      f = double( dms(3) ) / ydim;

      for( i = 1; i <= dms(2); i++ ) { // loop over x direction
         for( jh = 1; jh <= ydim; jh++ ) {
            k = where( j_in < f*jh + 1 & j_in > f*(jh-1) );
            nk = numberof(k);
            rsum = 0.0;
            for( l = 1; l <= nk; l++ ) {
               // compute fractional overlap
               fyh1 = f*(jh-1);
               fyh2 = f*jh;
               y1 = k(l)-1;
               y2 = k(l);
               // we must have fyh2 > y1
               if( fyh2 < y2 ) { // case A
                  if( fyh1 < y1 ) { // case Aa, fractional filling
                     rsum += im_in(i,k(l))*(fyh2 - y1)/f;
                  } else {          // case Ab, new pixel completely inside old one
                     rsum += im_in(i,k(l));
                  }
               } else {          // case B
                  if( fyh1 < y1 ) { // case Ba, old pixel completely inside new one
                     rsum += im_in(i,k(l))/f;
                  } else {          // case Bb, fractional filling
                     rsum += im_in(i,k(l))*(y2 - fyh1)/f;
                  }
               }
            }
            im_out( i, jh ) = rsum;
         }
      }
   }

   return im_out;

}
 /* Function distribute_bkg */

func distribute_bkg( im, level )
/* DOCUMENT newim = distribute_bkg( im, level )

     Returns an image updated with a randomly distributed background
     with 'level' counts per pixel.

*/
{
   dms = dimsof(im);

   npixels = numberof(im);
   ncounts = long(level * npixels + 0.5);
   x = int(random(ncounts) * dms(2) + 1);
   y = int(random(ncounts) * dms(3) + 1);
   newim = im;
   for(i=1;i<=ncounts;i++ ) newim(x(i),y(i))++;

   return newim;
}
