/* Function plotdoc */

extern plotdoc;
/* DOCUMENT ******************************************
 *
 *     A package of extra plot functions
 *
 *      2004-11-11/NJW
 *

       add_hgraph                 owplot                     
       add_vgraph                 plh                        
       annot                      plhis                      
       curtext                    plhisc                     
       curve_edit                 plhisv                     
       d3plot                     plot                       
       dataplot                   plot_p2spec                
       dataplotf                  plot_spectrum              
       def_sym                    plotcomments               
       devicetitles               plotdate                   
       draw_arrow                 plotname                   
       esc_uscore                 plotsign                   
       mcoord_conv                plthis                     
       mlogxy                     poly_fill                  
       mplot_setup                poly_fillc                 
       odataplot                  wr_qdp                        
       odataplotf                 wplot                      
       oplot                      xyouts                     
       oplot_spectrum             zps                        
       oplthis                    

   extern Logxy_flags_content, Logxy_flags_values;
                          // used in 'mlogxy' (to be used in stead of logxy)
   extern _Annot_text, _Annot_ps, _Annot_li, _Annot_symsize;
			  // used in 'annot'
   extern Def_sym_size_x, Def_sym_size_y, Def_sym_number, Def_sym_x, Def_sym_y;
			  // used in 'plot' and 'def_sym'
   extern PLOTCOMMENTS_YLEV; // array with 8 elements; used in 'plotcomments'

   Defines red, blue, green, black, and dtured

*******************************************************/

extern itype;
/* DOCUMENT   itype  Flag for axis transforms: 0(linlin), 1(linlog), 2(loglin), 3(loglog)
*/

// predefined color-strings:

red = "red";
blue = "blue";
green = "green";
black = "black";
cyan = "cyan";
magenta = "magenta";
yellow = "yellow";
dtured = [200,50,30];


/* Function annot */

func annot( text, mode=, ps=, li=, thick=, symsize=, color=, fill=, pos=, height= )
/* DOCUMENT annot, text, mode=, ps=, li=, thick=, symsize=, color=, fill=, pos=, height=

   Make annotations in the plot in a similar way as 'legend' in IDL.
   See example below.

   mode "i"  initialize
   mode "u" or "a" update (not required if the 'text' argument is given as a string)
   mode "x"  execute i.e. put on plot (not required if n 'text' and no 'mode="i"')

   'height' must be one of 8, 10, 12, 14, 16, 18, 24, or 36
     and represents the character height in points; 1 point = 0.0013 NDC units

   'pos' is a two-element array with (x,y) of the lower left corner of
    the annotation box in virtual coordinates i.e. where the viewing port
    is between (0,0) and (1,1).

   The keywords 'pos' and 'height' are only active in "execute" mode.

   Each time "annot" is invoked in update mode the text string is added to
   a buffer that is shown on the plot when invoked in "execute" mode.

   Example:
      > annot,mode="i"; // initialization
      > annot,"First curve",li=2,color="green"; // updates only
      > annot,"Points",ps=3,color="blue"; // updates only
      > annot,pos=[0.5,0.0],height=24;     // puts the annotations on the plot

   2008-08-15/NJW
   2008-10-24/NJW allow for composite symbols
   2012-06-12/NJW Use [r,g,b] representation of color

*/
{
   extern _Annot_text, _Annot_ps, _Annot_li, _Annot_symsize, \
          _Annot_thick, _Annot_color, _Annot_fill, _Annot_id;

   // _Annot_color is an integer matrix with dimension 3 x <number of colors>

   local x_wor, y_wor;  // result from mcoord_conv
   local x1_wor, x2_wor;  // result from mcoord_conv
   local px_ndc, py_ndc;  // result from mcoord_conv

   if( !is_void(mode) ) {
      if( mode != "i" && mode != "u" && mode != "a" && mode != "x" ) {
         write,"ANNOT: Illegal 'mode' keyword";
         return;
      }
   }

   if( is_void(text) ) { // -- 'text' is absent, only initialization or execution
      if( is_void(mode) ) {
         mode = "x"; // no arguments given, will execute
      } else {
         if( mode == "u" || mode == "a" ) {
            write,"ANNOT: Illegal arguments";
            return;
         }
      }
   } else { // -----  here 'text' contains something
      if( is_void(mode) ) {
         mode = "u"; //     just update
      }
   }

   if( is_void(height) ) {
      height = 14;
   } else {
      h = height;
      height = 36;
      if( h <= 30.0 ) height = 24;
      if( h <= 21.0 ) height = 18;
      if( h <= 17.0 ) height = 16;
      if( h <= 15.0 ) height = 14;
      if( h <= 13.0 ) height = 12;
      if( h <= 11.0 ) height = 10;
      if( h <=  9.0 ) height =  8;
   }
   // Compute character height in NDC units but
   // add 10% for separation of characters
   h_charsize = 1.1 * height * 0.0013;
   //+ x_text_offset = 0.1; // **
   x_text_offset = 0.06
   x_symbol_offset = 0.5 * x_text_offset;
   y_symbol_offset = 0.3 * h_charsize;
   y_line_offset =   0.3 * h_charsize;
   x1_line_offset =   0.1 * x_text_offset;
   x2_line_offset =   0.8 * x_text_offset;

   if( mode == "i" ) {
      _Annot_text = _Annot_ps = _Annot_li = _Annot_id = [];
      _Annot_symsize = _Annot_color = _Annot_fill = _Annot_thick = [];
   } else if( mode == "u" || mode == "a" ) {
      // find next identifier
      n_id = numberof(_Annot_id);
      if( n_id ) {
         next_id = _Annot_id(0) + 1; // 1 larger than latest
      } else next_id = 1;
      // find the non-void keywords and define multiplicity
      n_ps =  is_void(ps) ? 0 : numberof(ps);
      n_li =  is_void(li) ? 0 : numberof(li);
      n_thick =  is_void(thick) ? 0 : numberof(thick);
      n_symsize =  is_void(symsize) ? 0 : numberof(symsize);

      // Special handling of color

      if( is_void(color) ) {
         color = reform([0,0,0],3,1);
      } else {
         if( typeof(color) == "string" ) {
            color = resolve_color( color );
         }
         if( dimsof(color)(1) < 2 ) color = reform(color,3,1);
      }
      n_color =  dimsof(color)(3);
      multi = max([n_ps,n_li,n_thick,n_symsize,n_color]);
      if( n_ps > 0 && n_ps != multi ) error,"Bad ps multiplicity";
      if( n_li > 0 && n_li != multi ) error,"Bad li multiplicity";
      if( n_thick > 0 && n_thick != multi ) error,"Bad thick multiplicity";
      if( n_symsize > 0 && n_symsize != multi ) error,"Bad symsize multiplicity";
      if( n_color > 0 && n_color != multi ) error,"Bad color multiplicity";

      if( is_void(ps) ) ps = array(-1,multi);
      if( is_void(li) ) li = array(-1,multi);
      thick =  is_void(thick) ? array(-1.0,multi) : double(thick);
      symsize = is_void(symsize) ? array(1.0,multi) : double(symsize);
      if( is_void(fill) ) fill = array(0,multi);

      grow, _Annot_text, array(text,multi);
      grow, _Annot_ps, ps;
      grow, _Annot_li, li;
      grow, _Annot_thick, thick;
      grow, _Annot_symsize, symsize;
      grow, _Annot_color, color;
      grow, _Annot_fill, fill;
      grow, _Annot_id, array(next_id, multi);

   } else if( mode == "x" ) { // Put annotations on the plot

      // Check if there are any

      if( numberof(_Annot_id) ) {

         // calculate default annotation origin
         mcoord_conv, 0.0, 0.0, px_ndc, py_ndc, from="vir", to="ndc";
         if( !is_void(pos) ) {
            if( numberof(pos) != 2 ) {
               write,"'pos' has illegal dimensionality, disregard";
               pos = [];
            }
            mcoord_conv, pos(1), pos(2), px_ndc, py_ndc,from="vir",to="ndc";
         }

         u_id = uniq(_Annot_id);
         //+ n = numberof(_Annot_text);
         n = _Annot_id(0);
         for(i=1;i<=n;i++) {
            // determine multiplicity
            multi = i < n ? u_id(i+1)-u_id(i) : numberof(_Annot_id)-u_id(i)+1;
            j = u_id(i);
            //  Put the text on the plot
            y_ndc = (n-i+1)*h_charsize + py_ndc;
            x_ndc = x_text_offset + px_ndc;
            //+ print,"text xy ndc:", x_ndc, y_ndc;
            plt,_Annot_text(j),x_ndc,y_ndc,tosys=0,height=height;
            //  Add a symbol?
            for( k = j; k < j+multi; k++ ) {
               if( _Annot_ps(k) > -1 ) {
                  x_ndc = x_symbol_offset + px_ndc;
                  y_ndc = (n-i+1)*h_charsize + py_ndc + y_symbol_offset;
                  //+ print,"symbol xy ndc:",x_ndc, y_ndc;
                  mcoord_conv, x_ndc, y_ndc, x_wor, y_wor, from="ndc", to="wor";
                  //+ print,"symbol xy wor:", x_wor, y_wor;
                  clr = _Annot_color(,k);
                  fll = _Annot_fill(k) == 0 ? [] : _Annot_fill(k);
                  oplot,[x_wor],[y_wor],ps=_Annot_ps(k),symsize=_Annot_symsize(k),thick=_Annot_thick(k),color=clr,fill=fll;
               }
               //  Add a line?
               if( _Annot_li(k) > -1 ) {
                  x1_ndc = x1_line_offset + px_ndc;
                  x2_ndc = x2_line_offset + px_ndc;
                  y_ndc = (n-i+1)*h_charsize + py_ndc + y_line_offset;
                  //+ print,"line xy ndc:",x1_ndc, x2_ndc, y_ndc;
                  mcoord_conv, x1_ndc, y_ndc, x1_wor, y_wor, from="ndc", to="wor";
                  mcoord_conv, x2_ndc, y_ndc, x2_wor, y_wor, from="ndc", to="wor";
                  //+ print,"line xy wor:", x1_wor, x2_wor, y_wor;
                  clr = _Annot_color(,k);
                  thk = _Annot_thick(k) < 0 ? [] : _Annot_thick(k);
                  oplot,[x1_wor,x2_wor],[y_wor,y_wor],li=_Annot_li(k),thick=thk,color=clr;
               }
            }
         }
      }
   } // terminates 'put annotations on plot'

}

/* Function plot */

func plot( x, y, li=, ps=, color=, xr=, yr=, symsize=, \
              itype=, thick=, title=, xtitle=, ytitle=, mpl=, dbg=, \
              fill=, titleheight=, xytitleheight=, xyadjust=, titlefont= )
/* DOCUMENT plot, x, y, li=, ps=, color=, xr=, yr=,\
              itype=, symsize=, thick=, title=, xtitle=, ytitle=, mpl=, \
              dbg=, fill=, titleheight=, xytitleheight=, xyadjust=, titlefont=

   A function similar to the IDL 'plot' function. The Yorick
   'fma' call is included so that the plot is renewed. For
   overplot use 'oplot'.
   If only one argument is given then it is interpreted as 'y'
   and the x will be indgen(numberof(y)).

   KEYWORDS: li, ps, color, xr, yr, symsize, thick, title,
             xtitle, ytitle, itype, mpl, dbg, fill,
             titleheight, xytitleheight, xyadjust, titlefont

             li             Linetype
             ps             Plot symbol
             color          Color
             xr             Range in X (two element array)
             yr             Range in Y (two element array)
             symsize        Symbol size
             thick          Line thickness
             title          Plot title
             xtitle         Title for abscissa
             ytitle         Title for ordinate
             itype          Flag for axis transforms: 0(linlin), 1(linlog), 2(loglin), 3(loglog)
             mpl            Number of subframe
             dbg            Switch on debugging output
             fill           Flag for symbol filling whenever possible
             titleheight    Font size for plot title
             xytitleheight  Font size for axis titles
             xyadjust
             titlefont      Font for titles

   SEE ALSO: oplot, plg, mplot_setup, dataplot, dataplotf

   2007-02-28/NJW update with itype keyword
   2010-10-03/NJW update with fill keyword
*/

{

   extern Def_sym_size_x, Def_sym_size_y, Def_sym_number, Def_sym_x, Def_sym_y;
   extern pltitle_height, pltitle_font;

   // Save plot title externals
   Pltitle_height = pltitle_height;
   Pltitle_font = pltitle_font;

   // Check arguments

   if( is_void(y) ) {
      y = x;
      x = indgen(numberof(y));
   }
   // dbg
   if( dbg ) {
      if( is_void(mpl) ) {
         write,"PLOT called with void 'mpl'";
      } else {
         write,format="PLOT called with mpl = %i\n", mpl;
         write,format="PLOT ##2## plsys = %i\n", plsys();
      }
   }

   if( numberof(mpl) ) {
      // dbg
      if( dbg ) write,"PLOT calling plsys";
      plsys,mpl;
      if( dbg ) write,format="PLOT ##3## plsys = %i\n", plsys();
   } else {
      // dbg
      if( dbg ) write,"PLOT resetting frame";
      fma; // resets the frame i.e. current plotting window
   }

   // Define the plot type (lin-lin, lin-log, log-lin, log-log)

   it = is_void(itype) ? 0: itype;
   xlog = it > 1 ? 1 : 0;
   ylog = it % 2 ? 1 : 0;
   mlogxy, xlog, ylog, mpl=mpl,dbg=dbg;


   limits; // restarts axis range limits
   // dbg
   if( dbg ) write,format="PLOT ##4## plsys = %i\n", plsys();
   if( numberof(xr) ) {
      limits,xr(1),xr(2);
   } else {
      if( xlog ) {
         difference = log(max(x) / min(x));
         mx = max(x) * exp(0.05*difference);
         mn = min(x) / exp(0.05*difference);
      } else {
         difference = max(x) - min(x);
         mx = max(x) + 0.05*difference;
         mn = min(x) - 0.05*difference;
      }
      limits, mn, mx;
   }
   if( numberof(yr) ) {
      limits,,,yr(1),yr(2);
   } else {
      if( ylog )  {
         w = where(y > 0.0);
         difference = log(max(y(w)) / min(y(w)));
         mx = max(y) * exp(0.05*difference);
         mn = min(y) / exp(0.05*difference);
      } else {
         difference = max(y) - min(y);
         mx = max(y) + 0.05*difference;
         mn = min(y) - 0.05*difference;
      }
      limits,,, mn, mx;
   }
   // dbg
   if( dbg ) write,format="PLOT ##5## plsys = %i\n", plsys();

   if( ps ) {
      if( ps == 10 ) {
         plhis, y, x, marks=0, type=li, color=color, width=thick;
      } else if( ps == 100 ) {
         plhisv, y, x, marks=0, type=li, color=color, width=thick;
      } else if( abs(ps) > 10 ) {
         if( ps < 0 ) {
            // plot the curve
            plg, y, x, marks=0, type=li, color=color, msize=symsize, \
                   legend="", width=thick;
            ps = -ps;
         }
         // plot individual points
         wcoor = limits();
         if( is_void(symsize) ) symsize = 1.0;
         if( xlog ) {
            Def_sym_size_x = log(wcoor(2) / wcoor(1))*0.05*symsize;
         } else {
            Def_sym_size_x = (wcoor(2) - wcoor(1))*0.05*symsize;
         }
         if( ylog ) {
            Def_sym_size_y = log(wcoor(4) / wcoor(3))*0.05*symsize;
         } else {
            Def_sym_size_y = (wcoor(4) - wcoor(3))*0.05*symsize;
         }

         // Define the external arrays Def_sym_x/y
         def_sym, ps;
         possible_fills = [13,19,20,21,22,23,24,25,26,27,28,29];

         for(i=1;i<=numberof(x);i++) {
            if( xlog ) {
               xarr = x(i)*exp(Def_sym_x);
            } else {
               xarr = x(i)+Def_sym_x;
            }
            if( ylog ) {
               yarr = y(i)*exp(Def_sym_y);
            } else {
               yarr = y(i)+Def_sym_y;
            }
            dmsx = dimsof(xarr);
            if( dmsx(1) == 2 ) { // When the symbol consists of more than a single
                                 // polygon or line e.g. a cross
               for(j=1;j<=dmsx(3);j++) {
                  plg, yarr(,j), xarr(,j), marks=0, type=1, color=color, \
                    legend="",width=thick;
               }
            } else if( fill && nallof(ps - possible_fills) ) {
               poly_fillc, xarr, yarr, color=color;
            } else {
               plg, yarr, xarr, marks=0, type=1, color=color, \
                 legend="",width=thick;
            }
     /* Some of the symbols can be plotted as filled:
      *       
      *   13 - Circle                  23 - Star 6
      *   19 - Square                  24 - Star 7
      *   20 - Triangle on bottom      25 - Star 6
      *   21 - Triangle on tip         26 - Star 9
      *   22 - Star                    27 - Diamond
      */
         }
      } else {
         if( ps < 0 ) {
            // plot the curve
            plg, y, x, marks=0, type=li, color=color, msize=symsize, \
                   legend="", width=thick;
            ps = -ps;
         }
         // plot individual points
         plg, y, x, marks=0, type=0, marker=ps, color=color, \
                   legend="", msize=symsize, width=thick;
      }
   } else {
      plg, y, x, marks=0, type=li, color=color, msize=symsize, \
                   legend="", width=thick;
   }
   if( numberof(title) ) {
      if( !is_void(titleheight) ) pltitle_height = titleheight;
      if( !is_void(titlefont) ) pltitle_font = titlefont;
      pltitle,title;
   }
   if( is_void(xtitle) ) xtitle = "";
   if( is_void(ytitle) ) ytitle = "";
   if( !is_void(xytitleheight) ) pltitle_height = xytitleheight;
   if( !is_void(titlefont) ) pltitle_font = titlefont;
   xytitles,xtitle,ytitle,xyadjust;

   plotcomments, init=1; // initializes the text position on the page
       // by external PLOTCOMMENTS_YLEV

   // Reset default title height and font

   pltitle_font = Pltitle_font;
   pltitle_height = Pltitle_height;

}

/* Function oplot */

func oplot( x, y, li=, ps=, color=, symsize= ,thick=, fill=, mpl= )
/* DOCUMENT oplot, x, y, li=, ps=, color=, symsize= ,thick=, fill=, mpl=

   A function similar to the IDL 'oplot' function.
   For new plot use 'plot'.

   If only one argument is given then it is interpreted as 'y'
   and the x will be indgen(numberof(y)).

   KEYWORDS: li, ps, color, symsize, thick, fill, mpl
   SEE ALSO: plot, plg
*/

{

   extern Def_sym_size_x, Def_sym_size_y, Def_sym_number, Def_sym_x, Def_sym_y;

   // Check arguments

   if( is_void(y) ) {
      y = x;
      x = indgen(numberof(y));
   }

   if( !is_void(mpl) ) plsys, mpl;

   if( ps ) {
      if( ps == 10 ) {
         plhis, y, x, marks=0, type=li, color=color, width=thick;
      } else if( ps == 100 ) {
         plhisv, y, x, marks=0, type=li, color=color, width=thick;
      } else if( abs(ps) > 10 ) {
         if( ps < 0 ) {
            // plot the curve
            plg, y, x, marks=0, type=li, color=color, msize=symsize, \
                   legend="", width=thick;
            ps = -ps;
         }
         // Get log plotting flags

         local xlog, ylog;
         mlogxy, xlog, ylog, get=1;

         // plot individual points
         wcoor = limits();
         if( is_void(symsize) ) symsize = 1.0;
         if( xlog ) {
            Def_sym_size_x = log(wcoor(2) / wcoor(1))*0.05*symsize;
         } else {
            Def_sym_size_x = (wcoor(2) - wcoor(1))*0.05*symsize;
         }
         if( ylog ) {
            Def_sym_size_y = log(wcoor(4) / wcoor(3))*0.05*symsize;
         } else {
            Def_sym_size_y = (wcoor(4) - wcoor(3))*0.05*symsize;
         }
         // Define plotting symbol external variables: Def_sym_x/y
         def_sym, ps;
         possible_fills = [13,19,20,21,22,23,24,25,26,27,28,29];

         for(i=1;i<=numberof(x);i++) {
            if( xlog ) {
               xarr = x(i)*exp(Def_sym_x);
            } else {
               xarr = x(i)+Def_sym_x;
            }
            if( ylog ) {
               yarr = y(i)*exp(Def_sym_y);
            } else {
               yarr = y(i)+Def_sym_y;
            }
            dmsx = dimsof(xarr);
            if( dmsx(1) == 2 ) {
               for(j=1;j<=dmsx(3);j++) {
                  plg, yarr(,j), xarr(,j), marks=0, type=1, color=color, \
                    legend="",width=thick;
               }
            } else if( fill && nallof(ps - possible_fills) ) {
               poly_fillc, xarr, yarr, color=color;
     /* Some of the symbols can be plotted as filled:
      *       
      *   13 - Circle                  23 - Star 6
      *   19 - Square                  24 - Star 7
      *   20 - Triangle on bottom      25 - Star 6
      *   21 - Triangle on tip         26 - Star 9
      *   22 - Star                    27 - Diamond
      */
            } else {
               plg, yarr, xarr, marks=0, type=1, color=color, \
                 legend="",width=thick;
            }
         }
      } else {
         if( ps < 0 ) {
            plg, y, x, marks=0, type=li, color=color, msize=symsize, \
                legend="", width=thick;
            ps = -ps;
         }
         plg, y, x, marks=0, type=0, marker=ps, color=color, \
                   legend="", msize=symsize, width=thick;
      }
   } else {
      plg, y, x, marks=0, type=li, color=color, msize=symsize, \
                   legend="", width=thick;
   }
}

/* Function plh  */

func plh( y, x, marks=, color=, type=, width=)
/* DOCUMENT plh, y, x
   plot a histogram of Y versus X. If X has one more element
   than Y, the plot will begin and end with a horizontal segment;
   if Y has one more element than X, the plot will begin and and
   with a vertical segment. The keywords are a subset of those for
   plg.
   KEYWORDS: marks, color, type, width
   SEE ALSO: plg, plhis, plhisc, plhisv, plthis
*/
{
   //  Define x array if not given
   if( numberof(x) == 0 ) {
      x = span(0.0,1.0*numberof(y),numberof(y)+1);
   }

   swap = numberof(x)<numberof(y);
   if( swap ) { yy = y; y = x; x = yy; }
   yy = xx = array(0.0,2*min(numberof(y),numberof(x)));
   yy(1:-1:2) = yy(2:0:2) = y;
   xx(2:-2:2) = xx(3:-1:2) = x(2:-1);
   xx(1) = x(1);
   xx(0) = x(0);
   if( swap ) { y = yy; yy = xx; xx = y; }
   plg, yy, xx, marks=marks, color=color, type=type, \
                legend="", width=width;
}

/* Function plhis  */

func plhis( y, x, marks=, color=, type=, width=)
/* DOCUMENT plhis, y, x, marks=, color=, width=

   Plot a histogram where x and y have the same number of
   elements

   KEYWORDS: marks, color, type, width
   SEE ALSO: plh, plg, plhisc, plhisv, plthis, [plot,..,ps=10]
*/

{
   require,"idlx.i";

   if( numberof(x) == 0 ) x = span(0.5,0.5+numberof(y)-1,numberof(y));
   yy = xx = array(double,2*numberof(y));
   yy(1:-1:2) = yy(2:0:2) = y;
   z = shift(x,1);
   xx(2:-2:2) = xx(3:-1:2) = 0.5*(x(1:-1)+z(1:-1));
   xx(1) = x(1);
   xx(0) = x(0);
   plg,yy,xx,marks=marks, color=color, type=type, \
                legend="", width=width;
}

/* Function plthis */

func plthis( x1, x2, y, li=, color=, xr=, yr=, \
              itype=, thick=, title=, xtitle=, ytitle=, mpl=, dbg=, \
              titleheight=, xytitleheight=, xyadjust=, titlefont= )
/* DOCUMENT plthis, x1, x2, y
    or      plthis, x, y  {in this version x may have one more element than y}

     Keywords: li=, color=, xr=, yr=,
               itype=, thick=, title=, xtitle=, ytitle=, mpl=, dbg=,
               titleheight=, xytitleheight=, xyadjust=, titlefont=

     Plot a histogram where x1,x2 give the channel boundaries

     Alternatively: x has one more element that y so that
         x1 = x(1:-1) & x2 = x(2:)
     Alternatively: x same number of elements as y so that
         x1 = [x(1),x(zcen)] & x2 = [x(zcen),x(0)]

    SEE ALSO: plh, plg, plhis, plhisc, plhisv, [plot,..,ps=10]
*/
{
   nx1 = numberof(x1);
   if( is_void(y) ) { // can it be the alternative version?
      if( numberof(x2) + 1 == nx1 ) { // fine, go ahead
         x = x1;
         y = x2;
         x1 = x(1:-1);
         x2 = x(2:0);
         nx1--;
      } else if( numberof(x2) == nx1 ) { // also fine, go ahead
         x = x1;
         y = x2;
         x1 = grow(x(1),x(zcen));
         x2 = grow(x(zcen),x(0));
      }
   }

   if( nx1 != numberof(x2) ) error,"Mismatching array size x1 x2";
   if( nx1 != numberof(y) ) error,"Mismatching array size x1 y";

   yout = y(indgen(2:2*nx1+1)/2);
   xout = grow(x1(1),x2(indgen(2:2*nx1)/2));

   plot, xout, yout, li=li, color=color, xr=xr, yr=yr, \
              itype=itype, thick=thick, title=title, xtitle=xtitle, \
              ytitle=ytitle, mpl=mpl, dbg=dbg, \
              titleheight=titleheight, xytitleheight=xytitleheight, \
              xyadjust=xyadjust, titlefont=titlefont;

}
  
/* Function oplthis */

func oplthis( x1, x2, y, li=, color=, thick= )
/* DOCUMENT oplthis, x1, x2, y, li=, color=, thick=
     or      oplthis, x, y     {x may have one more element than y}

     Overplot a histogram where x1,x2, give the channel boundaries

     Alternatively: x has one more element that y so that
         x1 = x(1:-1) & x2 = x(2:)
     Alternatively: x same number of elements as y so that
         x1 = [x(1),x(zcen)] & x2 = [x(zcen),x(0)]
*/
{
   nx1 = numberof(x1);
   if( is_void(y) ) { // can it be the alternative version?
      if( numberof(x2) + 1 == nx1 ) { // fine, go ahead
         x = x1;
         y = x2;
         x1 = x(1:-1);
         x2 = x(2:0);
         nx1--;
      } else if( numberof(x2) == nx1 ) { // also fine, go ahead
         x = x1;
         y = x2;
         x1 = grow(x(1),x(zcen));
         x2 = grow(x(zcen),x(0));
      }
   }
   if( nx1 != numberof(x2) ) error,"Mismatching array size x1 x2";
   if( nx1 != numberof(y) ) error,"Mismatching array size x1 y";

   yout = y(indgen(2:2*nx1+1)/2);
   xout = grow(x1(1),x2(indgen(2:2*nx1)/2));

   oplot, xout, yout, li=li, color=color, thick=thick;

}

/* Function plhisv  */

func plhisv( y, x, marks=, color=, type=, width= )
/* DOCUMENT plhisv, y, x, marks=, color=, width=

   Plot a 'vertical' histogram where x and y have the same number of
   elements

   KEYWORDS: marks, color, type, width
   SEE ALSO: plh, plg, plhis
*/

{
   if( numberof(x) == 0 ) x = span(0.5,0.5+numberof(y)-1,numberof(y));
   yy = xx = array(double,2*numberof(y));
   yy(1:-1:2) = yy(2:0:2) = y;
   z = shift(x,1);
   xx(2:-2:2) = xx(3:-1:2) = 0.5*(x(1:-1)+z(1:-1));
   xx(1) = x(1);
   xx(0) = x(0);
   plg,xx,yy,marks=marks, color=color, type=type, \
                legend="", width=width;
}

/* Function plhiscv  */

func plhiscv( y, x, color=, y0= )
/* DOCUMENT plhiscv, y, x, color=, y0=

   Plot a 'vertical' histogram with color-filled columns
   where x and y have the same number of
   elements, just like 'plhisc' but on the vertical axis.

   'x' is still the axis values just plotted along the y axis.
   'y' are the amplitude values.

   KEYWORDS: color

   SEE ALSO: plh, plg, plhis, plhisc
*/

{
   local x1, x2;
   if( is_void(color) ) color= "black";
   if( is_void(y0) ) y0 = min(y);

   if( numberof(x) == 0 ) x = double(indgen(numberof(y)));
   yy = xx = array(double,2*numberof(y)+2);
   yy(1:-3:2) = yy(2:-2:2) = y;
   yy(-1) = yy(0) = y0;

   get_boundaries, x, x1, x2;
   xx(1:-3:2) = x1;
   xx(2:-2:2) = x2;
   xx(-1) = x2(0);
   xx(0) = x1(1);

   poly_fillc, yy, xx, color=color;
}

/* Function plhisc  */

func plhisc( y, x, color=, y0=)
/* DOCUMENT plhisc, y, x

   Plot a histogram with color-filled columns
   where x and y have the same number of elements.

   KEYWORDS: color, y0

   SEE ALSO: plh, plg, plhis, plhisv, plhiscv, plthis [plot,..,ps=10]
*/

{
   local x1, x2;
   if( is_void(color) ) color="black";
   if( is_void(y0) ) y0 = min(y);

   if( numberof(x) == 0 ) x = double(indgen(numberof(y)));
   yy = xx = array(double,2*numberof(y)+2);
   yy(1:-3:2) = yy(2:-2:2) = y;
   yy(-1) = yy(0) = y0;

   get_boundaries, x, x1, x2;
   xx(1:-3:2) = x1;
   xx(2:-2:2) = x2;
   xx(-1) = x2(0);
   xx(0) = x1(1);

   poly_fillc, xx, yy, color=color;
}

/* Function dataplot */

func dataplot( x, y, dy, xr= , yr= , ps= , mpl=, \
                 xbar= , fxbar= , itype=, color=, thick=, \
                 xtitle=, ytitle=, title=, symsize=, dbg=, \
                 titleheight=, xytitleheight=, titlefont=, xyadjust= )
/* DOCUMENT dataplot, x, y, dy, xr=,yr=,ps=,mpl=,xbar=,fxbar=,thick=,\
                  itype=,xtitle=,ytitle=,title=,color=, symsize=, dbg=,
                  titleheight=, xytitleheight=, titlefont=, xyadjust=

  Plot of datapoints with 'errorbars' in x-direction defined
  by channel limits derived from spacing between points.
  In the y-direction the length of the error bars are given by '+-dy'
  In the x-direction bars are plotted unless the keyword 'xbar' has been
  set to zero.

  The type of plot is indicated with the keyword 'itype'
    itype = 0  :    x lin - y lin (default)
    itype = 1  :    x lin - y log
    itype = 2  :    x log - y lin
    itype = 3  :    x log - y log

    ~1995/NJW
    2004-09-28/NJW  Updated with better error message
    2005-05-27/NJW  Included keyword 'fxbar': fixed x-bar
    2006-02-20/NJW  Yorick version

*/
{

   if( is_void(xbar) ) xbar=1; // default action
   if( xbar == 0 ) xbar = [];
   if( !is_void(fxbar) ) xbar = [];

   nx = numberof(x);
   if( (nx != numberof(y)) || (nx != numberof(dy)) ) {
     write,"DATAPLOT error: The arrays have different dimensions !\n";
     return;
   }

   // Define the plot type (lin-lin, lin-log, log-lin, log-log)

   it = is_void(itype) ? 0 : itype;
   xlog = it > 1 ? 1 : 0;
   ylog = it % 2 ? 1 : 0;

   // Define boundaries between abscissa values;

   if( nx > 1 ) {
      sx = shift(x,-1);
      sxl = array(double,nx+1);
      if( xlog ) {
         sx = sqrt(x*sx);
         sxl(1:nx) = sx;
         sxl(1) = sqrt(x(1)^3 / x(2));
         sxl(0) = sqrt(x(0)^3 / x(-1));
      } else {
         sx = 0.5*(x + sx);
         sxl(1:nx) = sx;
         sxl(1) = (3*x(1) - x(2))*0.5;
         sxl(0) = (3*x(0) - x(-1))*0.5;
      }
   }

   // Define abscissa plot range;

   xdmin = min(x);
   xdmax = max(x);
   if( numberof(xbar) && nx > 1 ) {
      xdmin = min(sxl);
      xdmax = max(sxl);
   }
   if( numberof(fxbar) ) {
      xdmin = xdmin - fxbar;
      xdmax = xdmax + fxbar;
   }
   if( !numberof(xr) ) {
      if( xlog ) {
         xmin = xdmin^1.05 / xdmax^0.05;
         xmax = xdmax^1.05 / xdmin^0.05;
      } else {
         xmin = xdmin - 0.05 * (xdmax - xdmin);
         xmax = xdmax + 0.05 * (xdmax - xdmin);
      }
      xrg = [xmin,xmax];
   } else xrg = xr;

   // Define ordinate plot range;

   if( !numberof(yr) ) {
      if( ylog ) {
         positive = where( y - dy > 0 );
         ymin = min(y(positive)-dy(positive))^1.05 / max(y+dy)^0.05;
         ymax = max(y+dy)^1.05 / min(y(positive)-dy(positive))^0.05;
      } else {
         ymin = min(y-dy) - 0.05 * (max(y+dy) - min(y-dy));
         ymax = max(y+dy) + 0.05 * (max(y+dy) - min(y-dy));
      }
      yrg = [ymin,ymax];
   } else yrg = yr;

   // Set up plot

   plot,[1],[1],xr=xrg,yr=yrg,itype=itype,title=title,xtitle=xtitle, \
                ytitle=ytitle,mpl=mpl,dbg=dbg,titleheight=titleheight, \
                xytitleheight=xytitleheight, titlefont=titlefont, xyadjust=xyadjust;
   //+ mlogxy,xlog,ylog;
   //+ limits,xrg(1),xrg(2),yrg(1),yrg(2);
   // dbg
   if( dbg ) write,format="DATAPLOT: After call of plot - plsys = %i\n", plsys();
   if( numberof(mpl) ) plsys, mpl;
   if( dbg ) write,format="DATAPLOT: After call of plsys - plsys = %i\n", plsys();


   for( i = 1; i <= nx; i++ ) {
     //  horizontal pieces (only if requested)
     if( numberof(xbar) && nx > 1 ) oplot,sxl(i:i+1),[y(i),y(i)],color=color,thick=thick;
     // vertical pieces
     ylow = y(i)-dy(i);
     if( ylog ) {
        if( ylow <= 0 ) ylow = limits()(3);
     }
     oplot,[x(i),x(i)],[ylow,y(i)+dy(i)],color=color,thick=thick;
   }

   // Plot fixed horizontal piece if requested;

   if( !is_void(fxbar) ) {
      for( i = 1; i<= nx; i++ ) {
         oplot,[x(i)-fxbar,x(i)+fxbar],[y(i),y(i)],color=color,thick=thick;
      }
   }

   if( !is_void(ps) ) oplot,x,y,ps=ps,color=color,symsize=symsize,thick=thick;

}


/* Function odataplot */

func odataplot( x, y, dy, ps= , xbar= , fxbar= , color=, thick=, mpl= )
/* DOCUMENT odataplot, x, y, dy, ps=, xbar=, fxbar=, color=, thick=, mpl=

  Overplot of datapoints with 'errorbars' in x-direction defined
  by channel limits derived from spacing between points.
  In the y-direction the length of the error bars are given by '+-dy'
  In the x-direction bars will be shown unless 'xbar' has been set to zero.

    2007-02-28/NJW  Yorick version

   SEE ALSO: dataplot, dataplotf, plot, oplot, odataplotf

*/
{
   if( is_void(dy) ) {
      write,"Syntax: odataplot, x, y, dy[, ps=]";
      write,"              [,xbar=][,fxbar=][,color=][,thick=][,mpl=]";
      return;
   }
   nx = numberof(x);
   if( (nx != numberof(y)) || (nx != numberof(dy)) ) {
     write,"ODATAPLOT error: The arrays have different dimensions !";
     return;
   }

   if( is_void(xbar) ) xbar=1; // default action
   if( xbar == 0 ) xbar = [];
   if( !is_void(fxbar) ) xbar = [];

   if( !is_void(mpl) ) plsys, mpl;

   // Define boundaries between abscissa values;

   if( nx > 1 ) {
      logset = long(limits()(5));
      xlog = logset & 128;
      sx = shift(x,-1);
      sxl = array(double,nx+1);
      if( xlog ) {
         sx = sqrt(x*sx);
         sxl(1:nx) = sx;
         sxl(1) = sqrt(x(1)^3 / x(2));
         sxl(0) = sqrt(x(0)^3 / x(-1));
      } else {
         sx = 0.5*(x + sx);
         sxl(1:nx) = sx;
         sxl(1) = (3*x(1) - x(2))*0.5;
         sxl(0) = (3*x(0) - x(-1))*0.5;
      }
   }


   for( i = 1; i <= nx; i++ ) {
     //  horizontal pieces (only if requested)
     if( numberof(xbar) && nx > 1 ) oplot,sxl(i:i+1),[y(i),y(i)],color=color,thick=thick;
     // vertical pieces
     ylow = y(i)-dy(i);
     if( ylog ) {
        if( ylow <= 0 ) ylow = yrg(1);
     }
     oplot,[x(i),x(i)],[y(i)-dy(i),y(i)+dy(i)],color=color,thick=thick;
   }

   // Plot fixed horizontal piece if requested;

   if( !is_void(fxbar) ) {
      for( i = 1; i<= nx; i++ ) {
         oplot,[x(i)-fxbar,x(i)+fxbar],[y(i),y(i)],color=color,thick=thick;
      }
   }

   if( !is_void(ps) ) oplot,x,y,ps=ps,color=color,thick=thick;

}

/* Function dataplotf */

func dataplotf( x1,x,x2,y1,y,y2, xr=, yr=, ps=, mpl=, itype=, color=, thick=, \
         xtitle=, ytitle=, title=, titleheight=, xytitleheight=, xyadjust=, titlefont= )
/* DOCUMENT dataplotf, x1,x,x2,y1,y,y2, xr=, yr=, ps=, mpl=, itype=, color=, thick=,
         xtitle=, ytitle=, title=, titleheight=, xytitleheight=, xyadjust=, titlefont=

   Lin-lin plot of datapoints with 'errorbars' in x-direction defined
   by start and stop arrays x1 and x2. Similarly
   in the y-direction the length of the error bars are given by
   lower and upper values y1 and y2

   The type of plot is indicated with the keyword 'itype'
     itype = 0  :    x lin - y lin (default)
     itype = 1  :    x lin - y log
     itype = 2  :    x log - y lin
     itype = 3  :    x log - y log

  SEE ALSO: dataplot

*/
{
   if( is_void(y2) ) {
      write,"dataplotf, x1, x, x2, y1, y, y2[,keywords]";
      return;
   }
   nx = numberof(x);
   nx1 = numberof(x1);
   nx2 = numberof(x2);
   ny = numberof(y);
   ny1 = numberof(y1);
   ny2 = numberof(y2);

   // Check out of input;

   if( (nx != nx1) || (nx != nx2) || (nx != ny) || (nx != ny1) || (nx != ny2) ) {
     write,"DATAPLOTF error: The arrays have different dimensions !";
     return;
   }

   // Set plot type

   if( is_void(itype) ) itype = 0;
   xlog = (itype > 1);
   ylog = itype % 2;

   iypos = where( y1 > 0 );
   n_iypos = numberof(iypos);

   if( is_void(xr) ) {
      if( xlog ) {
         xmin = min(x1)^1.05 / max(x2)^0.05;
         xmax = max(x2)^1.05 / min(x1)^0.05;
      } else {
         xmin = min(x1) - 0.05 * (max(x2) - min(x1));
         xmax = max(x2) + 0.05 * (max(x2) - min(x1));
      }
      xr = [xmin,xmax];
   }
   if( is_void(yr) ) {;
      if( ylog ) {
         if( n_iypos > 0 ) {
            ymin = min(y1(iypos))^1.05 / max(y2)^0.05;
            ymax = max(y2)^1.05 / min(y1(iypos))^0.05;
         } else {
            ymax = 1.3 * max(y2);
            ymin = 0.001 * ymax;
         }
      } else {
         ymin = min(y1) - 0.05 * (max(y2) - min(y1));
         ymax = max(y2) + 0.05 * (max(y2) - min(y1));
      }
      yr = [ymin,ymax];
   }

   // Set up plot;

   plot,[1],[1],xr=xr,yr=yr, mpl=mpl, itype=itype, xtitle=xtitle, \
           ytitle=ytitle, title=title, titleheight=titleheight, \
           xyadjust=xyadjust, xytitleheight=xytitleheight, titlefont=titlefont;
   // dbg
   if( dbg ) write,format="DATAPLOTF: After call of plot - plsys = %i\n", plsys();
   if( numberof(mpl) ) plsys, mpl;
   if( dbg ) write,format="DATAPLOTF: After call of plsys - plsys = %i\n", plsys();

   // Do the plotting;

   for( i = 1; i <= nx; i++)  {
      oplot,[x1(i),x2(i)],[y(i),y(i)],color=color,thick=thick;
      oplot,[x(i),x(i)],[y1(i),y2(i)],color=color,thick=thick;
      if( numberof(ps) ) oplot,[x(i)],[y(i)],ps=ps,color=color,thick=thick;
   }
}



/* Function odataplotf */

func odataplotf( x1,x,x2,y1,y,y2, ps=, color=, thick=, mpl= )
/* DOCUMENT odataplotf, x1,x,x2,y1,y,y2, ps=, color=, thick=, mpl=

   Plot of datapoints with 'errorbars' in x-direction defined
   by start and stop arrays x1 and x2. Similarly
   in the y-direction the length of the error bars are given by
   lower and upper values y1 and y2

   2008-02-01/NJW cloned from dataplotf

*/
{
   if( is_void(y2) ) {
      write,"Syntax: odataplotf, x1, x, x2, y1, y, y2[,keywords]";
      return;
   }

   if( numberof(mpl) ) plsys, mpl;

   nx = numberof(x);
   nx1 = numberof(x1);
   nx2 = numberof(x2);
   ny = numberof(y);
   ny1 = numberof(y1);
   ny2 = numberof(y2);

   // Check out of input;

   if( (nx != nx1) || (nx != nx2) || (nx != ny) || (nx != ny1) || (nx != ny2) ) {
     write,"DATAPLOTF error: The arrays have different dimensions !";
     return;
   }


   iypos = where( y1 > 0 );
   n_iypos = numberof(iypos);

   // Do the plotting;

   for( i = 1; i <= nx; i++)  {
     oplot,[x1(i),x2(i)],[y(i),y(i)],color=color,thick=thick;
     oplot,[x(i),x(i)],[y1(i),y2(i)],color=color,thick=thick;
     if( numberof(ps) ) oplot,[x(i)],[y(i)],ps=ps,color=color,thick=thick;
   }
}


/* Function plotsign */

func plotsign( pos= )
/* DOCUMENT plotsign[, pos=]
   Prints the string NSI/DTU/YYYY-MM-DD/NJW in the upper right
   corner of the plot.

   Keyword 'pos' will override the standard position in NDC (two elements).

   2006-11-17/NJW, updated 2011-10-19/NJW
*/
{
   if( numberof(pos) == 2 ) {
      plt,"NSI/DTU/"+ndate(2)+"/NJW",pos(1),pos(2),justify="RA",height=8;
   } else {
      port = viewport();
      plt,"NSI/DTU/"+ndate(2)+"/NJW",port(2),port(4)+0.01,justify="RA",height=8;
   }
}

/* Function plotdate */

func plotdate(pos=)
/* DOCUMENT plotdate[,pos=]
   Prints the string YYYY-MM-DD in the upper right
   corner of the plot.
   2011-05-27/NJW
*/
{
   if( numberof(pos) == 2 ) {
      plt,ndate(2),pos(1),pos(2),justify="RA",height=8;
   } else {
      port = viewport();
      plt,ndate(2),port(2),port(4)+0.01,justify="RA",height=8;
   }
}

/* Function esc_uscore */

func esc_uscore( strng )
/* DOCUMENT new_strng = esc_uscore( strng )

   "Escape underscore" which means remove special significance
   of the underscore character in a text string to be written
   in a plot e.g. by 'plt' or 'xyouts'
   2007-10-31/NJW
*/
{
   // Identify the occurrences of '_'
   newstring = strng;
   spos = 1;
   pos = strpos(strng,"_",1);
   if( pos > 0 ) {
      newstring = "";
      while( pos > 0 ) {
         if( pos == 1 ) {
            newstring += "!_";
         } else {
            newstring += strpart(strng,spos:pos-1)+"!_";
         }
         spos = pos + 1;
         pos = strpos(strng,"_",spos);
      }
      newstring += strpart(strng,spos:0);
   }

   return newstring;

}

/* Function plotname */

func plotname(name)
/* DOCUMENT plotname, name
   Prints the string 'name' in the upper left
   corner of the plot. The underscore character is kept. i.e. its
   special meaning for formatting (starting subscript) is lost here.
   2006-11-17/NJW
*/
{
   // Use function esc_underscore to eliminate unexpected consequences
   // of '_'
   port = viewport();
   plt,esc_underscore(name),port(1),port(4)+0.005,justify="LA",height=8;
}

/* Function plotcommments */

func plotcomments( file=, str=, init=, keep= )
/* DOCUMENT plotcomments, file=, str=, init=, keep=

   Add a text below the standard plotting window for a plot
   on A4 paper in portrait orientation.

   Keywords:
     init          1 : Reset y-position of text to just below viewport
                 double: y-position of text in normalized coordinates
     file         Name of text file
      str         A scalar string or a string array
     keep         (bool) keep the formatting symbols (like '^', '_', and '!')
                         in the text string

   The external PLOTCOMMENTS_YLEV keeps track of the line positions for each
           of the 8 possible windows

   2006-11-17/NJW
   2010-05-10/NJW, updated with new init keyword
*/
{

   extern PLOTCOMMENTS_YLEV; // array with 8 elements, one for each
                             // possible window

   if( !is_void(file) ) {
      if( !file_test(file) ) {
         write,format="%s does not exist\n", file;
         return;
      }
      text = read_slist( file );
   }
   if( !is_void(str) ) text = str;

   if( is_void(text) && is_void(init) ) {
      write,"PLOTCOMMENTS: Text info is missing, skip ...";
      return;
   }

   if( !is_void(text) ) {
      if( keep ) {
         line = strjoin( text, "\n" );
      } else {
         line = strjoin( esc_underscore(text), "\n" );
      }
   }

   win_idx = window() + 1;
   port = viewport();
   dy = port(4)-port(3);

   if( is_void(init) ) init = 0;
   if( typeof(init) == "double" ) {
      y_ndc = init;
   } else {
      y_ndc = port(3) - 0.1;
   }

  /*
   *   PLOTCOMMENTS_YLEV is an array with the y-position of
   *   where to place the next line
   */

   if( is_void(PLOTCOMMENTS_YLEV) ) {
      PLOTCOMMENTS_YLEV = array(port(3) - 0.1, 8);
   }
   if( init ) {
      PLOTCOMMENTS_YLEV(win_idx) = y_ndc;
   }
//    plt handles new-lines (line shifts) by itself
   if( !is_void(text) ) {
      plt, line, port(1), PLOTCOMMENTS_YLEV(win_idx), height=10,justify="LT";

//  update for next call of plotcomments:

      PLOTCOMMENTS_YLEV(win_idx) -= 0.015*numberof(text);
   }
}


/* Function zps */

func zps( &filename, outfile=, pr=, noc=, gv=, erase= )
/* DOCUMENT zps, >filename, outfile=, pr=, noc=, gv=, erase=
   Dump plot to next hard copy file from pattern yplot_????.ps
   unless keyword 'outfile' is given.

   Keyword
      'pr'    sends plot to a develop printer with this number.
      'noc'   prevents comment with plot file name in the plot file.
                 noc=1 is the default setting.
      'gv'    causes a ghostview/evince display of the plotfile.
      'erase' causes a removal of the plot file after printing or displaying
   

   2006-06-13/NJW
*/
{
   require, "basic.i";

   if( typeof(outfile) == "string" ) {
      filename = outfile;
   } else {
      filename = get_next_filename("yplot_????.ps",dir=".");
   }
   write,format="New plot file name: %s\n", filename;
   if( is_void(noc) ) noc = 1; // setting default
   if(noc) {
      plotcomments,init=1;
   } else {
      plotcomments,str=swrite(format="This plotfile is %s", fullpath(filename));
   }
   hcp_file, filename;
   hcp;
   hcp_finish;
   if( is_void(pr) ) pr = 0;
   if( pr == 1 ) {
      system,"lpr -PXerox1 "+filename;
      write,"Has executed lpr -PXerox1 "+filename;
   }
   if( pr == 2 ) {
      system,"lpr -Pdevelop2 "+filename;
      write,"Has executed lpr -Pdevelop2 "+filename;
   }
   if( pr == 3 ) {
      system,"lpr -PXerox3 "+filename;
      write,"Has executed lpr -PXerox3 "+filename;
   }
   if( gv ) {
      host = ghost();
      if( host == "maxwell" || host == "tesla" ) {
         system,"gv "+filename;
      } else system,"evince "+filename;
   }
   if( erase ) remove,filename;
}

/* Function gv */

func gv( filename )
/* DOCUMENT gv, filename

    Executes a ghostview/evince plot of the named file.
*/
{
   host = ghost();
   if( host == "maxwell" || host == "tesla" ) {
      system,"gv "+filename;
   } else system,"evince "+filename;
}

/* Function def_sym */

func def_sym( sym_number )
/* DOCUMENT def_sym, sym_number

   where 'sym_number' must be larger than 10.
   Sets the user defined plot symbols.

   11 - Cross            18 - Vertical line       25 - Star 8
   12 - Plus             19 - Square              26 - Star 9
   13 - Circle           20 - Triangle on bottom  27 - Diamond
   14 - Slash            21 - Triangle on top     28 - Ellipse horiz.
   15 - Backslash        22 - Star 5              29 - Ellipse vert.
   16 - Empty plus       23 - Star 6
   17 - Horizontal line  24 - Star 7

   2008-08-07/NJW, updated 2010-10-05 and 2017-01-11/NJW
*/
{
   extern Def_sym_size_x, Def_sym_size_y, Def_sym_number, Def_sym_x, Def_sym_y;

   if( is_void(sym_number) ) sym_number = 0;
   if( sym_number < 11 || sym_number > 29 ) {
      write,"DEF_SYM argument out of limits";
      return [];
   }

   dx = Def_sym_size_x/2;
   dy = Def_sym_size_y/2;

   staro = 1.1;
   stari = 0.5;
   if( sym_number == 11 ) { // Cross
      Def_sym_x = dx*0.707*[[-1,1],[-1,1]];
      Def_sym_y = dy*0.707*[[-1,1],[1,-1]];
   } else if( sym_number == 12 ) { // Plus
      Def_sym_x = dx*[[-1,1],[0,0]];
      Def_sym_y = dy*[[0,0],[1,-1]];
   } else if( sym_number == 13 ) { // Circle
      v = span(0,2*pi,50);
      Def_sym_x = dx*cos(v);
      Def_sym_y = dy*sin(v);
   } else if( sym_number == 14 ) { // Slash
      Def_sym_x = dx*0.707*[-1,1];
      Def_sym_y = dy*0.707*[-1,1];
   } else if( sym_number == 15 ) { // Backslash
      Def_sym_x = dx*0.707*[-1,1];
      Def_sym_y = dy*0.707*[1,-1];
   } else if( sym_number == 16 ) { // Empty plus
      Def_sym_x = dx*[[-1,-0.5],[0.5,1],[0,0],[0,0]];
      Def_sym_y = dy*[[0,0],[0,0],[-1,-0.5],[0.5,1]];
   } else if( sym_number == 17 ) { // Horizontal line
      Def_sym_x = dx*[-1,1];
      Def_sym_y = dy*[0,0];
   } else if( sym_number == 18 ) { // Vertical line
      Def_sym_x = dx*[0,0];
      Def_sym_y = dy*[-1,1];
   } else if( sym_number == 19 ) { // Square
      Def_sym_x = dx*[-1,1,1,-1,-1];
      Def_sym_y = dy*[-1,-1,1,1,-1];
   } else if( sym_number == 20 ) { // Triangle on bottom
      Def_sym_x = dx*[-0.866,0.866,0,-0.866];
      Def_sym_y = dy*[-0.5,-0.5,1,-0.5];
   } else if( sym_number == 21 ) { // Triangle on tip
      Def_sym_x = dx*[-0.866,0.866,0,-0.866];
      Def_sym_y = dy*[0.5,0.5,-1,0.5];
   } else if( sym_number >= 22 && sym_number <= 26 ) { // Five - to nine fold stars
      n = sym_number - 17;
      v = span(0,2*pi,n+1);
      Def_sym_x = array(double,2*n+1);
      Def_sym_y = array(double,2*n+1);
      Def_sym_x(1:2*n+1:2) = -staro*dx*sin(v);
      Def_sym_y(1:2*n+1:2) =  staro*dy*cos(v);
      Def_sym_x(2:2*n:2) = -stari*dx*sin(v(zcen));
      Def_sym_y(2:2*n:2) =  stari*dy*cos(v(zcen));
   } else if( sym_number == 27 ) { // Diamond
      Def_sym_x = dx*[-1,0,1,0,-1];
      Def_sym_y = dy*[0,1,0,-1,0];
   } else if( sym_number == 28 ) { // Ellipse lying down
      v = span(0,2*pi,50);
      Def_sym_x = dx*cos(v);
      Def_sym_y = dy*sin(v)/2;
   } else if( sym_number == 29 ) { // Ellipse standing up
      v = span(0,2*pi,50);
      Def_sym_x = dx*cos(v)/2;
      Def_sym_y = dy*sin(v);
   }

   Def_sym_number = sym_number;

}

/* Function mlogxy */

func mlogxy( &xflag, &yflag, mpl=, win=, get=, dbg= )
/* DOCUMENT mlogxy, (>)xflag, (>)yflag, mpl=, win=, get=, dbg=

   Same functionality as 'logxy' except that the external variables
   Logxy_flags_content and Logxy_flags_values are updated.
   When keyword 'get' is set then the values are returned in stead.

   Logxy_flags_content = mpl*10 + win

   If none of these keywords are given then the current values will
   be used.

   Log_flags_values is like 'itype': 0,1,2,3 for lin-lin, lin-log, log-lin, log-log

   2008-08-08/NJW
   2011-06-14/NJW updated to include the plsys functionality
*/
{
   //+ extern Log_flags_x, Log_flags_y;
   extern Logxy_flags_content, Logxy_flags_values;

   if( is_void(win) ) win = window();
   if( is_void(mpl) ) {
      mpl = plsys();
   } else {
      plsys,mpl;
   }
   if( dbg ) write,format="MLOGXY ##17## win = %i, mpl = %i\n", win, mpl;

   reference = 10*mpl + win;

   if( get ) { // Retrieve values

      if( is_void( Logxy_flags_content ) ) {
         xflag = 0;
         yflag = 0;
      } else {
         // Look for reference
         w = where( reference == Logxy_flags_content );
         if( numberof(w) ) { // It was indeed found
            xflag = Logxy_flags_values(w(1))/2;
            yflag = Logxy_flags_values(w(1))%2;
         } else { // Not found, not defined in this session
            xflag = 0;
            yflag = 0;
         }
      }

   } else {    // Store values and define plotting

      if( is_void(xflag) || is_void(yflag) ) error,"MLOGXY: missing argument(s)";
      value = 2*xflag + yflag;
      if( is_void(Logxy_flags_content) ) {
         grow, Logxy_flags_content, reference;
         grow, Logxy_flags_values, value;
      } else {
         // Look if reference already exists
         w = where( reference == Logxy_flags_content );
         if( numberof(w) ) { // It was indeed found
            Logxy_flags_values(w(1)) = value;
         } else { // not found, add to list
            grow, Logxy_flags_content, reference;
            grow, Logxy_flags_values, value;
         }
      }
      logxy, xflag, yflag;

   }

}

/* Function mcoord_conv */

func mcoord_conv( p_in, q_in, &p_out, &q_out, from=, to= )
/* DOCUMENT mcoord_conv, p_in, q_in, >p_out, >q_out, from=, to=

   Convert coordinates between NDC (Normalized Device Coordinates)
   keyword value "ndc", virtual coordinates (keyword value "vir")
   and world (i.e. data space) (keyword value "wor") coordinates.
   The (A4) device coordinates have been expanded by a factor of
   sqrt(2) in the y direction compared to NDC. [not implemented]

   2008-08-13/NJW
*/
{
   if( is_void(from) ) {
      write,"Keyword 'from' not defined";
      return;
   }
   if( is_void(to) ) {
      write,"Keyword 'to' not defined";
      return;
   }
   v = viewport()(1:4);
   w = limits()(1:4);
   wind = 1 + window();
   local xlog, ylog;
   mlogxy, xlog, ylog,get=1;

   from = strlowcase(from);
   to = strlowcase(to);
   if( from == "ndc" ) fco = v;
   if( from == "vir" ) fco = [0.,1.,0.,1.];
   if( from == "wor" ) fco = w;
   if( to == "ndc" ) tco = v;
   if( to == "vir" ) tco = [0.,1.,0.,1.];
   if( to == "wor" ) tco = w;

   if( from == "wor" ) {
      // World to linear
      if( xlog ) {
         fco(1) = log(fco(1));
         fco(2) = log(fco(2));
         p_in = log(p_in);
      }
      if( ylog ) {
         fco(3) = log(fco(3));
         fco(4) = log(fco(4));
         q_in = log(q_in);
      }
   }
   if( to == "wor" ) {
      // linear to world
      if( xlog ) {
         tco(1) = log(tco(1));
         tco(2) = log(tco(2));
      }
      if( ylog ) {
         tco(3) = log(tco(3));
         tco(4) = log(tco(4));
      }
   }
   fac = (p_in - fco(1))/(fco(2)-fco(1));
   p_out = tco(1) + fac*(tco(2) - tco(1));
   fac = (q_in - fco(3))/(fco(4) - fco(3));
   q_out = tco(3) + fac*(tco(4) - tco(3));

   if( to == "wor" ) {
      if( xlog ) p_out = exp(p_out);
      if( ylog ) q_out = exp(q_out);
   }

   return;
}

/* Function plot_spectrum */

func plot_spectrum( eb1, eb2, rate, rate_err, frac=, perkev=, itype=, \
                    xr=, yr=, color=, title=, ytitle=, backscale= )
/* DOCUMENT plot_spectrum, eb1, eb2, rate, rate_err, frac=, perkev=, \
                    itype=, xr=, yr=, color=, title=, ytitle=, backscal=

  Plot one or more spectra as cnt/keV/s

  Arguments:

       eb1 :   lower energy boundaries or an instance of struct 's_Spectrum' (see below)
       eb2 :   upper energy boundaries
      rate :   counts per channel per s
  rate_err :   error in counts per channel per s

  Alternatively the first (and only) argument can be an instance of the struct 's_Spectrum'
     that is defined in 'xray.i'. In this case the plot title is set to the spectral file name
     appended with the spectrum number, unless 'title' has been user specified.

  Several spectra can be given dimensions are: (nchan, nspectra)

  Keywords:   perkev     set if rate and error are given as /keV/s
              frac       for rebinning, combines channels until err/rate < frac
              backscal   From the BACKSCAL keyword. The value will be saved for renormalizing
                         spectra overplotted with 'oplot_spectrum' - has no effect for the
                         current plotting.
              itype      0 : lin-lin
                         1 : lin-log
                         2 : log-lin
                         3 : log-log

     The remaining keywords (xr, yr, color, title, and ytitle ) are as for 'plot'.

   2008-02-01/NJW translated to Yorick from IDL
   2014-10-16/NJW updated to include the saving of the BACKSCAL value
*/
{
   extern _plot_spectrum_backscal;

   if( is_void(itype) ) itype = 0;  // lin-lin plot is default

   if( typeof(eb1) == "struct_instance" ) { // case of struct s_Spectrum
      rate = *eb1.rate;                     // see 'read_spectrum' in 'xray.i'
      rate_err = *eb1.stat_err;
      _plot_spectrum_backscal = eb1.backscal;
      if( is_void(title) ) {
         local ffname, extno;
         get_exten_no, eb1.specdol, ffname, extno;
         title = esc_underscore(ffname+"{"+itoa(eb1.specnum)+"}");
      }
      eb2 = *eb1.e_max;
      eb1 = *eb1.e_min; // important that this statement is last
   }

   if( !is_void(backscal) ) _plot_spectrum_backscal = backscal;

   sz = dimsof(rate);
   sze = dimsof(rate_err);
   if( sz(1) != 1 && sz(1) != 2 ) {
      write,"PLOT_SPECTRUM: The spectral array has bad dimensions";
      return [];
   }

   if( sz(1) != sze(1) ) {
      write,"PLOT_SPECTRUM: Mismatch of dimensions in rate and rate_err arrays";
      return [];
   }
   if( anyof(sz-sze) ) {
      write,"PLOT_SPECTRUM: Mismatch of dimensions in rate and rate_err arrays";
      return [];
   }


   if( numberof(eb1) != numberof(eb2) ) {
      write,"The channel boundary arrays have different number of elements";
      return [];
   }

   nchan = sz(2);
   nspec = sz(1) == 2 ? sz(3) : 1;

   if( numberof(eb1) != nchan ) {
      write,"The number of channel boundaries does not match spectral array";
      return [];
   }

   if( ! perkev ) {
      spec = rate / (eb2 - eb1)(,-);
      erra = rate_err / (eb2 - eb1)(,-);
   } else {
      spec = rate;
      erra = rate_err;
      rate = spec * (eb2 - eb1)(,-);
      rate_err = erra * (eb2 - eb1)(,-);
   }

   if( !is_void(frac) ) {
      for( i = 1; i <= nspec; i++ ) {
         if( i == 1 ) {
            // use first spectrum as model for the rebinning
            REBIN = specrebinninga( eb1, eb2, rate(,i), rate_err(,i), frac, ob1, ob2, orate, orate_err);
            orate_all = orate(,-:1:nspec);
            orate_err_all = orate_err(,-:1:nspec);
         } else {
            specrebinninga, eb1, eb2, rate(,i), rate_err(,i), REBIN, ob1, ob2, orate, orate_err;
            orate_all(,i) = orate;
            orate_err_all(,i) = orate_err;
         }
      }
      spec = orate_all / (ob2 - ob1)(,-);
      erra = orate_err_all / (ob2 - ob1)(,-);
      eb1 = ob1;
      eb2 = ob2;
   }
   

   ymax = 1.05*max(spec+erra);
   aux = spec - erra;
   ymin = 0.95*min(aux(where(aux > 0)));

   xtitle = "Energy [keV]";
   if( is_void(ytitle) ) ytitle = "Counts [/keV/s]";

   if( itype >= 2 && eb1(1) == 0.0 ) eb1(1) = 0.5*eb2(1);

   eb = eb1(1) <=  0.0 ? 0.5*(eb1 + eb2) : sqrt(eb1*eb2);

   if( ! numberof(xr) ) xr=[0.95*min(eb1),1.05*max(eb2)];
   if( ! numberof(yr) ) yr=[ymin,ymax];
   plot,[1],[1],xr=xr,yr=yr,itype=itype,title=title, xtitle=xtitle,ytitle=ytitle;

   for( i = 1; i <= nspec; i++ ) {

      y1 = spec(,i) - erra(,i);
      y2 = spec(,i) + erra(,i);
      y = spec(,i);

      odataplotf,eb1,eb,eb2,y1,y,y2,color=color;

   }

   //+ plotsign;
   //+ plotname,"plot_spectrum";
}

/* Function oplot_spectrum */

func oplot_spectrum( eb1, eb2, rate, rate_err, frac=, perkev=, color=, bscal_ref=, f_adj=, backscal= )
/* DOCUMENT oplot_spectrum, eb1, eb2, rate, rate_err, frac=, perkev=, color=, bscal_ref=, f_adj=, backscal=

   Plot one or more spectra as cnt/keV/s

  eb1 : lower energy boundaries
  eb2 : upper energy boundaries
  rate : counts per channel per s
  rate_err : error in counts per channel per s

  Alternatively the first (and only) argument can be an instance of the struct 's_Spectrum'

  Several spectra can be given dimensions are: (nchan, nspectra)

  Keywords:   perkev     set if rate and error are given as /keV/s
              frac       rebin until err/rate < frac
              color      usual meaning (as in 'plot')
              bscal_ref  A reference BACKSCAL value. This spectrum will be adjusted
                         with 'bscal_ref / backscal'.
                         'bscal_ref' can either be given directly as a number or as an instance of
                         a struct 's_Spectrum'. Works only if first argument is
                         a struct 's_Spectrum' or 'backscal' is given.
                         If not given, the saved value of '_plot_spectrum_backscal' will be
                         used as a reference.
              f_adj      the spectrum and error is multiplied before adding to
                         plot if this factor if given
              backscal   The equivalent of the FITS BACKSCAL keyword for the present spectrum


   2008-02-01/NJW cloned from plot_spectrum
*/
{
   extern _plot_spectrum_backscal;

   if( typeof(eb1) == "struct_instance" ) { // case of struct s_Spectrum
      rate = *eb1.rate;                     // see 'read_spectrum.i'
      rate_err = *eb1.stat_err;
      if( is_void(backscal) ) backscal = eb1.backscal; // save before 'eb1' is changed
      eb2 = *eb1.e_max;
      eb1 = *eb1.e_min; // important that this statement is last
   }

   sz = dimsof(rate);
   sze = dimsof(rate_err);
   if( sz(1) != 1 && sz(1) != 2 ) {
      write,"OPLOT_SPECTRUM: The spectral array has bad dimensions";
      return [];
   }

   if( sz(1) != sze(1) ) {
      write,"OPLOT_SPECTRUM: Mismatch of dimensions in rate and rate_err arrays";
      return [];
   }
   if( anyof(sz-sze) ) {
      write,"OPLOT_SPECTRUM: Mismatch of dimensions in rate and rate_err arrays";
      return [];
   }


   if( numberof(eb1) != numberof(eb2) ) {
      write,"The channel boundary arrays have different number of elements";
      return [];
   }

   adj_factor = 1.0; // default value
   if( !is_void(bscal_ref) ) {
      if( is_void( backscal ) ) {
         write,"OPLOT_SPECTRUM: no BACKSCAL for this spectrum is available";
      } else {
         if( typeof(bscal_ref) == "struct_instance" ) {
            adj_factor = bscal_ref.backscal / backscal;
         } else {
            adj_factor = bscal_ref / backscal;
         }
      }
   } else { // try to use _plot_spectrum_backscal (extern)
      if( !is_void(_plot_spectrum_backscal) && !is_void(backscal) ) {
        adj_factor = _plot_spectrum_backscal / backscal;
      }
   }

   if( !is_void(f_adj) ) adj_factor *= f_ajd;

   write,format="Resulting adj_factor: %12.4e\n", adj_factor;

   nchan = sz(2);
   nspec = sz(1) == 2 ? sz(3) : 1;

   if( numberof(eb1) != nchan ) {
      write,"The number of channel boundaries does not match spectral array";
      return [];
   }

   if( ! perkev ) {
      spec = rate / (eb2 - eb1)(,-);
      erra = rate_err / (eb2 - eb1)(,-);
   } else {
      spec = rate;
      erra = rate_err;
      rate = spec * (eb2 - eb1)(,-);
      rate_err = erra * (eb2 - eb1)(,-);
   }

   if( !is_void(frac) ) {
      for( i = 1; i <= nspec; i++ ) {
         if( i == 1 ) {
            // use first spectrum as model for the rebinning
            REBIN = specrebinninga( eb1, eb2, rate(,i), rate_err(,i), frac, ob1, ob2, orate, orate_err);
            orate_all = orate(,-:1:nspec);
            orate_err_all = orate_err(,-:1:nspec);
            //+ xmax = ob2(0);
            //+ xmin = ob1(1);
            //+ ymax = max( (orate+orate_err)/(ob2-ob1) );
            //+ ymin = min( (orate-orate_err)/(ob2-ob1));
         } else {
            specrebinninga, eb1, eb2, rate(,i), rate_err(,i), REBIN, ob1, ob2, orate, orate_err;
            orate_all(,i) = orate;
            orate_err_all(,i) = orate_err;
            //+ xmax = max( xmax, ob2(0) );
            //+ xmin = min( xmin, ob1(1) );
            //+ ymax = max( ymax, max((orate+orate_err)/(ob2-ob1)) );
            //+ ymin = min( ymin, min((orate-orate_err)/(ob2-ob1)) );
         }
      }
      spec = orate_all / (ob2 - ob1)(,-);
      erra = orate_err_all / (ob2 - ob1)(,-);
      eb1 = ob1;
      eb2 = ob2;
   }

   eb = eb1(1) <=  0.0 ? 0.5*(eb1 + eb2) : sqrt(eb1*eb2);

   for( i = 1; i <= nspec; i++ ) {

      y1 = adj_factor*(spec(,i) - erra(,i));
      y2 = adj_factor*(spec(,i) + erra(,i));
      y  = adj_factor*spec(,i);

      odataplotf,eb1,eb,eb2,y1,y,y2,color=color;

   }

   //+ plotsign;
   //+ plotname,"plot_spectrum";
}

/* Function mplot_setup */

func mplot_setup( dims, pane=, spacing=, vport=, hideticklabels=, \
                  vrelsize=, hrelsize=, ticklen=, landscape= )
/* DOCUMENT mplot_setup, dims, pane=, spacing=, vport=, hideticklabels=, \
                  vrelsize=, hrelsize=, ticklen=, landscape=

   Defines a mplot.gs file in /home/njw/yorick/yorick-2.1/g
   with 'dims'/10 rows of frames and 'dims'%10 columns.
   Order is row by row.

   Keywords:
      pane      window number. Default 0.

      spacing   Spacing between frames in NDC.
                If given as 2 element array then
                'spacing(1)' will be the row spacing and
                'spacing(2)' the column spacing
                Default spacing is 0.1.

      vport     The exterior frame defined as 'viewport' in NDC
                coordinates [xmin, xmax, ymin, ymax]
                Default: [0.1, 0.7, 0.1, 0.95]

      hideticklabels  Will eliminate ticklabels between frames.

      vrelsize  Array of vertical relative framesizes. Must have
                the same number of elements as the requested number
                of rows (dims/10).

      hrelsize  Array of horizontal relative framesizes. Must have
                the same number of elements as the requested number
                of columns (dims%10).

      ticklen   Resize the tickLen with this relative size as compared to
                default.

      landscape (bool) When TRUE (1) the landscape format is chosen.

   Hint: The keyword 'mpl' for the plot command is used to decide where
         to plot.

   2009-11-02/NJW
*/
{

   if( is_void(pane) ) pane = 0;

   n_rows = dims/10;
   n_cols = dims%10;

   n_frames = n_rows*n_cols;

   // GISTDIR is an external variable defined when invoking Yorick
   gs = rdfile(GISTDIR+"mplot_tpl.gs");
   gs1 = gs(1:53);
   gs2 = gs(54:0);

   // -- make 'no comment' version

   n = numberof(gs);
   gsc = gs;
   for( i = 1; i <= n; i++ ) {
      p = strpos( gs(i), "#" );
      if( p ) {
         if( p == 1 ) gsc(i) = "";
         else gsc(i) = strpart( gs(i), 1:p-1);
      }
   }
   gs1c = gsc(1:53);
   gs2c = gsc(54:0);

   // -- Set the landscape option if required

   if( landscape ) {
      // set the 'landscape' keyword
      w = where( strmatch(gs1c,"landscape") );
      gs1(w(1)) = "landscape= 1";
      // redefine viewport
      // w = where( strmatch(gs1c, "viewport") );
      // gs1(w(1)) = "viewport={0.1, 1.05, 0.2, 0.7}";
   }

   // -- Take care of tick lengths

   if( numberof(ticklen) ) {
      w = where( strmatch(gs1,"tickLen") );
      if( numberof(w) == 2 ) {
         for( i = 1; i <= 2; i++ ) {
            line = gs1(w(1));
            p1 = strpos( line, "{" );
            p2 = strpos( line, "}" );
            l1 = strpart(line,1:p1);
            l2 = strpart(line,p1+1:p2-1);
            l3 = strpart(line,p2:0);
            tok = strsplit( l2, "," );
            ftok = atof( tok ) * ticklen;
            for( j = 1; j <= numberof(tok); j++ ) {
               l1 = l1 + ftoa(ftok(j));
               if( j < numberof(tok) ) l1 = l1 + ", ";
            }
            l1 = l1 + l3;
            gs1(w(i)) = l1;
         }
      } else {
         write,"Warning, less than two tickLen definitions found in mplot_tpl.gs";
      }
   }

   // -- Take care of spacing

   if( is_void(spacing) ) {
      vspace = 0.1;
      hspace = 0.1;
   } else {
      if( numberof(spacing) > 1 ) {
         vspace = spacing(1);
         hspace = spacing(2);
      } else {
         hspace = vspace = spacing;
      }
   }

   if( is_void(vrelsize) ) {
      vrelsize = array(1.,n_rows);
   } else {
      if( numberof(vrelsize) != n_rows ) error,"Illegal dimension of vrelsize";
   }
   vrelsize /= sum(double(vrelsize));

   if( is_void(hrelsize) ) {
      hrelsize = array(1.,n_cols);
   } else {
      if( numberof(hrelsize) != n_cols ) error,"Illegal dimension of hrelsize";
   }
   hrelsize /= sum(double(hrelsize));

   if( is_void(vport) ) vport = landscape ? [0.1, 1.05, 0.2, 0.7 ] : [0.1, 0.7, 0.1, 0.95];

   // Make arrays with frame sizes in NDC
   vunit = (vport(4) - vport(3) - vspace*(n_rows-1)) * vrelsize;
   hunit = (vport(2) - vport(1) - hspace*(n_cols-1)) * hrelsize;

   // Define the systems row by row

   for( irow = 1; irow <= n_rows; irow++ ) {
      //+ vp4 = vport(4) - (irow-1)*vunit*(1+vspace);
      // calculating from top
      sum_vunit = irow == 1 ? 0.0 : sum(vunit(1:irow-1));
      vp4 = vport(4) - (irow-1)*vspace - sum_vunit;
      vp3 = vp4 - vunit(irow); 
      for( icol = 1; icol <= n_cols; icol++ ) {
         //+ vp1 = vport(1) + (icol-1)*hunit*(1+hspace);
         // calculating from left side
         sum_hunit = icol == 1 ? 0.0 : sum(hunit(1:icol-1));
         vp1 = vport(1) + (icol-1)*hspace + sum_hunit; 
         vp2 = vp1 + hunit(icol); 
         line = swrite(format="system={viewport={ %.4f, %.4f, %.4f, %.4f}", \
                  vp1, vp2, vp3, vp4);
         if( hideticklabels ) {
            if( irow == n_rows && icol == 1 ) {  // lower left frame -> no change
               line += "}";
            } else if( irow == n_rows && icol > 1 ) { // bottom frame, skip vert tick labels
               grow,line,"    ticks= {";
               grow,line,"    vert= {";
               grow,line,"    flags= 0x0b }}}";
            } else if( irow < n_rows && icol == 1 ) { // left frame, skip horiz tick labels
               grow,line,"    ticks= {";
               grow,line,"    horiz= {";
               grow,line,"    flags= 0x0b }}}";
            } else {                             // 'inner' frame, skip both tick labels
               grow,line,"    ticks= {";
               grow,line,"    horiz= {";
               grow,line,"    flags= 0x0b }";
               grow,line,"    vert= {";
               grow,line,"    flags= 0x0b }}}";
            }
         } else line += "}"; // Finish up line if all tick labels are wanted
         grow, gs1, line;
      }
   }

   grow, gs1, gs2;

   //+ prstrarr, gs1;

   write_slist,GISTDIR+"mplot.gs", gs1;

   window,pane,style=GISTDIR+"mplot.gs";

}

/* Function curtext */

struct s_Curtext {
   double xpos;
   double ypos;
   string text_string;
   double align;
   double charsize;
      int colortype; // 0 for string, 1 for vector
      int vcolor(3); // red, green, blue
   string scolor;
   string font;
   string cosys;
}

func curtext( text_string, align=, charsize=, color=, font=, replay=, cosys= )
/* DOCUMENT curtext, text_string, align=, charsize=, color=, font=, replay=, cosys=

   Asks for a cursor input where to put the text in the
   textstring.

   Keywords as for 'xyouts'

   Previous input saved in external 'Curtext_arr' which is an array of
   the struct 's_Curtext' with elements: xpos, ypos, text_string, align, charsize,
   colortype, scolor, vcolor, font.

   Keywords
       align     as for 'xyouts'
    charsize     as for 'xyouts'
       color     as for 'xyouts'
        font     as for 'xyouts'
      replay     repeat one of previous input strings, 0 for all of them
       cosys     defines the coordinate system: "vir" for virtual, "wor" for world,
                 and "ndc" for normalized device coordinates. Default is "wor".

   SEE ALSO: curmark1

   2010-06-14/NJW
*/
{
   extern Curtext_arr;

   if( !is_void(replay) ) {
      n = numberof(Curtext_arr);
      if( replay > n ) error,"Plot text index out of range";
      if( replay == 0 ) {
         rep1 = 1;
         rep2 = n;
      } else { rep1 = rep2 = replay; }

      for( repl = rep1; repl <= rep2; repl++ ) {
         xpos = Curtext_arr(repl).xpos;
         ypos = Curtext_arr(repl).ypos;
         align = Curtext_arr(repl).align;
         charsize = Curtext_arr(repl).charsize;
         colortype = Curtext_arr(repl).colortype; // 0 for string, 1 for vector
         scolor = Curtext_arr(repl).scolor;
         vcolor = Curtext_arr(repl).vcolor;
         font = Curtext_arr(repl).font;
         cosys = Curtext_arr(repl).cosys;
         align = align == -99.0 ? [] : align;
         charsize = charsize == -99.0 ? [] : charsize;
         font = font == "none" ? [] : font;
         vir = ndc = [];
         if( cosys == "vir" ) {
            vir = 1;
         } else if( cosys == "ndc" ) {
            ndc = 1;
         }
         color = colortype ? vcolor : scolor;
         xyouts,xpos,ypos,Curtext_arr(repl).text_string,align=align, \
              charsize=charsize,color=color,font=font,ndc=ndc, vir=vir;
      }
   } else {
      pos = curmark1(style=0,prompt="Mark text position ...",nomark=1);
      s = s_Curtext();
      xyouts,pos(1),pos(2),text_string,align=align,charsize=charsize,color=color,font=font;
      s.xpos = pos(1); s.ypos = pos(2); s.text_string = text_string;
      align = is_void(align) ? -99.0 : align;
      s.align = align;
      charsize = is_void(charsize) ? -99.0 : charsize;
      s.charsize = charsize;
      color = is_void(color) ? "black" : color;
      if( typeof(color) == "string" ) {
         s.colortype = 0;
         s.scolor = color;
      } else { // assume vector
         s.colortype = 1;
         s.vcolor = color;
      }
      font = is_void(font) ? "none" : font;
      s.font = font;
      cosys = is_void(cosys) ? "wor" : cosys;
      s.cosys = cosys;
      // Change coordinates is cosys is'nt 'wor'
      local q1, q2;
      if( cosys == "vir" ) {
         mcoord_conv, pos(1), pos(2), q1, q2, from="wor", to="vir";
         s.xpos = q1; s.ypos = q2;
      } else if( cosys == "ndc" ) {
         mcoord_conv, pos(1), pos(2), q1, q2, from="wor", to="ndc";
         s.xpos = q1; s.ypos = q2;
      }
      grow, Curtext_arr, s;
   }
}

/* Function wplot */

func wplot( lon, lat, pane=, ps=, symsize=, color= )
/* DOCUMENT wplot, lon, lat, ps=, symsize=, color=

   "World plot"

   Plot in Aitoff projection with longitude (lon) and
   latitude (lat) in degrees.

   Keywords: As for 'plot'

   2010-07-28/NJW, Copied from 'pointing_plot'
*/

{

   window,pane,style="nobox.gs";

   /*  Plot the grid */

   plot,[0],[0],xr=[180,-180],yr=1.4*[-90,90];

   /* longitude grid */

   b = span(-90,90,100);
   for( longi = -179.99; longi < 180.1; longi += 89.99 ) {
      l = array(0,100) + longi;
      xy = aitoff(l,b);
      listy = 2;
      if( longi < -179 || longi > 179 ) listy = 1;
      oplot,xy(1,),xy(2,),li=listy;
   }

   /* latitude grid */

   l = span(-179.999,180,100);
   for( lati = -60.; lati < 60.01; lati += 30. ) {
      b = array(0,100) + lati;
      xy = aitoff(l,b);
      oplot,xy(1,),xy(2,),li=2;
   }

   xy = aitoff(lon,lat);

   oplot, xy(1,), xy(2,), ps=ps, symsize=symsize, color=color;

}

/* Function owplot */

func owplot( lon, lat, pane=, ps=, symsize=, color= )
/* DOCUMENT owplot, lon, lat, ps=, symsize=, color=

   Overplot "World plot"

   Plot in Aitoff projection with longitude (lon) and
   latitude (lat) in degrees.

   Keywords: As for 'plot'

   2010-07-28/NJW, Copied from 'pointing_plot'
*/

{

   xy = aitoff(lon,lat);

   oplot, xy(1,), xy(2,), ps=ps, symsize=symsize, color=color;

}

/* Function poly_fill */

func poly_fill( x, y, cval )
/* DOCUMENT poly_fill, x, y, cval
    Fills the polygon defined by x and y with the color 'cval' from
    the current palette.

    SEE ALSO: poly_fillc
*/
{
   plfp, [1.], y, x, [numberof(y)], top=cval;
}

/* Function poly_fillc */

func poly_fillc( x, y, color= )
/* DOCUMENT poly_fillc, x, y, color=
    Fills the polygon defined by x and y with the color defined
    either as "red", "green", ..., or as a three element array.

    SEE ALSO: poly_fill
*/
{
   if( typeof(color) == "string" ) {
      if( color == "red" ) {
         color_arr = [255,0,0];
      } else if( color == "green" ) {
         color_arr = [0,255,0];
      } else if( color == "blue" ) {
         color_arr = [0,0,255];
      } else if( color == "cyan" ) {
         color_arr = [0,255,255];
      } else if( color == "magenta" ) {
         color_arr = [255,0,255];
      } else if( color == "yellow" ) {
         color_arr = [255,255,0];
      } else if( color == "black" ) {
         color_arr = [0,0,0];
      } else if( color == "white" ) {
         color_arr = [255,255,255];
      } else error,"Illegal color name";
   } else if( numberof(color) == 3 ) {
      color_arr = long(color);
   }
   z = char(color_arr)(,-);
   plfp, z, y, x, [numberof(y)];
}

/* Function plot_p2spec */

func plot_p2spec( spec_dol, spec_num, frac=, itype=, xr=, yr=, color=, title=, ytitle= )
/* DOCUMENT plot_p2spec, spec_dol, spec_num, frac=, itype=, xr=, yr=, color=, title=, ytitle=

  Plot one or more PHAII spectra as cnt/keV/s.
  If 'spec_num' is not given then the first spectrum is plotted.
  If 'spec_num' is zero than all spectra are plotted.
  'spec_num' may be an array with spectrum numbers.

  Several spectra can be given dimensions are: (nchan, nspectra)

  Each spectrum will be rebinned so that rate_err/rate < frac (defaults to 0.2)

  Keywords:   itype      0 : lin-lin
                         1 : log-lin
                         2 : lin-log
                         3 : log-log (default)

     The remaining keywords are as for 'plot'.

   SEE ALSO:  oplot_p2spec

   2010-11-23/NJW
*/
{
   local filename, extno;
   local ob1, ob2, orate, orate_err;

   if( is_void(frac) ) frac = 0.2;
   if( is_void(itype) ) itype = 3;

   get_exten_no, spec_dol, filename, extno;

   if( !file_test(filename) ) {
      write,"Did not find "+filename;
      return;
   }

   if( is_void(spec_num) ) spec_num = 1;

   hdr = headfits( spec_dol );
   nrows = fxpar( hdr, "naxis2" );
   if( spec_num == 0 ) spec_num = indgen(nrows); // for all spectra
   if( anyof(spec_num > nrows) ) {
      write,"You have requested a non-existing spectrum number";
      return;
   }
   nspec = numberof(spec_num);

   rate = rdfitscol( spec_dol, "rate" );
   rate_err = rdfitscol( spec_dol, "stat_err" );
   respfile = fxpar( hdr, "respfile" );

   // select the requested spectra

   rate = rate(,spec_num);
   rate_err = rate_err(,spec_num);

   if( !file_test(respfile) ) {
      write,"Did not find response file: "+respfile;
      return;
   }

   e_min = rdfitscol( respfile+"[EBOUNDS]", "e_min" );
   e_max = rdfitscol( respfile+"[EBOUNDS]", "e_max" );

   for( i = 1; i <= nspec; i++ ) {
      specrebinninga, e_min, e_max, rate(,i), rate_err(,i), frac, ob1, ob2, orate, orate_err;
      if( i == 1 ) {
         xmax = ob2(0);
         xmin = ob1(1);
         ymax = max( (orate+orate_err)/(ob2-ob1) );
         ymin = min( (orate-orate_err)/(ob2-ob1));
      } else {
         xmax = max( xmax, ob2(0) );
         xmin = min( xmin, ob1(1) );
         ymax = max( ymax, max((orate+orate_err)/(ob2-ob1)) );
         ymin = min( ymin, min((orate-orate_err)/(ob2-ob1)) );
      }
   }

   if( is_void(xr) ) {
      xr = [xmin/exp(0.05*log(xmax/xmin)),xmax*exp(0.05*log(xmax/xmin))];
   }
   if( is_void(yr) ) {
      yr = [ymin/exp(0.05*log(ymax/ymin)),ymax*exp(0.05*log(ymax/ymin))];
   }

   xtitle = "Energy [keV]";
   if( is_void(ytitle) ) ytitle = "Counts [/keV/s]";

   for( i = 1; i <= nspec; i++ ) {
      specrebinninga, e_min, e_max, rate(,i), rate_err(,i), frac, ob1, ob2, orate, orate_err;
      if( i == 1 ) {
         dataplotf, ob1, sqrt(ob1*ob2), ob2, (orate - orate_err)/(ob2-ob1), \
              orate/(ob2-ob1), (orate + orate_err)/(ob2-ob1), itype=itype, \
              xr=xr, yr=yr, title=title, xtitle=xtitle, ytitle=ytitle, color=color;
      } else {
         odataplotf, ob1, sqrt(ob1*ob2), ob2, (orate - orate_err)/(ob2-ob1), \
              orate/(ob2-ob1), (orate + orate_err)/(ob2-ob1), color=color;
      }
   }
}

/* Function oplot_p2spec */

func oplot_p2spec( spec_dol, spec_num, frac=, color= )
/* DOCUMENT oplot_p2spec, spec_dol, spec_num, frac=, color=

  Plot one or more PHAII spectra as cnt/keV/s.
  If 'spec_num' is not given then the first spectrum is plotted.
  If 'spec_num' is zero than all spectra are plotted.
  'spec_num' may be an array with spectrum numbers.

  Several spectra can be given, dimensions are: (nchan, nspectra)

  Each spectrum will be rebinned so that rate_err/rate < frac (defaults to 0.2)

     The remaining keywords are as for 'plot'.

   SEE ALSO:  plot_p2spec

   2010-11-23/NJW
*/
{
   local filename, extno;
   local ob1, ob2, orate, orate_err;

   if( is_void(frac) ) frac = 0.2;

   get_exten_no, spec_dol, filename, extno;

   if( !file_test(filename) ) {
      write,"Did not find "+filename;
      return;
   }

   if( is_void(spec_num) ) spec_num = 1;

   hdr = headfits( spec_dol );
   nrows = fxpar( hdr, "naxis2" );
   if( spec_num == 0 ) spec_num = indgen(nrows); // for all spectra
   if( anyof(spec_num > nrows) ) {
      write,"You have requested a non-existing spectrum number";
      return;
   }
   nspec = numberof(spec_num);

   rate = rdfitscol( spec_dol, "rate" );
   rate_err = rdfitscol( spec_dol, "stat_err" );
   respfile = fxpar( hdr, "respfile" );

   // select the requested spectra

   rate = rate(,spec_num);
   rate_err = rate_err(,spec_num);

   if( !file_test(respfile) ) {
      write,"Did not find response file: "+respfile;
      return;
   }

   e_min = rdfitscol( respfile+"[EBOUNDS]", "e_min" );
   e_max = rdfitscol( respfile+"[EBOUNDS]", "e_max" );

   for( i = 1; i <= nspec; i++ ) {
      specrebinninga, e_min, e_max, rate(,i), rate_err(,i), frac, ob1, ob2, orate, orate_err;
      odataplotf, ob1, sqrt(ob1*ob2), ob2, (orate - orate_err)/(ob2-ob1), \
              orate/(ob2-ob1), (orate + orate_err)/(ob2-ob1), color=color;
   }
}

/* Function wr_qdp */

func wr_qdp( name, x, y, dy, dy2, dx=, ps=, li=, symsize= )
/* DOCUMENT wr_qdp, name, x, y[, dy, [dy2]], dx=, ps=, li=, symsize=

   writes a text file by name of <name>.qdp for interpretation by QDP

   2011-05-03/NJW
*/
{
   nx = numberof(x);
   ymode = "none";
   xmode = "none";
   if( numberof(dy) ) {
      if( numberof(dy) != nx ) error,"Non compliant number of elements in dy";
      ymode = "serr"; // symmetrical errors
   }
   if( numberof(dy2) ) {
      if( numberof(dy2) != nx ) error,"Non compliant number of elements in dy2";
      ymode = "terr"; // two-sided errors
   }
   if( numberof(y) != nx ) error,"Non compliant number of elements in y";
   if( numberof(dx) ) {
      if( numberof(dx) != nx ) error,"Non compliant number of elements in dx";
      xmode = "serr"; // symmetrical errors in x
   }

   if( typeof(name) != "string" ) error,"First argument must be a string";

   outfile = name+".qdp";
   f = open( outfile, "w" );
   write,f,format="cpd /%s\n", "xw"; // Change plot device command
   // plot symbols - markers in PLT parlance
   if( !is_void(ps) ) {
      if( ps < 0 ) {
         write,f,format="lines %s\n","on";
         ps = -ps;
      }
      if( ps == 10 ) {
         write,f,format="lines %s\n","step";
      } else write,f,format="marker %i on\n",ps;
   }
   if( xmode != "none" ) write,f,format="read %s 1\n", xmode; 
   if( ymode != "none" ) write,f,format="read %s 2\n", ymode; 

   for( i = 1; i <= nx; i++ ) {
      write,f,format="%15.6e", double(x(i));
      if( numberof(dx) ) write,f,format=" %15.6e", double(dx(i));
      write,f,format=" %15.6e", double(y(i));
      if( numberof(dy) ) write,f,format=" %15.6e", double(dy(i));
      if( numberof(dy2) ) write,f,format=" %15.6e", double(dy2(i));
      write,f,"";
   }

   close, f;

   write,outfile," has been written ...";

}

/* Function devicetitles */

func devicetitles( title, titpos, xtitle, xtitpos, ytitle, ytitpos, \
                   height=, font=, align=, color= )
/* DOCUMENT devicetitles, title, titpos, xtitle, xtitpos, ytitle, ytitpos, \
                   height=, font=, align=, color=

    Default titpos  : [0.4,0.9]
            xtitpos : [0.4,0.05]
            ytitpos : [0.05,0.5]

    Keywords: height   Face value for title, 2 smaller for xtitle and ytitle
              font, align, color
*/
{
   if( is_void(height) ) height = 16;
   if( is_void(font) ) font = "schoolbook";
   if( is_void(color) ) color = "black";
   if( is_void(align) ) {
      justify = "CA";
   } else {
     if( align < 0.25 ) {
        justify = "LA";
     } else if( align < 0.75 ) {
        justify = "CA";
     } else justify = "RA";
   }
   xheight = height - 2;
   
   if( !is_void(title) ) {
      if( structof(title) != string ) error,"'title' must be a string";
      if( is_void(titpos) ) {
         titpos = [0.4,0.9];
      } else {
         if( numberof(titpos) != 2 ) error,"'titpos' must have 2 elements";
      }
      plt,title,titpos(1),titpos(2),orient=0,height=height,font=font,justify=justify;
   }
   if( !is_void(xtitle) ) {
      if( structof(xtitle) != string ) error,"'xtitle' must be a string";
      if( is_void(xtitpos) ) {
         xtitpos = [0.4,0.05];
      } else {
         if( numberof(xtitpos) != 2 ) error,"'xtitpos' must have 2 elements";
      }
      plt,xtitle,xtitpos(1),xtitpos(2),orient=0,height=xheight,font=font,justify=justify;
   }
   if( !is_void(ytitle) ) {
      if( structof(ytitle) != string ) error,"'ytitle' must be a string";
      if( is_void(ytitpos) ) {
         ytitpos = [0.05,0.5];
      } else {
         if( numberof(ytitpos) != 2 ) error,"'ytitpos' must have 2 elements";
      }
      plt,ytitle,ytitpos(1),ytitpos(2),orient=1,height=xheight,font=font,justify=justify;
   }
}

/* Function xyouts */

func xyouts( x, y, text, align=, valign=, charsize=, ndc=, vir=, color=, font= )
/* DOCUMENT xyouts, x, y, text, align=, valign=, charsize=, ndc=, vir=, color=, font=

   Write text in plot.

   Arguments:
       x, y     Text position in indicated coordinate system (defaults to 'world')
       text     The text string

   Keywords:
       align    Horizontal alignment, can be 0, 0.5, or 1.0 for left, center, or right adjusted.
      valign    Vertical alignment, can be 0, 0.5, or 1.0 for bottom, center, or top adjusted.
         ndc    (bool) To be set when (x,y) is in 'ndc', Normalized Device Coordinates.
         vir    (bool) To be set when (x,y) is in virtual coordinates (range 0 - 1)
       color    as in 'plot'
    charsize    Sets 'height' (see yorick 'plt' function) to 14*charsize
        font    can be "times", "schoolbook" (see Yorick manual)

   Original version: before 2008
   2015-05-28/NJW updated with keyword 'valign'
*/

{
   local xnew, ynew, xaux, yaux;

   csz = 1.;
   tosys = numberof(ndc) ? [] : 1;
   if( numberof(charsize) ) csz = charsize;
   // Keyword 'vir' will override 'ndc'
   if( vir ) {
      mcoord_conv, x, y, xnew, ynew, from="vir", to="wor";
      x = xnew;
      y = ynew;
      tosys = 1;
   }
   if( valign ) {
      //+ write,"valign was seen as "+ftoa(valign,ndec=2);
      if( tosys ) {  // (x,y) is in 'wor'
         //+ write, "case 1";
         mcoord_conv, x, y, xnew, ynew, from="wor", to="ndc";
         mcoord_conv, xnew, ynew - csz * 14 * 0.0013 * valign, xaux, yaux, from="ndc", to="wor";
         y = yaux;
         //+ write,"y is now: "+ftoa(double(y),ndec=4);
      } else { // (x,y) is in 'ndc'
         //+ write, "case 2";
         y -= (csz * 14 * 0.0013 * valign);
      }
   }
   if( numberof(align) ) {
      if( align < 0.25 ) {
         plt, text, x, y, tosys=tosys, justify="LA",height=14.*csz, \
                  color=color, font=font;
      } else if( align >= 0.25 && align < 0.75 ) {
         plt, text, x, y, tosys=tosys, justify="CA",height=14.*csz, \
                  color=color, font=font;
      } else {
         plt, text, x, y, tosys=tosys, justify="RA",height=14.*csz, \
                  color=color, font=font;
      }
   } else {
      plt, text, x, y, tosys=tosys,height=14.*csz, \
                  color=color, font=font;
   }

}

/* Function curve_edit */

func curve_edit( x, y, &newx, &newy, aux=, step=, itype=, xr=, yr= )
/* DOCUMENT curve_edit, x, y, >newx, >newy, aux=, step=, itype=, xr=, yr=

     When points are entered with the cursor, no change will happen
     outside min x and max x. Between these the returned curve will be
     linearly interpolated to match the new y-values.

     The pointed-to x values will stick to the nearest existing x value.

     Always returns arrays of type double.

     The 'aux' keyword gives the option of showing an addition curve
     e.g. as a guide. Must have same dimension as x and y.
*/
{
   if( is_void(step) ) step = 1;
   if( is_void(itype) ) itype = 0;

   plot,x,y,xr=xr,yr=yr,itype=itype;
   oplot,x,y,ps=2;
   if( numberof(aux) ) oplot,x,aux,li=2;

   newx = double(x);
   newy = double(y);

   pts = curmark();
   npts = numberof(pts);
   xpts = pts(1:npts:2);
   ypts = pts(2:npts:2);
   npts /= 2;

   is = sort(xpts);
   xpts = xpts(is);
   ypts = ypts(is);

   for( i = 1; i <= npts; i++ ) xpts(i) = newx(abs(xpts(i)-newx)(mnx));
   // avoid duplicate xpts
   newxpts = xpts(1);
   for( i = 2; i <= npts; i++ ) {
      if( xpts(i-1) == xpts(i) ) { // average y value
         if( itype%2 == 1 ) {
            ypts(i-1) = sqrt(ypts(i-1)*ypts(i));
         } else {
            ypts(i-1) = 0.5*(ypts(i-1) + ypts(i));
         }
      } else {
         grow, newxpts, xpts(i);
      }
   }
   npts = numberof(newxpts);
   xpts = newxpts;
   ypts = ypts(1:npts);

   oplot,xpts,ypts,ps=3,color="blue";
   i1 = where( xpts(1) == newx )(1);
   i2 = where( xpts(0) == newx )(1);
   ir = indgen(i1:i2);
   newy(ir) = interp(ypts,xpts,newx(ir));

   oplot, newx, newy, color="red";

}

/* Function add_vgraph */

func add_vgraph( im, ampl=, ia=, ixr=, wxr=, color=, zero=, showxr=, cshowxr= )
/* DOCUMENT add_vgraph, im, keywords ...

    Adds a vertical histogram of image values summed
    in the specified range.

    Keywords:
       ampl       amplitude (in world coordinate) of histogram
       ia         flag for interactive setting of 'ixr'
       ixr        range in x, a scalar of a 2-element array in pixels
       wxr        alternative to 'ixr' but expressed in world coordinates
       color      color of histogram
       zero       flag to force histogram to include zero
       showxr     flag for adding the bounding lines for 'ixr'
       cshowxr    the color of these bounding lines

    Typically this function is used following a 'disp,im' command.

   2012-05-03/NJW, 2013-11-05/NJW updated with showxr, cshowxr
*/
{
   // Check for conflict
   if( !ia && (numberof(ixr) && numberof(wxr)) ) error,"xr conflict!";

   dms = dimsof(im);

   // get the current limits for the plot

   lims = limits(); // [xmin,xmax,ymin,ymax]

   //  x(world) * a + b = ix(image)
   a = b = array(double,2);
   a(1) = (dms(2) - 1) / (lims(2) - lims(1));
   b(1) = 1 - lims(1)*a(1);
   a(2) = (dms(3) - 1) / (lims(4) - lims(3));
   b(2) = 1 - lims(3)*a(2);

   if( is_void(ampl) ) ampl = (lims(1) + lims(2))/2;

   // see if 'wxr' has been set and - if so - derive 'ixr'
   if( numberof(wxr) ) {
      wxr = double(wxr);
      if( numberof(wxr) == 1 ) wxr = [wxr,wxr];
      ixr = long(wxr*a(1) + b(1) + 0.5);
   }
   // 'ixr' may also be set directly
   if( is_void(ixr) ) ixr = [1,dms(2)];
   ixr = long(ixr + 0.5);
   if( numberof(ixr) == 1 ) ixr = [ixr,ixr];

   if( ia ) {
      s = curmark1();
      ixr1 = long(s(1)*a(1)+b(1)+0.5);
      s = curmark1();
      ixr2 = long(s(1)*a(1)+b(1)+0.5);
      if( ixr1 > ixr2 ) { ix = ixr1; ixr1 = ixr2; ixr2 = ix; }
      ixr = [ixr1, ixr2];
   }

   //  Add bounding lines if required

   if( showxr ) {
      oplot,ixr(1)*[1,1],[1,dms(3)],color=cshowxr;
      oplot,ixr(2)*[1,1],[1,dms(3)],color=cshowxr;
   }

   x = (indgen(dms(3))-b(2))/a(2);
   q = im(ixr(1):ixr(2),);
   y = q(sum,);
   local plin, qlin;
   miny = zero ? 0.0 : min(y);
   linscale, miny, max(y), lims(1), ampl, plin, qlin;
   plhisv,y*plin+qlin,x,color=color,marks=0;
}

/* Function add_hgraph */

func add_hgraph( im, ampl=, ia=, iyr=, wyr=, color=, zero=, showyr=, cshowyr= )
/* DOCUMENT add_hgraph, im, keywords ...

    Adds a horizontal histogram of image values summed
    in the specified range.

    Keywords:
       ampl       amplitude (in world coordinate) of histogram
       ia         flag for interactive setting of 'iyr'
       iyr        range in y, a scalar of a 2-element array in pixels
       wyr        alternative to 'iyr' but expressed in world coordinates
       color      color of histogram
       zero       flag to force histogram to include zero
       showyr     flag for adding the bounding lines for 'iyr'
       cshowyr    the color of these bounding lines

    Typically this function is used following a 'disp,im' command.

   2012-05-03/NJW, 2013-11-05/NJW, updated with showyr, cshowyr
*/
{
   // Check for conflict
   if( !ia && (numberof(iyr) && numberof(wyr)) ) error,"yr conflict!";

   dms = dimsof(im);

   // get the current limits for the plot

   lims = limits(); // [xmin,xmax,ymin,ymax]

   //  y(world) * a + b = iy(image)
   a = b = array(double,2);
   a(1) = (dms(2) - 1) / (lims(2) - lims(1));
   b(1) = 1 - lims(1)*a(1);
   a(2) = (dms(3) - 1) / (lims(4) - lims(3));
   b(2) = 1 - lims(3)*a(2);

   if( is_void(ampl) ) ampl = (lims(3) + lims(4))/2;

   // At this point there are 3 possibilities
   //   1) wyr has been defined (but not iyr because of test above)
   //   2) iyr has been defined (but not wyr because of test above)
   //   3) neither wyr nor iyr has been set - default to entire image

   //   ------ deal with case 3 first ----------
   if( is_void(iyr) ) iyr = [1,dms(3)];
   if( numberof(iyr) == 1 ) iyr = [iyr,iyr];
   iyr = long(iyr + 0.5); // make sure that iyr is of type 'long'

   // see if 'wyr' has been set and - if so - derive 'iyr'
   //  and override the previous setting of iyr
   if( numberof(wyr) ) {
      wyr = double(wyr);
      if( numberof(wyr) == 1 ) wyr = [wyr,wyr];
      iyr = long(wyr*a(2) + b(2) + 0.5);
   } else { // derive wyr for later plotting
      wyr = (iyr - b(2)) / a(2);
   }

   if( ia ) {
      s = curmark1();
      iyr1 = long(s(2)*a(2)+b(2)+0.5);
      s = curmark1();
      iyr2 = long(s(2)*a(2)+b(2)+0.5);
      if( iyr1 > iyr2 ) { iy = iyr1; iyr1 = iyr2; iyr2 = iy; }
      iyr = [iyr1, iyr2];
   }

   //  Add the bounding lines if required

   if( showyr ) {
      xyl = limits()(1:4);
      //+ oplot,[1,dms(2)],iyr(1)*[1,1],color=cshowyr;
      oplot,xyl(1:2),wyr(1)*[1,1],color=cshowyr;
      oplot,xyl(1:2),wyr(2)*[1,1],color=cshowyr;
   }

   x = (indgen(dms(2))-b(1))/a(1);
   q = im(,iyr(1):iyr(2));
   y = q(,sum);
   local plin, qlin;
   miny = zero ? 0. : min(y);
   linscale, miny, max(y), lims(3), ampl, plin, qlin;
   plhis,y*plin+qlin,x,color=color,marks=0;
}

/* Function resolve_color */

func resolve_color( strcolor )
/* DOCUMENT int_color = resolve_color( strcolor )

    Returns 3-element long array for the color representation
    given by a string value.

*/
{
   if( typeof(strcolor) != "string" ) return strcolor; // NoOp if not a string

   n = numberof(strcolor);
   res = array(long,3,n);
   for( i = 1; i <= n; i++ ) {
      res(,i) = [0,0,0]; // black is default
      if( strcolor(i) == "white" ) res(,i) = [255,255,255];
      if( strcolor(i) == "red" ) res(,i) = [255,0,0];
      if( strcolor(i) == "green" ) res(,i) = [0,255,0];
      if( strcolor(i) == "blue" ) res(,i) = [0,0,255];
      if( strcolor(i) == "magenta" ) res(,i) = [255,0,255];
      if( strcolor(i) == "cyan" ) res(,i) = [0,255,255];
      if( strcolor(i) == "yellow" ) res(,i) = [255,255,0];
      if( strcolor(i) == "black" ) res(,i) = [0,0,0];
   }
   return res;
}

/* Function d3plot */

#include "plwf.i"

func d3plot( image, pane=, theta=, phi=, cage=, pal=, shade=, edges= )
/* DOCUMENT d3plot, image, pane=, theta=, phi=, cage=, pal=, shade=, edges=

    keywords:

       theta     Tilt in radians (default 0.3)
       phi       Rotation in radians (default 0.6)
       cage      cage or no cage (Boolean)
       pal       Name of palette, omitting .gp
       shade     shade or no shade (Boolean)
       edges     edges or no edges (Boolean)

    2013-01-08/NJW
*/
{
   if( is_void(pane) ) pane = 0;
   window,pane,style="nobox.gs";
   if( is_void(theta) ) theta = 0.3;
   if( is_void(phi) ) phi = 0.6;
   if( is_void(cage) ) cage = 0;
   if( is_void(pal) ) {
      palette,"gray.gp";
   } else {
      palette,pal+".gp";
   }
   orient3,phi,theta;
   cage3,cage;
   plwf,image, shade=shade, edges=edges;
}

/* Function draw_arrow */

func draw_arrow( x0, y0, x1, y1, sw, hw, hl, color=, thick=, tri=, fill= )
/* DOCUMENT draw_arrow,  x0, y0, x1, y1, sw, hw, hl, color=, thick=, tri=, fill=

   Draws an arrow in the current plot window

   Arguments:
            (x0,y0) start point in data coordinates
            (x1,y1) end   point in data coordinates
            sw   width of stem in normalized device coordinates
                    not used unless keyword 'tri' has been set
            hw   width of arrow head in normalized device coordinates
                    defaults to 0.01
            hl   length of arrow head in normalized device coordinates
                    defaults to 0.015
   Keywords:
        color   standard
        thick   standard
        tri     triangular arrowhead
        fill    for filled arrow; takes only effect if 'tri' has been set

   2002-08-30/NJW, translated to Yorick 2009-11-19
   2012-07-03/NJW, 2015-06-16/NJW updated
*/
{

   if( tri && is_void(sw) ) sw = 0.005;
   if( is_void(hw) ) hw = 0.01;
   if( is_void(hl) ) hl = 0.015;

   // find the aspect ratio:  ysize / xsize

   port = viewport();
   aspect = (port(4)-port(3))/(port(2)-port(1));

   // get start/end coordinates in normalized device coordinates

   mcoord_conv, x0, y0, x0ndc, y0ndc, from="wor", to="ndc";
   mcoord_conv, x1, y1, x1ndc, y1ndc, from="wor", to="ndc";
   //+ y0ndc = y0ndc * aspect
   //+ y1ndc = y1ndc * aspect

   length = sqrt((x1ndc-x0ndc)^2 + (y1ndc-y0ndc)^2);

   om = atan(x1ndc-x0ndc, y1ndc-y0ndc);
   omega = [[cos(om),-sin(om)],[sin(om),cos(om)]];

   if( tri ) {
      x_arrow_ndc = [-sw/2,sw/2,sw/2,hw/2,0,-hw/2,-sw/2,-sw/2];
      y_arrow_ndc = [0,0,length-hl,length-hl,length,length-hl,length-hl,0];
   } else {
      x_arrow_ndc = [0,0,hw/2,0,-hw/2];
      y_arrow_ndc = [0,length,length-hl,length,length-hl];
   }

   arrow_ndc = array(double,2,numberof(x_arrow_ndc));
   arrow_ndc(1,) = x_arrow_ndc;
   arrow_ndc(2,) = y_arrow_ndc;

   arrow_tilt = omega(,+)*arrow_ndc(+,);
   arrow_tilt(1,) = arrow_tilt(1,) + x0ndc;
   arrow_tilt(2,) = arrow_tilt(2,) + y0ndc;

   mcoord_conv, arrow_tilt(1,),arrow_tilt(2,),arrow_worx, arrow_wory, from="ndc",to="wor";

   oplot,arrow_worx,arrow_wory, color=color, thick=thick;
   if( fill && tri ) poly_fillc, arrow_worx,arrow_wory, color=color;

}
